/*************************************************************************/
/*                                                                       */
/*   COMPILE.C - The routines in this file deal with the compilation,    */
/*               linking and running of source-files from within EMAX.   */
/*               This compiles into nothing if not on a PRIMOS system.   */
/*                                                                       */
/*   Author: Peter Eriksson, V{sterh|jdskolan Sk|vde Sweden (April 87)   */
/*                                                                       */
/*************************************************************************/

#include        <stdio.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"


#if PR1MOS
#include        <errd>

char compiler_name[200]   = "";
char compiler_option[200] = "";
extern int system_error1;
extern int system_error2;

#define COMCC      100
#define COMCI      101
#define COMPASCAL  200
#define COMPASCALS 201
#define COMFTN     300
#define COMF77     301
#define COMPL1G    400
#define COMPLP     401
#define COMSPL     402
#define COMCBL     500
#define COMCOBOL   501
#define COMPMA     600
#define COMRPG     700
#define COMVRPG    701

char *comsuffixes[] =
{
   "c",
   "cc",
   "pas",
   "pascal",
   "spl",
   "plp",
   "ftn",
   "f77",
   NULL
};

struct
{
   char *name;
   char *option;
   char *bind;
   int  flag;
} comdata[] =
   {
      {
         "ci",        
         "-include c>source",                
         "-li ccmain -lo %s -li c_lib -li -dynt -all",
         COMCI
      },
      {
         "cc",        
         "-include c>source -newfortran",    
         "-li ccmain -lo %s -li c_lib -li -dynt -all",
         COMCC
      },
      {
         "pascals",   
         "-listing no",
         "-lo %s -li vpaslb -li -dynt -all",
         COMPASCALS
      },
      {
         "pascal",    
         "",                                 
         "-lo %s -li paslib -li -dynt -all",
         COMPASCAL
      },
      {
         "spl",       
         "",                                
         "-lo %s -li -dynt -all",
         COMSPL
      },
      {
         "plp",      
         "",                                 
         "-lo %s -li -dynt -all",
         COMPLP
      },
      {
         "ftn",       
         "",                                 
         "-lo %s -li -dynt -all",
         COMFTN
      },
      {
         "f77",       
         "",                                 
         "-lo %s -li -dynt -all",
         COMF77
      },
      {
         NULL,
         NULL,
         NULL
      }
   };


#define ERRORMAX 200
int   errorline[ERRORMAX];             /* List of lines where errors exist */
char *errortext[ERRORMAX];             /* List of error types              */
int   errorcntr = 0;                   /* Number of errors in errorline    */
int   errorcurr = 0;                   /* Current error displayed          */


compile(f, n)
{
   char  *malloc(), *strleft(), *strcat();
   char  filename[81];
   char  prompt[81];
   char  name[81];
   char  systext[200];
   char  tname[81];
   short error,cerror;
   FILE  *infile;
   int   loop;
   int   ctp = 0;
   int   cpval;

   /* Prompt user for filename */
   strcpy(filename,curbp->b_fname);
   strupper(filename);
   sprintf(prompt,"Compile file [%s]: ",filename);
   if (mlreply(prompt, name, 80) == ABORT)
     return(ABORT);

   /* Process filename */
   trimleft(name);
   if (name[0])
      strcpy(filename,name);
   strlower(filename);
   /* Locate last '.' in name */
   for (loop=strlen(filename); loop>=0 && filename[loop]!='.'; loop--);

   /* Generate compiler name */
   if (!compiler_name[0])
   {
      /* If not '.' or unknown suffix -> error */
      if (loop<0 || (ctp = strlocate(filename+loop+1,comsuffixes))==-1)
      {
         mlwrite("[Unknown source file type - Can't compile!]");
         return(FALSE);
      }
      strcpy(systext, comdata[ctp].name);
   }
   else
      strcpy(systext, compiler_name);

   /* Append filename */
   strcat(systext," ");
   if (loop>=0)
      filename[loop] = 0;
   strcat(systext,filename);

   /* Append option(s) */
   strcat(systext," ");
   if (!compiler_option[0] && !compiler_name[0])
      strcat(systext,comdata[ctp].option);
   else
      strcat(systext,compiler_option);

   /* Compile file */
   tmpnam(tname);
   strupper(filename);
   mlwrite("Compiling %s...",filename);
   strcpy(prompt,"COMO ");
   strcat(prompt,tname);
   (*term.t_flush)();
   (*term.t_close)();
   system(strcat(prompt," -NTTY"));
   cerror = system(systext);
   clearerror(FALSE,0);
   system("COMO -TTY -END");
   (*term.t_open)();

   if (system_error1 == e$ninf)
   {
      updgar();    /* Force an update of the screen */
      unlink(tname);
      mlwrite("[The compiler specified doesn't exist!]");
      return(FALSE);
   }

   system("COMO -TTY -END");
   if ((infile=fopen(tname,"r"))==NULL)
   {
      unlink(tname);
      if (!cerror)
      {
         mlwrite("[An error occured during compilation!]");
         return(FALSE);
      }
      mlwrite("[The file was successfully compiled]");
      return(TRUE);
   }

   switch (comdata[ctp].flag * (compiler_name[0]==0))
   {
      case COMCC:
      case COMCI:
      case COMPASCAL:
      case COMSPL:
      case COMF77:
         do
         {
            fgets(systext,199,infile);
            trimsides(systext);
            switch (comdata[ctp].flag)
            {
               case COMCC:
                  cpval = strcmp(strleft(systext,6),"Error#") == 0;
                  break;
               case COMCI:
                  cpval = strcmp(strleft(systext,7),"Error #") == 0;
                  break;
               case COMPASCAL:
               case COMSPL:
               case COMF77:
                  cpval = strcmp(strleft(systext,5),"ERROR") == 0;
                  break;
            }
            if (cpval)
            {
               if (errorcntr>ERRORMAX)
               {
                  mlwrite("[Too many errors - The error list has been truncated!]");
                  goto ERROREXIT;
               }
               if (!(errortext[errorcntr] = malloc(strlen(systext)+1)))
               {
                  mlwrite("[Unable to allocate memory for error text!]");
                  goto ERROREXIT;
               }
               strcpy(errortext[errorcntr],systext);
               switch(comdata[ctp].flag)
               {
                  case COMCC:
                     sscanf(systext,"%*s%*s%*s%*s%*s%*s%*s%*s%*s%d",&errorline[errorcntr++]);
                     break;
                  case COMCI:
                     sscanf(systext,"%*s%*s%*s%*s%*s%*s%d",&errorline[errorcntr++]);
                     break;
                  case COMPASCAL:
                  case COMSPL:
                     sscanf(systext,"%*s%*s%*s%*s%*s%*s%*s%d",&errorline[errorcntr++]);
                     break;
               }
            }
         }
         while (!isdigit(systext[0]) && !feof(infile));
         if (feof(infile))
            mlwrite("[An error occured during compilation!]");
         else
            mlwrite("[%s]",systext);
         ERROREXIT:
         fclose(infile);
         unlink(tname);                             /* Remove error file */
         if (!cerror)
            return(FALSE);
         else
            return(TRUE);
         break;
      default:
            if (!cerror)
               {
               mlwrite("[An error occured during compilation!]");
               unlink(tname);
               return(FALSE);
               }
            mlwrite("[The file was successfully compiled]");
            unlink(tname);
            return(TRUE);
      }
}


/* Display an error and move the cursor to the erroneous line.     */
/* Remove the last error from the error list                       */

displayerror(f, n)
{
   if (errorcurr>0)
      free(errortext[errorcurr-1]);

   if (errorcurr>=errorcntr)
      {
      mlwrite("[No more errors]");
      return(FALSE);
      }

   gotoline(TRUE,errorline[errorcurr]);
   mlwrite("[%s]",errortext[errorcurr++]);
   return(TRUE);
}




/* Clear the error list, don't bother if it was empty before calling it     */
/* or not, it will however be cleared after this routine has done it's work */

clearerror(f, n)
{
   int loop;

   for( loop=errorcurr; loop<errorcntr; loop++)
      free(errortext[loop]);
   errorcurr = errorcntr = 0;
   return(TRUE);
}


/*
 * Force usage of a specific compiler. If called with an argument, reset
 * the forced usage. It is also reset if an empty name is entered
 */

setcompiler(f, n)
{
   int flag;
   char reply[81];

   reply[0] = 0;
   if (!f)
   {
      if (mlreply("Compiler name: ", reply, 80)==ABORT)
         return(ABORT);
      trimsides(reply);
      if (reply[0])
      {
         strcpy(compiler_name,reply);
         mlerase();
         return(TRUE);
      }
   }

   compiler_name[0] = 0;
   mlwrite("[Autoselect compiler mode]");
   return(TRUE);
}

/*
 * This function forces the usage of a specific set of command line arguments.
 * The forced usage is resetted if called with an argument.
 */
setoption(f, n)
{
   int flag;
   char reply[81];

   reply[0] = 0;
   if (!f)
   {
      if (mlreply("Compiler option(s): ", reply, 80) == ABORT)
         return(ABORT);
      trimsides(reply);
      if (reply[0])
      {
         strcpy(compiler_option,reply);
         mlerase();
         return(TRUE);
      }
   }

   compiler_option[0] = 0;
   mlwrite("[Forced compiler options disabled]");
   return(TRUE);
}



/*
 * Bind a compiled file
 */

bindfile(f, n)
{
   char filename[200];
   char prompt[81];
   char name[81];
   int loop;
   int ctp;
   
   strcpy(filename,curbp->b_fname);
   for (loop=strlen(filename); loop>=0 && filename[loop]!='.'; loop--);
   if (loop>=0)
      filename[loop] = '\0';
   strupper(filename);

   sprintf(prompt,"Bind file [%s]: ",filename);
   if (mlreply(prompt, name, 80) == ABORT)
     return(ABORT);

   trimleft(name);
      
   /* If not '.' or unknown suffix -> error */
   if (loop<0 || (ctp = strlocate(filename+loop+1,comsuffixes))==-1)
   {
      mlwrite("[Unknown source file type - Can't compile!]");
      return(FALSE);
   }

   strcpy(prompt,"BIND ");

   switch (name[0])
   {
      case '-':
         strcat(prompt,filename);   /* Default name */
         strcat(prompt," ");
         strcat(prompt,name);       /* Add options  */
         break;
      case '\0':
         sprintf(prompt + strlen(prompt), comdata[ctp].bind, filename);
         break;
      default:
         strcat(prompt,name);       /* Name and options */
   }

   if (filename[0] || (name[0] && name[0]!='-'))
      if (!name[0] && name[0] == '-')
      {
         strupper(filename);
         mlwrite("[Binding %s...]\n", filename);
      }
      else
      {
         strupper(name);
         mlwrite("[Binding %s...]\n", name);
      }
   else
      mlwrite("Entering the BIND linker...");

   (*term.t_flush)();
   (*term.t_close)();
   system(prompt);
   (*term.t_open)();

   mlwrite("\nPress any key to return to MicroEMACS: ");
   (*term.t_getchar)();
   sgarbf = TRUE;
   return(TRUE);
}



/*
 * Run a compiled and linked file.
 */

runfile(f, n)
{
   int  loop;
   char filename[200];
   char prompt[200];
   char name[81];

   /* Prompt user for filename */
   strcpy(filename,curbp->b_fname);

   for (loop=strlen(filename); loop>=0 && filename[loop]!='.'; loop--);
   if (loop>=0)
      filename[loop] = '\0';
   strupper(filename);

   sprintf(prompt,"Run file [%s]: ",filename);
   if (mlreply(prompt, name, 80) == ABORT)
     return(ABORT);

   trimleft(name);

   if (!name[0] && !filename[0])
   {
      mlwrite("[No default filename for this buffer!]");
      return(FALSE);
   }

   strcpy(prompt,"RESUME ");

   switch (name[0])
   {
      case '-':
         strcat(prompt,filename);   /* Default name */
         strcat(prompt," ");
         strcat(prompt,name);       /* Add options  */
         break;
      case '\0':
         strcat(prompt,filename);   /* Default name only */
         break;
      default:
         strcat(prompt,name);       /* Name and options */
   }

   sscanf(prompt,"%*s%s",name);

   strupper(name);
   mlerase();
   mlwrite("[Executing %s...]\n",name);

   (*term.t_flush)();
   (*term.t_close)();
   system(prompt);
   (*term.t_open)();

   mlwrite("\nPress any key to return to MicroEMACS: ");
   (*term.t_getchar)();
   sgarbf = TRUE;
   return(TRUE);
}

#endif
