/* fileio.c
 * The routines in this file read and write ASCII files from the disk. All of
 * the knowledge about files are here. A better message writing scheme should
 * be used.
 */
#include        <stdio.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"

FILE    *ffp;                           /* File pointer, all functions. */

/*
 * Open a file for reading.
 */
ffropen(fn)
char    *fn;
{
        if ((ffp=fopen(fn, "r")) == NULL)
                return (FIOFNF);
        return (FIOSUC);
}

/*
 * Open a file for writing. Return TRUE if all is well, and FALSE on error
 * (cannot create).
 */
ffwopen(fn)
char    *fn;
{
#if     VMS
        register int    fd;

        if ((fd=creat(fn, 0666, "rfm=var", "rat=cr")) < 0
        || (ffp=fdopen(fd, "w")) == NULL) {
#else
        if ((ffp=fopen(fn, "w")) == NULL) {
#endif
                mlwrite("Cannot open file for writing");
                return (FIOERR);
        }
        return (FIOSUC);
}

/*
 * Open a file for writing at the end of it. Return TRUE if all is well,
 * and FALSE on error (cannot create).
 */
ffaopen(fn)
char    *fn;
{
#if VMS
        mlwrite("[File error: VAX/VMS - are you serious?????]");
        return (FIOERR); /* ZZZZorry, I don't know about VMS (VAX) */
#else
        if ((ffp=fopen(fn, "wa")) == NULL) {
                mlwrite("[Cannot open file for write-append]");
                return (FIOERR);
        }
        return (FIOSUC);
#endif
}


/*
 * Close a file. Should look at the status in all systems.
 */
ffclose()
{
/* Do not  add a ^Z at the end of the file */
/* #if MSDOS     fputc(26, ffp); #endif */

#if     V7 | USG | BSD | (MSDOS & (LATTICE | MSC)) | PR1MOS
        if (fclose(ffp) != FALSE) {
                mlwrite("Error closing file");
                return(FIOERR);
        }
        return(FIOSUC);
#else
        fclose(ffp);
        return (FIOSUC);
#endif
}
/*
 * Write a line to the already opened file. The "buf" points to the buffer,
 * and the "nbuf" is its length, less the free newline. Return the status.
 * Check only at the newline.
 */
ffputline(buf, nbuf)
char    buf[];
{
        register int    i, j, c, pos;

        pos = 1;
        if (transtab != 1) {
                for (i = 0; i < nbuf; ++i)
                        fputc(buf[i]&0xFF, ffp);
        }
        else {
                for (i = 0; i < nbuf; ++i) {
                        if ((c=buf[i]&0xFF)=='\t')
                                do {
                                        fputc(' ', ffp);
                                }  while (pos++%8);
                        else {
                                fputc(c, ffp);
                                pos++;
                        }
                }
        }

        fputc('\n', ffp);

        if (ferror(ffp)) {
                mlwrite("Write I/O error");
                return (FIOERR);
        }

        return (FIOSUC);
}

/*
 * Read a line from a file, and store the bytes in the supplied buffer. The
 * "nbuf" is the length of the buffer. Complain about long lines and lines
 * at the end of the file that don't have a newline present. Check for I/O
 * errors too. Return status.
 */
ffgetline(buf, nbuf)
register char   buf[];
{
        register int    c;
        register int    i;
        register int    pos, rempos, instab, val;

        i = 0;
        pos = 1;

        while ((c = fgetc(ffp)) != EOF && c != '\n') {
                if (i >= nbuf-2) {
                         buf[nbuf - 2] = c;   /* store last char read */
                         buf[nbuf - 1] = 0;   /* and terminate it */
                        mlwrite("File has long line");
                        return (FIOLNG);
                }
                if (transtab && c == ' ') {
                        rempos = pos;
                        do {
                                instab = !(pos++%8);
                        }  while (!instab && ((c = fgetc(ffp)) == ' '));

                        if (instab)
                                buf[i++] = pos==rempos+1? ' ': '\t';
                        else {
                                ungetc(c, ffp);
                                while (rempos++ < pos && !(i >= nbuf-2))
                                        buf[i++] = ' ';
                        }
                }
                else {
                        buf[i++] = c;
                        if (c=='\t') {
                                if (val = (pos+8)%8)
                                        pos += val;
                                else
                                        pos += 8;
                        }
                        else
                                pos++;
                }
        }

        if (c == EOF) {
                if (ferror(ffp)) {
                        mlwrite("File read error");
                        return (FIOERR);
                }

                if (i != 0) {
#if FUNNYEOF
                        mlwrite("File has funny line at EOF");
                        return (FIOERR);
#else
                        buf[i] = '\0';
                        return (FIOSUC);
#endif
                }
                return (FIOEOF);
        }

        buf[i] = 0;
        return (FIOSUC);
}



/* filename completion */
#define MAXFILES       1024

static char **intbuflist;

getfilenamelist(bufn, bufl, listing)
char *bufn;
char ***listing;
{
   register BUFFER *bp;
   int    listc;
   char **listv;
   

      if ((listv = (char **)malloc((MAXFILES+1)*sizeof(char *)))
                 == (char **) NULL) {
         mlwrite("[Memory allocation error %d (completion:listv)]", 
            (MAXFILES+1)*sizeof(char *));
         return;
      }

      if (!getdirlist(bufn, bufl, &listv, &listc, MAXFILES)) {
         if (listv != (char **) NULL)
            free((char *)listv);
         *listing = (char **) NULL;
         return;
      }

      listv[listc] = (char *) NULL;
      intbuflist = listv;

   *listing = intbuflist;
}

mlfilereply(prompt, buf, nbuf)
char *prompt;
char *buf;
{
   int res;
   char **freehelp;
   
   if ((completion=(char **)malloc((MAXFILES+1)*sizeof(char *)))
                  ==(char **)NULL) {
      mlwrite("mlfilereply: memory allocation error (%d:completion)]", 
            (MAXFILES+1)*sizeof(char *) );
      return (FALSE);
   }

   res = mlmatchreply(prompt, buf, nbuf, getfilenamelist);

   if ((freehelp=intbuflist) != (char **)NULL) {
      while (*freehelp != (char *) NULL)
         free(*freehelp++);
      free((char *)(freehelp = intbuflist));
      intbuflist = (char **) NULL;
   }
   free((char *)completion);
   completion = (char **) NULL;
   return (res);
}

/* this is the directory listing part */

#if  PR1MOS
#include <keys>
#include <errd>

#define SAMFILE 0
#define DAMFILE 1
#define SAMSEG  2
#define DAMSEG  3
#define UFDFILE 4
#define ACATFL  6
#define CAMFILE 7

#define IS_DIR  128
static void StripPath(str,out)
char *str,*out;
{
   char *cp, *strrchr();


   if (cp = strrchr(str, '>')) {
      if (strlen(++cp) > 32) { /* note: dot */
         strncpy(out, cp, 32);
         out[32] = 0;
      }
      else
         strcpy(out,cp);
   }
   else {
      if (strlen(str) > 32) { /* note: no dot */
         strncpy(out, cp, 32);
         out[32] = 0;
      }
      else
         strcpy(out,str);      
   }
}
#endif


getdirlist(bufn, bufl, listing, listlen, listmax)
char  *bufn;
char  ***listing;
int   *listlen;
{
#if PR1MOS
   char dirnam[NFILEN], filnam[NFILEN],*cp, pathstr[NFILEN+40];
   char **DirBuf;
   int xx,dpos,unit,filtyp;
   short primosFileUnit;
   short dtype,suf_used,code;
   struct       {
      short     len;
      char      str[NFILEN];
   } pathname;
   char locbuf1[34], locbuf2[34];
   char *errtxt();
   char *lsdir();
   fortran void srsfx$();


   *listlen = 0;
   DirBuf = *listing;

      for (xx= dpos = 0; xx < bufl; xx++) {
         pathstr[xx] = bufn[xx];
         if (bufn[xx] == '>')
            dpos = xx;
      }
      pathstr[bufl] = '\0';
      strcat(pathstr, "@@"); /* primos wildcard */

      if (dpos) {
         strncpy(dirnam,pathstr,dpos);
         dirnam[dpos] = '\0';
         strcpy(pathname.str,dirnam);
         pathname.len = strlen(dirnam);


         srsfx$( (short) IS_DIR | k$read | k$getu, pathname,
                 primosFileUnit, dtype, (short)0, "", "", suf_used, code);

         if (code != 0) {
            strcpy(pathstr, "[getdirlist:");
            strcat(pathstr, errtxt(code));
            strcat(pathstr, "]");
            mlwrite(pathstr);
            return 0;
         } /* if error */
         else {
            if (dtype != UFDFILE) {
               strcpy(pathstr, "[getdirlist:");
               strcat(pathstr, dirnam);
               strcat(pathstr, "is not a directory]");
               mlwrite(pathstr);
               return 0;
            }
            unit = open(dirnam, -1, primosFileUnit);
         } /* if error else */

      } /* if dpos */
      else {
         dirnam[0] = '\0';
         unit = open("_current-ufd_",0);
      } /* if (dpos) else*/

      if (unit==-1) {
         mlwrite("[getdirlist: pathname error (wildcard)]");
         return 0;
      }

      upcase(dirnam,dirnam,NFILEN);
      upcase(pathstr,pathstr,NFILEN);
      while ((cp=lsdir(unit))!=NULL) {
         if ((DirBuf[*listlen] = malloc(NFILEN))==NULL) {
            mlwrite("[getdirlist:memory allocation error (%d)]", NFILEN);
            close(unit);
            return 0;
         }
         strcpy(DirBuf[*listlen],dirnam);
         if (dirnam[0])
            strcat(DirBuf[*listlen],">");
         strcat(DirBuf[*listlen],cp);

         StripPath(pathstr,locbuf1);
         StripPath(DirBuf[*listlen],locbuf2);

         if (wildc(locbuf1,locbuf2)) {
            strlower(DirBuf[*listlen]);
            switch(ftype(DirBuf[*listlen])) {
               case SAMFILE:
               case DAMFILE:
               case CAMFILE:
                  completion[*listlen] = (char *) NULL;
                  break;
               case UFDFILE:
                  if (dirnam[0]==0) {
                     strcpy(filnam, DirBuf[*listlen]);
                     strcpy(DirBuf[*listlen], "*>");
                     strcat(DirBuf[*listlen], filnam);
                  }
                  completion[*listlen] = ">";
                  break;
               case ACATFL:
                  completion[*listlen] = " (acat)";
                  break;
               case SAMSEG:
                  completion[*listlen] = " (samseg dir)";
                  break;
               case DAMSEG:
                  completion[*listlen] = " (damseg dir)";
                  break;
               default:
                  completion[*listlen] = " (unknown type)";
                  break;
            }

            if (++(*listlen) > listmax) {
               mlwrite("getdirlist: too many files (>%d)", listmax);
               close(unit);
               return (0);
            } /* if maxargs */
         } /* if wildc */
         else
            free(DirBuf[*listlen]);
      } /* while more entries in dir */

   shellsortstr(DirBuf, *listlen, NFILEN);
   close(unit);
   return 1;
#endif

   return 0;
}
