/* fintcm.c
 * The routines in this file provide support for FINTCM style terminals
 * over a serial line. The serial I/O services are provided by routines in
 * "termio.c". It compiles into nothing if not an FINTCM device.
 */

#define   termdef   1         /* don't define "term" external */

#include        <stdio.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"

#if   FINTCM

#include <tcm.h>
#define NROW    24                      /* Screen size.                 */
#define NCOL    80                      /* Edit if you want to.         */


#define   NPAUSE   300         /* # times thru update to pause */
#define   MARGIN   8         /* size of minimim margin and   */
#define   SCRSIZ   64         /* scroll size for extended lines */

#define BEL     ZZSETP(0x07)                    /* BEL character.     */

extern  int     ttopen();               /* Forward references.          */
extern  int     ttgetc();
extern  int     ttputc();
extern  int     ttflush();
extern  int     ttclose();
extern  int     tc_setcur();
extern  int     tc_clreol();
extern  int     tc_cls();
extern  int     tcmbeep();
extern  int     tcmopen();
extern  int     tcmrev();
extern  int     tcmclose();

#if   COLOR
extern   int   tcmfcol();
extern   int   tcmbcol();

int   cfcolor = -1;      /* current forground color */
int   cbcolor = -1;      /* current background color */
#endif

/*
 * Standard terminal interface dispatch table. Most of the fields point into
 * "termio" code.
 */
TERM    term    = {
        NROW-1,
        NCOL,
        MARGIN,
        SCRSIZ,
        NPAUSE,
        tcmopen,
        tcmclose,
        ttgetc,
        ttputc,
        ttflush,
        tc_setcur,
        tc_clreol,
        tc_cls,
        tcmbeep,
        tcmrev
#if   COLOR
   , tcmfcol,
   tcmbcol
#endif
};

#if   COLOR
tcmfcol(color)      /* set the current output color */
int color;
{

}

tcmbcol(color)      /* set the current background color */
int color;
{
}
#endif

tcmrev(state)      /* change reverse video state */

int state;   /* TRUE = reverse, FALSE = normal */

{
#if   COLOR
   int ftmp, btmp;      /* temporaries for colors */
#endif
   
tc_attrib(state ? TC_ATTR_REVERSE: TC_ATTR_NORMAL);

#if   COLOR
   if (state == FALSE) {
      ftmp = cfcolor;
      btmp = cbcolor;
      cfcolor = -1;
      cbcolor = -1;
      tcmfcol(ftmp);
      tcmbcol(btmp);
   }
#endif
}

tcmbeep()
{
        ttputc(BEL);
        ttflush();
}

#if !PR1MOS
        char *getenv();
#else
        char *gvget();
#endif

tcmopen()
{
   char *cp;
   extern int scr_change; /* should be placed in header file */
   int rows;

   tc_init((char *) 0);


#if MSDOS | V7 | BSD | USG
   if ((cp = getenv("WIDE")) != NULL)
      tc_bigscreen(1);
#endif              
#if PR1MOS
   if ((cp = gvget(".WIDE")) != NULL)
      tc_bigscreen(1);
#endif


   if (!scr_change) {
      term.t_nrow = tc_length()-1;
      term.t_ncol = tc_width();
      revexist = (tc_attrib(TC_ATTR_REVERSE)==RC_OK);
      if (revexist)
         tc_attrib(TC_ATTR_NORMAL);
   }
#if MSDOS | V7 | BSD | USG
   if ((cp = getenv("ROWS")) != NULL)
      if (sscanf(cp,"%d",&rows) == 1 && rows > 3) {
         scr_change = 1;
         term.t_nrow = rows-1;
      }
#endif              
#if PR1MOS
   if ((cp = gvget(".ROWS")) != NULL)
      if (sscanf(cp,"%d",&rows) == 1 && rows > 3) {
         scr_change = 1;
         term.t_nrow = rows-1;
      }
#endif

   ttopen();
}

tcmclose()
{
   char *cp;

#if   COLOR
   tcmfcol(7);
   tcmbcol(0);
#endif

#if MSDOS | V7 | BSD | USG
        if ((cp = getenv("WIDE")) != NULL) {
           if  (*cp=='e' || *cp=='E')
              tc_bigscreen(0);
        }
#endif
#if PR1MOS
        if ((cp = gvget(".WIDE")) != NULL) {
           if  (*cp=='e' || *cp=='E')
              tc_bigscreen(0);
        }
#endif
   
   ttclose();
}
#else
tcmhello()
{
}
#endif
