/* fintfc.c
 * The routines in this file provide support for FINTFC style terminals
 * over a serial line. The serial I/O services are provided by routines in
 * "termio.c". It compiles into nothing if not an FINTFC device.
 */

#define   termdef   1         /* don't define "term" external */

#include        <stdio.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"

#if   FINTFC

#include <fincap.h>
#define NROW    24                      /* Screen size.                 */
#define NCOL    80                      /* Edit if you want to.         */


#define   NPAUSE   300         /* # times thru update to pause */
#define   MARGIN   8         /* size of minimim margin and   */
#define   SCRSIZ   64         /* scroll size for extended lines */

#define BEL     ZZSETP(0x07)                    /* BEL character.     */

extern  int     ttopen();               /* Forward references.          */
extern  int     ttgetc();
extern  int     ttputc();
extern  int     ttflush();
extern  int     ttclose();
extern  int     tf_setcur();
extern  int     tf_clreol();
extern  int     tf_cls();
extern  int     tfcbeep();
extern  int     tfcopen();
extern  int     tfcrev();
extern  int     tfcclose();

#if   COLOR
extern   int   tfcfcol();
extern   int   tfcbcol();

int   cfcolor = -1;      /* current forground color */
int   cbcolor = -1;      /* current background color */
#endif

/*
 * Standard terminal interface dispatch table. Most of the fields point into
 * "termio" code.
 */
TERM    term    = {
        NROW-1,
        NCOL,
        MARGIN,
        SCRSIZ,
        NPAUSE,
        tfcopen,
        tfcclose,
        ttgetc,
        ttputc,
        ttflush,
        tf_setcur,
        tf_clreol,
        tf_cls,
        tfcbeep,
        tfcrev
#if   COLOR
   , tfcfcol,
   tfcbcol
#endif
};

#if   COLOR
tfcfcol(color)      /* set the current output color */
int color;
{

}

tfcbcol(color)      /* set the current background color */
int color;
{
}
#endif

tfcrev(state)      /* change reverse video state */

int state;   /* TRUE = reverse, FALSE = normal */

{
#if   COLOR
   int ftmp, btmp;      /* temporaries for colors */
#endif

tf_attrib(state ? ATTR_REVERSE: ATTR_NORMAL);

#if   COLOR
   if (state == FALSE) {
      ftmp = cfcolor;
      btmp = cbcolor;
      cfcolor = -1;
      cbcolor = -1;
      tfcfcol(ftmp);
      tfcbcol(btmp);
   }
#endif
}

tfcbeep()
{
        ttputc(BEL);
        ttflush();
}

#if !PR1MOS
        char *getenv();
#else
        char *gvget();
#endif

tfcopen()
{
   char *cp;
   extern int scr_change; /* should be placed in header file */
   int rows;

   tf_init((char *) 0);


   if (!scr_change) {
      term.t_nrow = tf_height()-1;
      term.t_ncol = tf_width();
      revexist = (tf_attrib(ATTR_REVERSE)==RC_OK);
      if (revexist)
         tf_attrib(ATTR_NORMAL);
   }
#if MSDOS | V7 | BSD | USG
   if ((cp = getenv("ROWS")) != NULL)
      if (sscanf(cp,"%d",&rows) == 1 && rows > 3) {
         scr_change = 1;
         term.t_nrow = rows-1;
      }
#endif
#if PR1MOS
   if ((cp = gvget(".ROWS")) != NULL)
      if (sscanf(cp,"%d",&rows) == 1 && rows > 3) {
         scr_change = 1;
         term.t_nrow = rows-1;
      }
#endif

   ttopen();
}

tfcclose()
{
   char *cp;

#if   COLOR
   tfcfcol(7);
   tfcbcol(0);
#endif


   ttclose();
}
#else
tfchello()
{
}
#endif
