/* indent.c
 * This file contains the indentation functions
 */

#include        <stdio.h>
#include        "estruct.h"
#include        "edef.h"

static char indentbuf[NSTRING] = "";   /* buffer to hold indent of last line */
static int  indentflag = 0;
static LINE *indentcheck = NULL;


insstartbrace(n)
{
   return (executeindstr(startindent, n, 1));
}


insendbrace(n)
{
   return (executeindstr(endindent, n, 1));
}


inspound(n)   /* insert a # into the text here...we are in cmode */

{
   register int ch;   /* last character before input */
   register int i;

   /* if we are at the beginning of the line, no go */
   if (curwp->w_doto == 0)
      return(linsert(1,'#'));

   /* scan to see if all space before this is white space */
   for (i = curwp->w_doto - 1; i >= 0; --i) {
      ch = lgetc(curwp->w_dotp, i);
      if (ch != ' ' && ch != '\t')
         return(linsert(1, '#'));
   }

   /* delete back first */
   while (getccol(FALSE) >= 1)
      backdel(FALSE, 1);

   /* and insert the required pound */
   return(linsert(1, '#'));
}

/*
 * if given argument, n, of zero, change to true tabs.
 * if n > 1, simulate tab stop every n-characters using spaces.
 */
settabsize(f, n)
{
        int s, r;
        char number[20];

        /* Have we used any argument? */
        if (f==FALSE) {
                /* Prompt for the tab size */
                if (r = mlreply("Tab size [0=Real tabs]: ", number, 20) != TRUE) {
                        mlwrite("");
                        return (r);
                }
                n = atoi(number);
        }

        s = FALSE;
        if (n == 0 || n > 1) {
                tabsize = n;
                s = TRUE;
        }

        if (s==FALSE)
                mlwrite("[Bad tab size %d]", n);
        else if (tabsize)
                mlwrite("[Tab size set to %d]", tabsize);
        else
                mlwrite("[Real tabs in effect]");
        return (s);
}


/*
 * insert a tab into file. this has to be
 * done in this slightly funny way because the tab (in ascii) has been turned
 * into "c-i" (in 10 bit code) already. bound to "c-i".
 */
tab(f, n)
{
        int s;

        if (n < 0)
                return (backtab(f,-n));

        for (s = TRUE; n-- > 0 && s;) {
                if (! tabsize)
                        s = linsert(1, '\t');
                else
                        s = linsert(tabsize - (getccol(FALSE) % tabsize), ' ');
        }

        return (TRUE);
}


/*
 * set tab size if given non-default argument (n <> 1).  otherwise, delete a
 * tab.  if given argument, n, of zero, change to true tabs.
 * if n > 1, simulate tab stop every n-characters using spaces. this has to be
 * done in this slightly funny way because the tab (in ascii) has been turned
 * into "c-i" (in 10 bit code) already. bound to "m-c-i".
 */
backtab(f, n)
{
        int target, source, s, ch;

        if (n < 0)
                return (tab(f,-n));

        for (s = TRUE; n-- > 0 && s;) {
                target = getccol(FALSE);   /* calc where we will delete to */
                target -= 1;
                target -= target % (tabsize == 0 ? 8 : tabsize);

        	s = ((ch=prevchar())==' ' || ch == '\t' || ch=='\n');

                for (source = getccol(FALSE);
                     source > target && s; 
                     source = getccol(FALSE) ) {
        		s = backdel(FALSE, 1);
        		if ( s && ((ch=prevchar())==' ' || ch == '\t' || ch=='\n'))
        			s = TRUE;
        		else
        			s = FALSE;
        	}
        }

        return (TRUE);
}


indentinsert(n, ch)
{
   int len;
   len = strlen(indentbuf);
   while (n -- > 0) {
      if (len < NSTRING-1)
         indentbuf[len++] = ch;
   }
   indentbuf[len] = '\0';
}

insertnewline()   /* insert a newline */

{
   register char *cptr;   /* string pointer into text to copy */
   register int tptr;   /* index to scan into line */
   register int i;
   register int bracef;   /* was there a brace at the end of line? */
   char  brace;           /* Which brace ???? */
   char ichar[NSTRING];   /* buffer to hold indent of last line */

   /* grab a pointer to text to copy indentation from */
   cptr = &curwp->w_dotp->l_text[0];

   /* check for a brace */
   tptr = curwp->w_doto - 1;
   bracef = FALSE;
   if (curbp->b_mode & MDCMOD)  { /* Do this only if in C mode */
      for (i=tptr; i>=0 &&
         !(bracef = ((brace=cptr[i]) == '{' || brace == '}')); i--)
         ;
   }

   /* save the indent of the previous line */
   i = 0;
   while ((i <= tptr) && (cptr[i] == ' ' || cptr[i] == '\t')
      && (i < NSTRING - 1)) {
      ichar[i] = cptr[i];
      ++i;
   }
   ichar[i] = 0;      /* terminate it */

   /* put in the newline */
   if (lnewline() == FALSE)
      return(FALSE);

   /* and the saved indentation */
      i = 0;
      while (ichar[i])
         linsert(1, ichar[i++]);

   if (indentcheck == curwp->w_dotp || bracef) {
      if (bracef) {
         switch (brace) {
            case '{':
               executeindstr(startindent, 1, 0);
               break;
            case '}':
               executeindstr(endindent, 1, 0);
               break;
            default:
               indentflag = 0;
               break;
         }
      }

      if (indentflag > 0)
         tab(FALSE, indentflag);
      else if (indentflag < 0)
         backtab(FALSE, -indentflag);
   }

   indentflag = 0;

   return(TRUE);
}


executeindstr(str, n, e)
char *str;
{
   char *cp, c;

   indentflag = 0;
   while ( n -- > 0 ) {
      for (cp = str; c=*cp++; ) {
         switch (c) {
            case '{':
               if (e)
                  linsert(1, c);
               break;
            case '}':
               if (e)
                  linsert(1, c);
               break;
            case 'i':
               indentflag++;
               break;
            case 'd':
               indentflag--;
               break;
            case 't':
               if (e)
                  tab(FALSE, 1);
               break;
            case 'b':
               if (e)
                  backtab(FALSE, 1);
               break;
            case 'n':
               if (e) /* Careful here, could cause recursion */
                  insertnewline();
               break;
            default:
               break;
         }
      }
   }
   indentcheck = curwp->w_dotp;
   return (TRUE);
}


/*
 * Return current column.  Stop at first non-blank given TRUE argument.
 */
getccol(bflg)
int bflg;
{
        register int c, i, col;
        col = 0;
        for (i=0; i<curwp->w_doto; ++i) {
                c = lgetc(curwp->w_dotp, i);
                if (c!=' ' && c!='\t' && bflg)
                        break;
                if (c == '\t')
                        col |= 0x07;
                else if (c<ZZSETP(0x20) || c==ZZSETP(0x7F))
                        ++col;
                ++col;
        }
        return(col);
}


currchar()
{
   int curchar;

   if ((curwp->w_doto) >= llength(curwp->w_dotp))
      curchar = '\n';
   else
      curchar = lgetc(curwp->w_dotp, curwp->w_doto);

   return (curchar);
}


prevchar()
{
   int prvchar;
   int curpos;
   LINE *curlin;

   curlin = curwp->w_dotp;
   curpos = curwp->w_doto >= llength(curlin) ? llength(curlin) : curwp->w_doto;
   if (curpos - 1 < 0)
      prvchar = '\n';
   else
      prvchar = lgetc(curlin, curpos - 1);

   return (prvchar);
}
