/*   INPUT:   Various input routines for MicroEMACS 3.7
      written by Daniel Lawrence
      5/9/86                  */

#include   <stdio.h>
#include   "estruct.h"
#include   "edef.h"

/*
 * Ask a yes or no question in the message line. Return either TRUE, FALSE, or
 * ABORT. The ABORT status is returned if the user bumps out of the question
 * with a ^G. Used any time a confirmation is required.
 */

mlyesno(prompt)

char *prompt;

{
   char c;         /* input character */
   char buf[NPAT];      /* prompt to user */

   for (;;) {
      /* build and prompt the user */
      strcpy(buf, prompt);
      strcat(buf, " [y/n]? ");
      mlwrite(buf);

      /* get the responce */
      c = (*term.t_getchar)();

      if (c == BELL)      /* Bail out! */
         return(ABORT);

      if (c=='y' || c=='Y')
         return(TRUE);

      if (c=='n' || c=='N')
         return(FALSE);
   }
}

/*
 *     mlreply calls mlmatchreply...
 */

mlreply(prompt, buf, nbuf)
char *prompt;
char *buf;
{
   return (mlmatchreply(prompt, buf, nbuf, (PFI) 0));
}

/*
 * Write a prompt into the message line, then read back a response. Keep
 * track of the physical position of the cursor. If we are in a keyboard
 * macro throw the prompt away, and return the remembered response. This
 * lets macros run at full speed. The reply is always terminated by a carriage
 * return. Handle erase, kill, and abort keys.
 */

static int ttend;    /* variable for column max */
static int ttbeg;    /* variable for column min */

mlmatchreply(prompt, buf, nbuf, func)
char *prompt;
char *buf;
PFI func;
{
#define  DECRTT   { --ttcol; --ttend; }
            int cpos;     /* current character position in string */
   register int i;
   register int c;
   register int quotef;   /* are we quoting the next char? */
   register int status;   /* status return value */
            int epos;     /* end of line pos */
            int tttmp;    /* temporary variable for ttcol */

   epos = cpos = 0;
   quotef = FALSE;
 
   if (kbdmop != NULL) {
      while ((c = *kbdmop++) != '\0')
         buf[cpos++] = c;

      buf[cpos] = 0;

      if (buf[0] == 0)
         return(FALSE);

      return(TRUE);
   }

   /* check to see if we are executing a command line */
   if (clexec) {
      status = nxtarg(buf);
      buf[nbuf-1] = 0;   /* make sure we null terminate it */
      return(status);
   }

   mlwrite(prompt);
   ttbeg = ttend = ttcol;

   for (;;) {
   /* get a character from the user. if it is a <ret>, change it
      to a <NL>                     */
      c = (*term.t_getchar)();
      if (c == ZZSETP(0x0d))
         c = '\n';
      else if (c=='\n')              /* Kludge: ^J is converted to ^M */
         c = '\r';

      if ((c == '\n'|| c == '\r') && quotef == FALSE) {
         buf[epos++] = 0;

         if (kbdmip != NULL) {
            if (kbdmip+epos > &kbdm[NKBDM-3]) {
               ctrlg(FALSE, 0);
               (*term.t_flush)();
               return(ABORT);
            }

            for (i=0; i<cpos; ++i)
               *kbdmip++ = buf[i];
            }

            (*term.t_move)(term.t_nrow, 0);
            ttend = ttcol = 0;
            (*term.t_flush)();

            if (buf[0] == 0)
               return(FALSE);

            return(TRUE);

         } else if (c == ZZSETP(0x07) && quotef == FALSE) {
            /* Bell, abort */
            (*term.t_putchar)('^');
            (*term.t_putchar)('G');
            ttcol += 2;
            ctrlg(FALSE, 0);
            (*term.t_flush)();
            buf[0] = 0;
            return(ABORT);

         } else if (c==ZZSETP(0x08) && quotef==FALSE) {
            /* erase */
            if (cpos != 0) {
               (*term.t_putchar)('\b');
               (*term.t_putchar)(' ');
               (*term.t_putchar)('\b');
               DECRTT;

               if (buf[--cpos] < ZZSETP(0x20) || buf[cpos]==ZZSETP(0x7F)) {
                  (*term.t_putchar)('\b');
                  (*term.t_putchar)(' ');
                  (*term.t_putchar)('\b');
                  DECRTT;
               }

               if (buf[cpos] == '\n') {
                  (*term.t_putchar)('\b');
                  (*term.t_putchar)('\b');
                  (*term.t_putchar)(' ');
                  (*term.t_putchar)(' ');
                  (*term.t_putchar)('\b');
                  (*term.t_putchar)('\b');
                  DECRTT;
                  DECRTT;
               }

               if (cpos < --epos) {
                  tttmp = ttcol;
                  for (i=cpos; i<epos; i++)
                     inpctrlout(buf[i] = buf[i+1]);
                  ttend = ttcol;
                  (*term.t_eeol)();
                  (*term.t_move)(term.t_nrow, ttcol=tttmp);
               }
               (*term.t_flush)();
            }
         } else if ((c==ZZSETP(0x7F) || c==ZZSETP(0x04)) && quotef==FALSE) {
            /* C-D */
            if (cpos < epos) {
               tttmp = ttcol;
               epos--;
               for (i=cpos; i<epos; i++)
                  inpctrlout(buf[i] = buf[i+1]);
               ttend = ttcol;
               (*term.t_eeol)();
               (*term.t_move)(term.t_nrow, ttcol=tttmp);
               (*term.t_flush)();
            }
         } else if (c == ZZSETP(0x15) && quotef == FALSE) {
            /* C-U, kill */

            epos = 0;
            cpos = 0;
            (*term.t_move)(term.t_nrow, ttcol=0);
            mlwrite(prompt);
            ttend = ttcol;
            (*term.t_flush)();

         } else if (c == ZZSETP(0x02) && quotef == FALSE) {
            /* C-B */
            if (cpos > 0) {
               (*term.t_putchar)('\b');
               --ttcol;
               if (buf[--cpos] < ZZSETP(0x20) || buf[cpos]==ZZSETP(0x7F)) {
                  (*term.t_putchar)('\b');
                  --ttcol;
               }
               if (buf[cpos] == '\n') {
                  (*term.t_putchar)('\b');
                  (*term.t_putchar)('\b');
                  --ttcol;
                  --ttcol;
               }
               (*term.t_flush)();
            }
         } else if (c == ZZSETP(0x06) && quotef == FALSE) {
            /* C-F */
            if (cpos < epos) {
               inpctrlout(buf[cpos]);
               ++cpos;
               (*term.t_flush)();
            }
         } else if (c == ZZSETP(0x01) && quotef == FALSE) {
            /* C-A */
            if (cpos) {
               (*term.t_move)(term.t_nrow, ttcol = ttbeg);
               (*term.t_flush)();
               cpos = 0;
            }
         } else if (c == ZZSETP(0x05) && quotef == FALSE) {
            /* C-E */
            if (cpos < epos) {
               (*term.t_move)(term.t_nrow, ttcol = ttend);
               (*term.t_flush)();
               cpos = epos;
            }
         } else if (c == ZZSETP(0x0B) && quotef == FALSE) {
            /* C-K */
            (*term.t_eeol)();
            (*term.t_flush)();
            (*term.t_move)(term.t_nrow, ttend = ttcol = ttbeg+cpos);
            epos = cpos;
         } else if (c == ZZSETP(0x0C) && quotef == FALSE) {
            /* C-L */
            (*term.t_move)(term.t_nrow, ttcol=0);
            mlwrite(prompt);
            for (i=0; i<epos; i++)
               inpctrlout(buf[i]);
            ttend = ttcol;
            (*term.t_move)(term.t_nrow, ttcol=ttbeg+cpos);
            (*term.t_flush)();
         } else if (c == '\t' && func != (PFI) 0 && quotef == FALSE) {
            if (!handlecompletion(buf, nbuf, &cpos, &epos, func)) {
               buf[0] = 0;
               return (ABORT);
            }
         } else if (c == quotec && quotef == FALSE) {
            quotef = TRUE;
         } else {
            quotef = FALSE;
            if (epos < nbuf-1 && ttend < term.t_ncol-3) {
               inpctrlout(c);
               epos ++;
               cpos ++;
               if (cpos < epos) {
                  tttmp = ttcol;
                  for (i=epos; i >= cpos; i--)
                     buf[i] = buf[i-1];
                  for (i=cpos; i < epos; i++)
                     inpctrlout(buf[i]);
                  ttend = ttcol;
                  (*term.t_move)(term.t_nrow, ttcol=tttmp);
               }
               buf[cpos-1] = c;
               (*term.t_flush)();
            }
         }

      if (epos < cpos)
         epos = cpos;
      if (ttend < ttcol)
         ttend = ttcol;
   }
   buf[0] = 0;
   return(ABORT);
}
#undef DECRTT


/* Kludge! This should be a general control codes output routine */

inpctrlout(c)
{
   if ((c < ' ' || c==ZZSETP(0x7F)) && (c != '\n')) {
      (*term.t_putchar)('^');
      ++ttcol;

      c = (c==ZZSETP(0x7F)? '?': c ^ ZZSETP(0x40));
   }

   if (c != '\n')
      (*term.t_putchar)(c);
   else {   /* put out <NL> for <ret> */
      (*term.t_putchar)('<');
      (*term.t_putchar)('N');
      (*term.t_putchar)('L');
      (*term.t_putchar)('>');
      ttcol += 3;
   }
   ttcol ++;
}

/* get a command name from the command line. Command completion means
   that pressing a <SPACE> will attempt to complete an unfinished command
   name if it is unique.
*/

int (*getname())()

{
   register int cpos;   /* current column on screen output */
   register int c;
   register char *sp;   /* pointer to string for output */
   register NBIND *ffp;   /* first ptr to entry in name binding table */
   register NBIND *cffp;   /* current ptr to entry in name binding table */
   register NBIND *lffp;   /* last ptr to entry in name binding table */
   char buf[NSTRING];   /* buffer to hold tentative command name */
   int (*fncmatch())();

   /* starting at the begining of the string buffer */
   cpos = 0;

   /* if we are executing a keyboard macro, fill our buffer from there,
      and attempt a straight match */
   if (kbdmop != NULL) {
      while ((c = *kbdmop++) != '\0')
         buf[cpos++] = c;

      buf[cpos] = 0;

      /* return the result of a match */
      return(fncmatch(&buf[0]));
   }

   /* if we are executing a command line get the next arg and match it */
   if (clexec) {
      if (nxtarg(buf) != TRUE)
         return((PFI) FALSE);
      return(fncmatch(&buf[0]));
   }

   /* build a name string from the keyboard */
   while (TRUE) {
      c = (*term.t_getchar)();

      /* if we are at the end, just match it */
      if (c == ZZSETP(0x0d)) {
            c = '\n';
      }
      if (c == '\n') {
         buf[cpos] = 0;

         /* save keyboard macro string if needed */
         if (kbdtext(&buf[0]) == ABORT)
            return( (int (*)()) NULL);

         /* and match it off */
         return(fncmatch(&buf[0]));

      } else if (c == ZZSETP(0x07)) {   /* Bell, abort */
         (*term.t_putchar)('^');
         (*term.t_putchar)('G');
         ttcol += 2;
         ctrlg(FALSE, 0);
         (*term.t_flush)();
         return( (int (*)()) NULL);

      } else if (c == ZZSETP(0x7F) || c == ZZSETP(0x08)) {   /* rubout/erase */
         if (cpos != 0) {
            (*term.t_putchar)('\b');
            (*term.t_putchar)(' ');
            (*term.t_putchar)('\b');
            --ttcol;
            --cpos;
            (*term.t_flush)();
         }

      } else if (c == ZZSETP(0x15)) {   /* C-U, kill */
         while (cpos != 0) {
            (*term.t_putchar)('\b');
            (*term.t_putchar)(' ');
            (*term.t_putchar)('\b');
            --cpos;
            --ttcol;
         }

         (*term.t_flush)();

      } else if (c == ' ' || c == '\t') {
/* <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */
   /* attempt a completion */
   buf[cpos] = 0;      /* terminate it for us */
   ffp = &names[0];   /* scan for matches */
   while (ffp->n_func != NULL) {
      if (strncmp(buf, ffp->n_name, strlen(buf)) == 0) {
         /* a possible match! More than one? */
         if ((ffp + 1)->n_func == NULL ||
            (strncmp(buf, (ffp+1)->n_name, strlen(buf)) != 0)) {
            /* no...we match, print it */
            sp = ffp->n_name + cpos;
            while (*sp)
               (*term.t_putchar)(*sp++);
            (*term.t_flush)();
            return(ffp->n_func);
         } else {
/* << << << << << << << << << << << << << << << << << */
   /* try for a partial match against the list */

   /* first scan down until we no longer match the current input */
   lffp = (ffp + 1);
   while ((lffp+1)->n_func != NULL) {
      if (strncmp(buf, (lffp+1)->n_name, strlen(buf)) != 0)
         break;
      ++lffp;
   }

   /* and now, attempt to partial complete the string, char at a time */
   while (TRUE) {
      /* add the next char in */
      buf[cpos] = ffp->n_name[cpos];

      /* scan through the candidates */
      cffp = ffp + 1;
      while (cffp <= lffp) {
         if (cffp->n_name[cpos] != buf[cpos])
            goto onward;
         ++cffp;
      }

      /* add the character */
      (*term.t_putchar)(buf[cpos++]);
   }
/* << << << << << << << << << << << << << << << << << */
         }
      }
      ++ffp;
   }

   /* no match.....beep and onward */
   (*term.t_beep)();
onward:;
   (*term.t_flush)();
/* <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */
      } else {
         if (cpos < NSTRING-1 && c > ' ') {
            buf[cpos++] = c;
            (*term.t_putchar)(c);
         }

         ++ttcol;
         (*term.t_flush)();
      }
   }
}

kbdtext(buf)   /* add this text string to the current keyboard macro
         definition                  */

char *buf;   /* text to add to keyboard macro */

{
   /* if we are defining a keyboard macro, save it */
   if (kbdmip != NULL) {
      if (kbdmip+strlen(buf) > &kbdm[NKBDM-4]) {
         ctrlg(FALSE, 0);
         (*term.t_flush)();
         return(ABORT);
      }

      /* copy string in and null terminate it */
      while (*buf)
         *kbdmip++ = *buf++;
      *kbdmip++ = 0;
   }
   return(TRUE);
}

/*   GET1KEY:   Get one keystroke. The only prefixs legal here
         are the SPEC and CTRL prefixes.
                        */

get1key()

{
   int    c;
#if FUNCFIX
   extern int GlobalFuncFlag;
#endif
#if   AMIGA
   int   d;
#endif

   /* get a keystroke */
        c = (*term.t_getchar)();

#if RAINBOW

        if (c & Function_Key)
                {
                int i;

                for (i = 0; i < lk_map_size; i++)
                        if (c == lk_map[i][0])
                                return lk_map[i][1];
                }
        else if (c == Shift + 015) return CTRL | 'J';
        else if (c == Shift + 0x7F) return META | 0x7F;
#endif

#if   MSDOS
   if (c == 0) {            /* Apply SPEC prefix   */
           c = (*term.t_getchar)();
           if (c>=0x00 && c<=0x1F)      /* control key? */
                   c = CTRL | (c+'@');
      return(SPEC | c);
   }
#endif

#if   AMIGA
   /* apply SPEC prefix */
   if ((unsigned)c == 155) {
      c = (*term.t_getchar)();

      /* first try to see if it is a cursor key */
      if ((c >= 'A' && c <= 'D') || c == 'S' || c == 'T')
         return(SPEC | c);

      /* next, a 2 char sequence */
      d = (*term.t_getchar)();
      if (d == '~')
         return(SPEC | c);

      /* decode a 3 char sequence */
      c = d + 32;
      /* if a shifted function key, eat the tilde */
      if (d >= '0' && d <= '9')
         d = (*term.t_getchar)();
      return(SPEC | c);
   }
#endif

#if  WANGPC
   if (c == 0x1F) {         /* Apply SPEC prefix    */
           c = (*term.t_getchar)();
      return(SPEC | c);
   }
#endif

        if (c>=ZZSETP(0x00) && c<=ZZSETP(0x1F))   /* C0 control -> C-     */
                c = ZZSETP(CTRL | (c+'@'));
#if FUNCFIX
        if (GlobalFuncFlag) {
           GlobalFuncFlag = 0;
           return (SPEC | c);
        }
#endif
        return (c);
}

/*   GETCMD:   Get a command from the keyboard. Process all applicable */
/*             prefix keys                                             */

getcmd()

{
   int c;      /* fetched keystroke */

   /* get initial character */
   c = get1key();

   /* process META prefix */
   if (c == metac) {
      c = get1key();
           if (c>='a' && c<='z')      /* Force to upper */
                   c -= 0x20;
           if (c>=ZZSETP(0x00) && c<=ZZSETP(0x1F))      /* control key */
              c = ZZSETP(CTRL | (c+'@'));
      return(META | c);
   }

   /* process CTLX prefix */
   if (c == ctlxc) {
      c = get1key();
           if (c>='a' && c<='z')      /* Force to upper */
                   c -= 0x20;
           if (c>=ZZSETP(0x00) && c<=ZZSETP(0x1F))      /* control key */
              c = ZZSETP(CTRL | (c+'@'));
      return(CTLX | c);
   }

   /* otherwise, just return it */
   return(c);
}


/* 
 * get a buffer name from the command line. Command completion means
 * that pressing a <TAB> will attempt to complete an unfinished buffer
 * name if it is unique.
 */
static char **intbuflist;
static intgetlist;

getbufnamelist(bufn, bufl, listing)
char *bufn;
char ***listing;
{
   BUFFER *tmpbp;
   int    listc;
   char **listv;
   
      for (listc = 0, tmpbp = bheadp; tmpbp != NULL;
                tmpbp = tmpbp->b_bufp, listc++)
          ;

      if ((listv = (char **)malloc((listc+1)*sizeof(char *))) == (char **) NULL)
         return;

      for (listc = 0, tmpbp = bheadp; tmpbp != NULL; tmpbp = tmpbp->b_bufp) {
         if ((tmpbp->b_flag&BFINVS)==0 &&
               strncmp(bufn, tmpbp->b_bname, bufl)==0) {
            listv[listc++] = tmpbp->b_bname;
         }
      }

      listv[listc] = (char *) NULL;
      intbuflist = listv;

   *listing = intbuflist;
}


mlbufreply(prompt, buf, nbuf)
char *prompt;
char *buf;
{
   int res;
   
   completion = (char **) NULL;
   res = mlmatchreply(prompt, buf, nbuf, getbufnamelist);
   if (intbuflist != (char **)NULL) {
      free((char *)intbuflist);
      intbuflist = (char **) NULL;
   }
   return (res);
}

/* filename completion --> look in fileio.c */



handlecompletion(buf, nbuf, curpos, endpos, func)
char *buf;
int *curpos, *endpos;
PFI func;
{
   char **list, **clist, **llist;
   char *sp;
   int index = 0;
   /* attempt a completion */
   *curpos = *endpos;
   (*term.t_move)(term.t_nrow, ttcol = ttend);

   (*func)(buf, *endpos, &list);
   if (list == (char **)NULL) {
      (*term.t_beep)();
      (*term.t_flush)();
      return 0;
   }
   while (*list != NULL) {
      if (strncmp(buf, *list, *endpos) == 0) {
         /* a possible match! More than one? */
         if (*(list+1) == NULL ||
            (strncmp(buf, *(list+1), *endpos) != 0)) {
            /* no...we match, print it */
            sp = *list + *endpos;
            while (*sp && *endpos < nbuf && ttcol < term.t_ncol-3)
               inpctrlout(buf[(*endpos)++] = *sp++);

            if (completion != (char **) NULL) {
               if (completion[index] != (char *) NULL) {
                  sp = completion[index];
                  while (*sp && *endpos < nbuf && ttcol < term.t_ncol-3)
                     inpctrlout(buf[(*endpos)++] = *sp++);
               }
            }
            (*term.t_flush)();
            *curpos = *endpos;
            return 1; /* <------- ... */
         } else {
            /* try for a partial match against the list */
            /* first scan down until we no longer match the current input */
            llist = list + 1;
            while (*(llist+1) != NULL) {
               if (strncmp(buf, *(llist+1), *endpos) != 0)
                  break;
               ++llist;
            }

            /* and now, attempt a partial complete, char at a time */
            while (TRUE) {
               /* add the next char in */
               buf[*endpos] = (*list)[*endpos];

               /* scan through the candidates */
               clist = list + 1;
               while (clist <= llist) {
                  if ((*clist)[*endpos] != buf[*endpos]) {
                     /* show the user this is a partial ... */
                     (*term.t_putchar)('*');
                     (*term.t_flush)();
                     emaxdelay(FALSE);
                     (*term.t_putchar)('\b');
                     (*term.t_putchar)(' ');
                     (*term.t_putchar)('\b');
                     (*term.t_flush)();
                     goto onward2;
                  }
                  ++clist;
               }
               /* add the character */
               if (*endpos < nbuf && ttcol < term.t_ncol-3)
                  inpctrlout(buf[(*endpos)++]);
               else {
                  *curpos = *endpos;
                  return 1;
               }
            }
         }
      }
      ++list;
      ++index;
   }
   /* no match.....beep and onward */
   (*term.t_beep)();
onward2:;
   (*term.t_flush)();
   *curpos = *endpos;
   return 1;
}
