/* main.c
 * This program is in public domain; written by Dave G. Conroy.
 * This file contains the main driving routine, and some keyboard processing
 * code, for the MicroEMACS screen editor.
 *
 */

#include        <stdio.h>

/* make global definitions not external */
#define   maindef

#include   "estruct.h"        /* global structures and defines */
#include   "efunc.h"          /* function declarations and name table   */
#include   "edef.h"           /* global definitions */
#include   "ebind.h"          /* default key bindings */

#if     VMS
#include        <ssdef.h>
#define GOOD    (SS$_NORMAL)
#endif

#ifndef GOOD
#define GOOD    0
#endif

#if   APROF   /* Declarations needed for AZTEC C profiling */
int _Corg();   /* first address of program */
int _Cend();   /* last address of program */

short monbuf[NBUCK];   /* buffer for gather info */
#endif

#if PR1MOS
#include <keys>
#if !AMLCTRAN
fortran void quit_handler()
#endif
fortran void logout_handler();
fortran void mkon$p(), logo$$();
#endif

main(argc, argv)
char    *argv[];
{
        register int    c;
        register int    f;
        register int    n;
        register int    mflag;
        register BUFFER *bp;
        register int   ffile;   /* first file flag                      */
        register int   carg;    /* current arg to scan                  */
        register int   startf;  /* startup executed flag                */
        int basec;              /* c stripped of meta character         */
        int viewflag;           /* are we starting in view mode?        */
        int gotoflag;           /* do we need to goto a line at start?  */
        int gline;              /* if so, what line?                    */
        int searchflag;         /* Do we need to search at start?       */
        char bname[NBUFN];      /* buffer name of file to read          */
        char fname[NFILEN];
        char *indentcp;

#if   APROF
   /* if we are doing AZTEC C profiling, start it up */
   /*_intr_sp(18);    set clock interupt for 60/second */
   monitor(_Corg, _Cend, monbuf, NBUCK, 0);
#endif

   /* initialize the editor and process the command line arguments */
   strcpy(bname, "main");   /* default buffer name */
   vtinit();      /* Displays.            */
   edinit(bname);      /* Buffers, windows.    */
   viewflag = FALSE;   /* view mode defaults off in command line */
   gotoflag = FALSE;   /* set to off to begin with */
   searchflag = FALSE;   /* set to off to begin with */
   ffile = TRUE;      /* no file to edit yet */
   startf = FALSE;      /* startup file not executed yet */
#if   COLOR
   curwp->w_fcolor = gfcolor;      /* and set colors   */
   curwp->w_bcolor = gbcolor;
#endif

   /* scan through the command line and get the files to edit */
   for (carg = 1; carg < argc; ++carg) {
      /* if its a switch, process it */
      if (argv[carg][0] == '-') {
         switch (argv[carg][1]) {
            case 'v':   /* -v for View File */
            case 'V':
               viewflag = TRUE;
               break;
            case 'e':   /* -e for Edit file */
            case 'E':
               viewflag = FALSE;
               break;
            case 's':   /* -s for initial search string */
            case 'S':
               searchflag = TRUE;
               strcpy(pat,&argv[carg][2]);
               break;
            case 'g':   /* -g for initial goto */
            case 'G':
               gotoflag = TRUE;
               gline = atoi(&argv[carg][2]);
               break;
            default:   /* unknown switch */
               /* ignore this for now */
               break;
         }
      } else    /* check for a macro file */

#if PR1MOS
         if (argv[carg][0]== '#') {
#else
         if (argv[carg][0]== '@') {
#endif

         if (startup(&argv[carg][1]) == TRUE)
            startf = TRUE;   /* don't execute emacs.rc */

      } else {   /* process a file name */
         /* if we haven't run emacs.rc, do it now */
         if (startf == FALSE) {
            startup("");
            startf = TRUE;
         }

         /* set up a buffer for this file */
         strcpy(fname,argv[carg]);
#if FIXSUFF
        fix_file_suffix(fname);
#endif
         makename(bname, fname);

         /* if this is the first file, read it in */
         if (ffile) {
            bp = curbp;
            makename(bname, fname);
            strcpy(bp->b_bname, bname);
            strcpy(bp->b_fname, fname);
            if (readin(fname, (viewflag==FALSE))
                        == ABORT) {
               strcpy(bp->b_bname, "main");
               strcpy(bp->b_fname, "");
            }
            bp->b_dotp = bp->b_linep;
            bp->b_doto = 0;
            ffile = FALSE;
         } else {
            /* set this to inactive */
            bp = bfind(bname, TRUE, 0);
            strcpy(bp->b_fname, fname);
            bp->b_active = FALSE;
         }

         /* set the view mode appropriatly */
         if (viewflag)
            bp->b_mode |= MDVIEW;
      }
   }

   /* if invoked with nothing, run the startup file here */
   if (startf == FALSE) {
      startup("");
      startf = TRUE;
   }

        /* Deal with startup gotos and searches */

        if (gotoflag && searchflag) {
           update(FALSE);
      mlwrite("[Can not search and goto at the same time!]");
   }
        else if (gotoflag) {
                if (gotoline(TRUE,gline) == FALSE) {
                   update(FALSE);
         mlwrite("[Bogus goto argument]");
      }
        } else if (searchflag) {
                if (forscan(&pat[0], 2) == FALSE) {
                   update(FALSE);
         mlwrite("Not found.");
      }
        }

   /* setup to process commands */
        lastflag = 0;                           /* Fake last flags.     */
   curbp->b_mode |= gmode;       /* and set default modes*/
   curwp->w_flag |= WFMODE;      /* and force an update   */
#if   COLOR
   curwp->w_fcolor = gfcolor;      /* and set colors   */
   curwp->w_bcolor = gbcolor;
#endif

#if PR1MOS
#if !AMLCTRAN
        mkon$p("QUIT$",     5, quit_handler);
#endif
        mkon$p("LOGOUT$",   7, logout_handler);
#endif

        if (transtab)
                tabsize = 0;

/* Get the c-mode indentation rules */
#if   (MSDOS & (LATTICE | AZTEC | MSC)) | V7 | USG | BSD
   if ((indentcp = getenv("OPENINDENT")) != NULL)
        startindent = strdup(indentcp);
   if ((indentcp = getenv("CLOSEINDENT")) != NULL)
        endindent = strdup(indentcp);
#endif
#if  PR1MOS
   if ((indentcp = gvget(".OPENINDENT")) != NULL)
        startindent = strdup(indentcp);
   if ((indentcp = gvget(".CLOSEINDENT")) != NULL)
        endindent = strdup(indentcp);
#endif

loop:
/* And now folks, we're finally there in the main loop of MicroEMACS */

        update(FALSE);                          /* Fix up the screen    */
        c = getcmd();
        if (mpresf != FALSE) {
                mlerase();
                update(FALSE);
#if   CLRMSG
                if (c == ' ')                   /* ITS EMACS does this  */
                        goto loop;
#endif
        }
        f = FALSE;
        n = 1;

   /* do META-# processing if needed */

   basec = c & ~META;      /* strip meta char off if there */
   if ((c & META) && ((basec >= '0' && basec <= '9') || basec == '-')) {
      f = TRUE;      /* there is a # arg */
      n = 0;         /* start with a zero default */
      mflag = 1;      /* current minus flag */
      c = basec;      /* strip the META */
      while ((c >= '0' && c <= '9') || (c == '-')) {
         if (c == '-') {
            /* already hit a minus or digit? */
            if ((mflag == -1) || (n != 0))
               break;
            mflag = -1;
         } else {
            n = n * 10 + (c - '0');
         }
         if ((n == 0) && (mflag == -1))   /* lonely - */
            mlwrite("Arg:");
         else
            mlwrite("Arg: %d",n * mflag);

         c = getcmd();   /* get the next key */
      }
      n = n * mflag;   /* figure in the sign */
   }

   /* do ^U repeat argument processing */

        if (c == (CTRL|'U')) {                  /* ^U, start argument   */
                f = TRUE;
                n = 4;                          /* with argument of 4 */
                mflag = 0;                      /* that can be discarded. */
                mlwrite("Arg: 4");
                while ((c=getcmd()) >='0' && c<='9' || c==(CTRL|'U') || c=='-'){
                        if (c == (CTRL|'U'))
                                n = n*4;
                        /*
                         * If dash, and start of argument string, set arg.
                         * to -1.  Otherwise, insert it.
                         */
                        else if (c == '-') {
                                if (mflag)
                                        break;
                                n = 0;
                                mflag = -1;
                        }
                        /*
                         * If first digit entered, replace previous argument
                         * with digit and set sign.  Otherwise, append to arg.
                         */
                        else {
                                if (!mflag) {
                                        n = 0;
                                        mflag = 1;
                                }
                                n = 10*n + c - '0';
                        }
                        mlwrite("Arg: %d", (mflag >=0) ? n : (n ? -n : -1));
                }
                /*
                 * Make arguments preceded by a minus sign negative and change
                 * the special argument "^U -" to an effective "^U -1".
                 */
                if (mflag == -1) {
                        if (n == 0)
                                n++;
                        n = -n;
                }
        }

        if (kbdmip != NULL) {                   /* Save macro strokes.  */
                if (c!=(CTLX|')') && kbdmip>&kbdm[NKBDM-6]) {
                        ctrlg(FALSE, 0);
                        goto loop;
                }
                if (f != FALSE) {
                        *kbdmip++ = (CTRL|'U');
                        *kbdmip++ = n;
                }
                *kbdmip++ = c;
        }
        rpt_lev = 0;
        rpt_save = TRUE;
        execute( c, f, n );
        first_flag = FALSE;
        goto loop;
}

/*
 * Initialize all of the buffers and windows. The buffer name is passed down
 * as an argument, because the main routine may have been told to read in a
 * file by default, and we want the buffer name to be right.
 */
edinit(bname)
char    bname[];
{
        register BUFFER *bp;
        register WINDOW *wp;
   char *malloc();

        bp = bfind(bname, TRUE, 0);             /* First buffer         */
        blistp = bfind("[List]", TRUE, BFINVS); /* Buffer list buffer   */
        wp = (WINDOW *) malloc(sizeof(WINDOW)); /* First window         */
        if (bp==NULL || wp==NULL || blistp==NULL)
                exit(1);
        curbp  = bp;                            /* Make this current    */
        wheadp = wp;
        curwp  = wp;
        wp->w_wndp  = NULL;                     /* Initialize window    */
        wp->w_bufp  = bp;
        bp->b_nwnd  = 1;                        /* Displayed.           */
        wp->w_linep = bp->b_linep;
        wp->w_dotp  = bp->b_linep;
        wp->w_doto  = 0;
        wp->w_markp = NULL;
        wp->w_marko = 0;
        wp->w_toprow = 0;
#if   COLOR
   /* initalize colors to global defaults */
   wp->w_fcolor = gfcolor;
   wp->w_bcolor = gbcolor;
#endif
        wp->w_ntrows = term.t_nrow-1;           /* "-1" for mode line.  */
        wp->w_force = 0;
        wp->w_flag  = WFMODE|WFHARD;            /* Full.                */
}

/*
 * This is the general command execution routine. It handles the fake binding
 * of all the keys to "self-insert". It also clears out the "thisflag" word,
 * and arranges to move it to the "lastflag", so that the next command can
 * look at it. Return the status of command.
 */
execute(c, f, n)
{
        register KEYTAB *ktp;
        register int    status;


        ktp = &keytab[0];                       /* Look in key table.   */
        while (ktp->k_fp != NULL) {
                if (ktp->k_code == c) {
                        thisflag = 0;
                        status   = (*ktp->k_fp)(f, n);
                        lastflag = thisflag;
                        if (rpt_save) {
                                old_c = c;
                                old_f = f;
                                old_n = n;
                        }
                        return (status);
                }
                ++ktp;
        }
        if (rpt_save) {
                old_c = c;
                old_f = f;
                old_n = n;
        }

        /*
         * If a space was typed, fill column is defined, the argument is non-
         * negative, wrap mode is enabled, and we are now past fill column,
         * and we are not read-only, perform word wrap.
         */
        if (c == ' ' && (curwp->w_bufp->b_mode & MDWRAP) && fillcol > 0 &&
       n >= 0 && getccol(FALSE) > fillcol &&
       (curwp->w_bufp->b_mode & MDVIEW) == FALSE)
                wrapword();

        if ((c>=ZZSETP(0x20) && c<=ZZSETP(0x7E))    /* Self inserting.      */
        ||  (c>=ZZSETP(0xA0) && c<=ZZSETP(0xFE))) {
                if (n <= 0) {                   /* Fenceposts.          */
                        lastflag = 0;

                        return (n<0 ? FALSE : TRUE);
                }
                thisflag = 0;                   /* For the future.      */

      /* if we are in overwrite mode, not at eol,
         and next char is not a tab or we are at a tab stop,
         delete a char forword         */
      if (curwp->w_bufp->b_mode & MDOVER &&
          curwp->w_doto < curwp->w_dotp->l_used &&
         (lgetc(curwp->w_dotp, curwp->w_doto) != '\t' ||
          (curwp->w_doto) % 8 == 7))
            ldelete(1L, FALSE);

      /* do the appropriate insertion */
      if (c == '{' && (curbp->b_mode & MDCMOD))
              status = insstartbrace(n);
      else if (c == '}' && (curbp->b_mode & MDCMOD))
              status = insendbrace(n);
      else if (c == '#' && (curbp->b_mode & MDCMOD) != 0)
              status = inspound(n);
      else
              status = linsert(n, c);
#if   CFENCE
      /* check for CMODE fence matching */
      if ((c == '}' || c == ')' || c==']') &&
            (curbp->b_mode & MDCMOD || curbp->b_mode & MDLISP) != 0)
         fcmatch(c, 1, 0); /* character to match, move first, do not jump */
#endif

                lastflag = thisflag;
                return (status);
        }
        (*term.t_beep)();
        mlwrite("[Key not bound]");      /* complain      */
        lastflag = 0;                           /* Fake last flags.     */

        return (FALSE);
}

/* reexecute */
reexecute(f, n)
{
        register int s;

        rpt_lev++;
        rpt_save = FALSE;
        if (first_flag || rpt_lev>1) {
                mlwrite("[Reexecute error]");
                return (FALSE);
        }
        rpt_flag = TRUE;
        s = TRUE;
        if (f != TRUE)
                s = execute(old_c, old_f, old_n);
        else
                while (n-- > 0 && (s=execute(old_c, old_f, old_n))==TRUE)
                                rpt_flag = TRUE;

        rpt_flag = FALSE;
        rpt_save = FALSE;
        return (s);
}

/*
 * Fancy quit command, as implemented by Norm. If the any buffer has
 * changed do a write on that buffer and exit emacs, otherwise simply exit.
 */
quickexit(f, n)
{
   register BUFFER *bp;   /* scanning pointer to buffers */

   bp = bheadp;
   while (bp != NULL) {
           if ((bp->b_flag&BFCHG) != 0   /* Changed.             */
           && (bp->b_flag&BFINVS) == 0) {   /* Real.                */
         curbp = bp;      /* make that buffer cur   */
         mlwrite("[Saving %s]",bp->b_fname);
                   filesave(f, n);
      }
   bp = bp->b_bufp;         /* on to the next buffer */
   }
   upmode();
   quit(f, n);                          /* conditionally quit   */
}

/*
 * Quit command. If an argument, always quit. Otherwise confirm if a buffer
 * has been changed and not written out. Normally bound to "C-X C-C".
 */
quit(f, n)
{
        register int    s;

        update(TRUE);
        if (f != FALSE                          /* Argument forces it.  */
        || anycb() == FALSE                     /* All buffers clean.   */
                  /* User says it's OK.   */
        || (s=mlyesno("Modified buffers exist. Leave anyway")) == TRUE) {
#if   FILOCK
      if (lockrel() != TRUE) {
         (*term.t_putchar)('\n');
         (*term.t_putchar)('\r');
         (*term.t_close)();
         exit(1);
      }
#endif
                vttidy();
#if   APROF
      /* if doing AZTEC C profiling, close up and write it out */
      monitor(0,0,0,0,0);
#endif
                exit(GOOD);
        }
        mlwrite("");
        return (s);
}

/*
 * Begin a keyboard macro.
 * Error if not at the top level in keyboard processing. Set up variables and
 * return.
 */
ctlxlp(f, n)
{
        if (kbdmip!=NULL || kbdmop!=NULL) {
                mlwrite("Not now");
                return (FALSE);
        }
        mlwrite("[Start macro]");
        kbdmip = &kbdm[0];
        return (TRUE);
}

/*
 * End keyboard macro. Check for the same limit conditions as the above
 * routine. Set up the variables and return to the caller.
 */
ctlxrp(f, n)
{
        if (kbdmip == NULL) {
                mlwrite("Not now");
                return (FALSE);
        }
        mlwrite("[End macro]");
        kbdmip = NULL;
        return (TRUE);
}

/*
 * Execute a macro.
 * The command argument is the number of times to loop. Quit as soon as a
 * command gets an error. Return TRUE if all ok, else FALSE.
 */
ctlxe(f, n)
{
        register int    c;
        register int    af;
        register int    an;
        register int    s;

        if (kbdmip!=NULL || kbdmop!=NULL) {
                mlwrite("[Not now]");
                return (FALSE);
        }
        if (n <= 0)
                return (TRUE);
        /* Don't make routines under this level believe they are reexecuted */
        rpt_flag = FALSE;
        do {
                kbdmop = &kbdm[0];
                do {
                        af = FALSE;
                        an = 1;
                        if ((c = *kbdmop++) == (CTRL|'U')) {
                                af = TRUE;
                                an = *kbdmop++;
                                c  = *kbdmop++;
                        }
                        s = TRUE;
                        rpt_lev = 0;
                        rpt_save = TRUE;
                } while (c!=(CTLX|')') && (s=execute(c, af, an))==TRUE);
                kbdmop = NULL;
        } while (s==TRUE && --n);
        return (s);
}

/*
 * Abort.
 * Beep the beeper. Kill off any keyboard macro, etc., that is in progress.
 * Sometimes called as a routine, to do general aborting of stuff.
 */
ctrlg(f, n)
{
        (*term.t_beep)();
        if (kbdmip != NULL) {
                kbdm[0] = (CTLX|')');
                kbdmip  = NULL;
        }
   mlwrite("[Aborted]");
        return (ABORT);
}

/* tell the user that this command is illegal while we are in
   VIEW (read-only) mode            */

rdonly()

{
   (*term.t_beep)();
   mlwrite("[Key illegal in VIEW mode]");
   return(FALSE);
}
int global_meta_flag = 0;
meta()   /* dummy function for binding to meta prefix */
{
        global_meta_flag = 1;
        mlwrite("[Meta prefix key]");
}

cex()   /* dummy function for binding to control-x prefix */
{
        mlwrite("[Control-X prefix key]");
}

#if FUNCFIX
/* Special function prefix characters fix ...... */
int GlobalFuncFlag=0;
funcprefix(f,n)
{
   GlobalFuncFlag=1;
}
#endif

#if PR1MOS

#if !AMLCTRAN
fortran void quit_handler(cp)
int *cp;
{
        (*term.t_beep)();
        spawncli(1,1);
}
#endif

fortran void logout_handler(cp)
int *cp;
{
   BUFFER *bp;   /* scanning pointer to buffers */
   int f,n;
   
   mlwrite("[Logout has been signalled]");
   (*term.t_beep)();
   bp = bheadp;
   while (bp != NULL) {
           if ((bp->b_flag&BFCHG) != 0   /* Changed.             */
           && (bp->b_flag&BFINVS) == 0) {   /* Real.                */
         curbp = bp;      /* make that buffer cur   */
         strcat(bp->b_fname,".sav");
         mlwrite("[Logout in progress, saving %s]",bp->b_fname);
                   filesave(f, n);
      }
   bp = bp->b_bufp;         /* on to the next buffer */
   }
   upmode();
   update(TRUE);
   vttidy();
   logo$$(0,0,0,0,0,0);
}
#endif
