#include "estruct.h"

#if PR1MOS
/*
** MEMORY.C - This file contains memory allocation routines that can be used
**            instead of those found in the CC_LIBRARY library.
**
** Date     Programmer     History
** -------- -------------- ----------------------------------------------------
** 880603   P. Eriksson    Template constructed.
*/

#include <stdio.h>
#include <errd>
#include "memory.h"


typedef unsigned long shortptr;

typedef union
{
   shortptr  s;
   void     *l;
} pointer;


/*
** Allocate a block of memory.
*/
void *malloc(size)
size_t size;
{
   fortran shortptr STR$AL(); /* Allocate user class storage                  */
   fortran void STR$FR();     /* Free previously allocated user class storage */
   pointer block;
   short status;


   if (size < 1)
   {
      errno = e$alsz;
      return NULL;
   }

   block.s = STR$AL(0, size / 2 + 1 + sizeof(size_t) / 2, 0, status);

   if (errno = status)
   {
      if (status == e$room)
      {
         perror("STR$AL");
         STR$FR(0, block.s, status);
         errno = status;
      }
      return NULL;
   }
   else
   {
      *( ((size_t *) block.l)++ ) = size;
      return block.l;
   }
}



/*
** Allocate a number of objects of a specified size.
*/
void *calloc(size, object_size)
size_t size, object_size;
{
   void *ptr;

   ptr = malloc(size * object_size);      /* Allocate storage        */
   memset(ptr, 0, size * object_size);    /* Zero the area           */
}



/*
** Free a previously allocated block.
*/
free(block)
void *block;
{
   fortran void STR$FR();
   short status;


   STR$FR(0, --((size_t *) block), status);
   if (errno = status)
      return -1;
   else
      return 0;
}



/*
** Free a previously allocated block.
*/
cfree(block)
void *block;
{
   return free(block);
}



/*
** This routine is used by the CC_LIBRARY internally. This routine is used
** in conjunction with the BIO$ALLOC routine. I however doesn't have to
** rewrite this routine since it calls calloc() and I've rewritten those...
*/
bio$free(block)
void *block;
{
   return free(block);
}



/*
** Change a previously allocated block's size. If the new block size is
** smaller than the old size then nothing is done (time saving...)
*/
void *realloc(oldblock, newsize)
void *oldblock;
size_t newsize;
{
   int oldsize;
   void *newblock;

   if (newsize < 1)
   {
      errno = e$alsz;
      return NULL;
   }

   if (newsize <= *(((size_t *) oldblock) - 1))
      return oldblock;

   if ((newblock = malloc(newsize)) == NULL)
      return NULL;

   memcpy(newblock, oldblock, oldsize);
   memset(((char *) newblock) + oldsize, 0, newsize - oldsize);
   return newblock;
}
#endif
