/* random.c
 * This file contains the command processing functions for a number of random
 * commands. There is no functional grouping here, for sure.
 */

#include        <stdio.h>
#include        "estruct.h"
#include        "edef.h"

#if PR1MOS
fortran void sleep$();
#endif

/*
 * Set fill column to n.
 */
setfillcol(f, n)
{
        char number[21];
        int retcode;
      
        /* Have we used any argument? */
        if (!f)  {
                /* Prompt for the line number */
                if (retcode = mlreply("Fill column: ", number, 20) != TRUE) {
                        mlwrite("");
                        return (retcode);
                }
                n = atoi(number);
        }

        fillcol = n;
        mlwrite("[Fill column is %d]",n);
        return(TRUE);
}

/*
 * Display the current position of the cursor, in origin 1 X-Y coordinates,
 * the character that is under the cursor (in hex), and the fraction of the
 * text that is before the cursor. The displayed column is not the current
 * column, but the column that would be used on an infinite width display.
 * Normally this is bound to "C-X =".
 */
showcpos(f, n)
{
        register LINE   *lp;      /* current line */
        register long   numchars;   /* # of chars in file */
        register int   numlines;   /* # of lines in file */
        register long   predchars;   /* # chars preceding point */
        register int   predlines;   /* # lines preceding point */
        register int    curchar;   /* character under cursor */
        int ratio;
        int col;
   int savepos;         /* temp save for current offset */
   int ecol;         /* column pos/end of current line */

   /* starting at the beginning of the buffer */
        lp = lforw(curbp->b_linep);

   /* start counting chars and lines */
        numchars = 0;
        numlines = 0;
        while (lp != curbp->b_linep) {
      /* if we are on the current line, record it */
      if (lp == curwp->w_dotp) {
         predlines = numlines;
         predchars = numchars + curwp->w_doto;
         if ((curwp->w_doto) == llength(lp))
            curchar = '\n';
         else
            curchar = lgetc(lp, curwp->w_doto);
      }
      /* on to the next line */
      ++numlines;
      numchars += llength(lp) + 1;
      lp = lforw(lp);
        }

   /* if at end of file, record it */
   if (curwp->w_dotp == curbp->b_linep) {
      predlines = numlines;
      predchars = numchars;
   }

   /* Get real column and end-of-line column. */
   col = getccol(FALSE);
   savepos = curwp->w_doto;
   curwp->w_doto = llength(curwp->w_dotp);
   ecol = getccol(FALSE);
   curwp->w_doto = savepos;

        ratio = 0;              /* Ratio before dot. */
        if (numchars != 0)
                ratio = (100L*predchars) / numchars;

   /* summarize and report the info */
   mlwrite("Line %d/%d Col %d/%d Char %D/%D (%d%%) char = 0x%x",
      predlines+1, numlines+1, col, ecol,
      predchars, numchars, ratio, curchar);
        return (TRUE);
}


/*
 * Twiddle the two characters on either side of dot. If dot is at the end of
 * the line twiddle the two characters before it. Return with an error if dot
 * is at the beginning of line; it seems to be a bit pointless to make this
 * work. This fixes up a very common typo with a single stroke. Normally bound
 * to "C-T". This always works within a line, so "WFEDIT" is good enough.
 */
twiddle(f, n)
{
        register LINE   *dotp;
        register int    doto;
        register int    cl;
        register int    cr;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        dotp = curwp->w_dotp;
        doto = curwp->w_doto;
        if (doto==llength(dotp) && --doto<0)
                return (FALSE);
        cr = lgetc(dotp, doto);
        if (--doto < 0)
                return (FALSE);
        cl = lgetc(dotp, doto);
        lputc(dotp, doto+0, cr);
        lputc(dotp, doto+1, cl);
        lchange(WFEDIT);
        return (TRUE);
}

/*
 * Quote the next character, and insert it into the buffer. All the characters
 * are taken literally, with the exception of the newline, which always has
 * its line splitting meaning. The character is always read, even if it is
 * inserted 0 times, for regularity. Bound to "C-Q"
 */
quote(f, n)
{
        register int    s;
        register int    c;

        if (curbp->b_mode&MDVIEW)   /* don't allow this command if      */
                return(rdonly());   /* we are in read only mode */
        mlwrite("[Quote]");
        c = (*term.t_getchar)();
        mlerase ();

        if (n < 0)
                return (FALSE);
        if (n == 0)
                return (TRUE);
        if (c == '\n') {
                do {
                        s = lnewline();
                } while (s==TRUE && --n);
                return (s);
        }
        return (linsert(n, c));
}

/*
 * Open up some blank space. The basic plan is to insert a bunch of newlines,
 * and then back up over them. Everything is done by the subcommand
 * procerssors. They even handle the looping. Normally this is bound to "C-O".
 */
openline(f, n)
{
        register int    i;
        register int    s;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        if (n < 0)
                return (FALSE);
        if (n == 0)
                return (TRUE);
        i = n;                                  /* Insert newlines.     */
        do {
                s = lnewline();
        } while (s==TRUE && --i);
        if (s == TRUE)                          /* Then back up overtop */
                s = backchar(f, n);             /* of them all.         */
        return (s);
}

/*
 * Insert a newline. Bound to "C-M". If we are in CMODE or INDENT, do automatic
 * indentation as specified (if in CSMODE destroy the beauty of the code).
 */
newline(f, n)
{
   register int    s;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
   if (n < 0)
      return (FALSE);

   /* if we are in C mode and this is a default <NL> */
   if (n == 1 && curwp->w_dotp != curbp->b_linep) {
      if (curbp->b_mode & MDINDENT || curbp->b_mode & MDCMOD)
         return(insertnewline());
   }

   /* insert some lines */
   while (n--) {
      if ((s=lnewline()) != TRUE)
         return (s);
   }
   return (TRUE);
}


/*
 * Delete blank lines around dot. What this command does depends if dot is
 * sitting on a blank line. If dot is sitting on a blank line, this command
 * deletes all the blank lines above and below the current line. If it is
 * sitting on a non blank line then it deletes all of the blank lines after
 * the line. Normally this command is bound to "C-X C-O". Any argument is
 * ignored.
 */
deblank(f, n)
{
        register LINE   *lp1;
        register LINE   *lp2;
        long nld;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        lp1 = curwp->w_dotp;
        while (llength(lp1)==0 && (lp2=lback(lp1))!=curbp->b_linep)
                lp1 = lp2;
        lp2 = lp1;
        nld = 0;
        while ((lp2=lforw(lp2))!=curbp->b_linep && llength(lp2)==0)
                ++nld;
        if (nld == 0)
                return (TRUE);
        curwp->w_dotp = lforw(lp1);
        curwp->w_doto = 0;
        return (ldelete(nld, FALSE));
}

/*
 * Insert a newline, then enough tabs and spaces to duplicate the indentation
 * of the previous line. Assumes tabs are every eight characters. Quite simple.
 * Figure out the indentation of the current line. Insert a newline by calling
 * the standard routine. Insert the indentation by inserting the right number
 * of tabs and spaces. Return TRUE if all ok. Return FALSE if one of the
 * subcomands failed. Normally bound to "C-J".
 */
indent(f, n)
{
        register int    nicol;
        register int    c;
        register int    i;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        if (n < 0)
                return (FALSE);
        while (n--) {
                nicol = 0;
                for (i=0; i<llength(curwp->w_dotp); ++i) {
                        c = lgetc(curwp->w_dotp, i);
                        if (c!=' ' && c!='\t')
                                break;
                        if (c == '\t')
                                nicol |= 0x07;
                        ++nicol;
                }
                if (i+1 == llength(curwp->w_dotp))
                        ++nicol;
                if (lnewline() == FALSE
                || ((i=nicol/8)!=0 && linsert(i, '\t')==FALSE)
                || ((i=nicol%8)!=0 && linsert(i,  ' ')==FALSE))
                        return (FALSE);
        }
        return (TRUE);
}

/*
 * Go to beginning of next line. Dead easy!
 * Now bound to "C-J"     ///// dnm
 */
begnextline(f,n)
{
   forwline(f,n);     /* no error code (only TRUE) */
   gotobol(f,n);      /* same thing */
   return (TRUE);
}

/*
 * Delete forward. This is real easy, because the basic delete routine does
 * all of the work. Watches for negative arguments, and does the right thing.
 * If any argument is present, it kills rather than deletes, to prevent loss
 * of text if typed with a big argument. Normally bound to "C-D".
 */
forwdel(f, n)
{
   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        if (n < 0)
                return (backdel(f, -n));
        if (f != FALSE) {                       /* Really a kill.       */
                if ((lastflag&CFKILL) == 0)
                        kdelete();
                thisflag |= CFKILL;
        }
        return (ldelete((long)n, f));
}

/*
 * Delete backwards. This is quite easy too, because it's all done with other
 * functions. Just move the cursor back, and delete forwards. Like delete
 * forward, this actually does a kill if presented with an argument. Bound to
 * both "RUBOUT" and "C-H".
 */
backdel(f, n)
{
        register int    s;

        if (curbp->b_mode&MDVIEW)   /* don't allow this command if      */
                return(rdonly());   /* we are in read only mode         */

        if (n < 0)
                return (forwdel(f, -n));

        if (f != FALSE) {                       /* Really a kill.       */
                if ((lastflag&CFKILL) == 0)
                        kdelete();
                thisflag |= CFKILL;
        }
        if ((s=backchar(f, n)) == TRUE)
                s = ldelete((long)n, f);
        /* P. Eriksson */
        if (curbp->b_mode&MDOVER && s) {
                s = linsert(n,' ');
                if (s)
                s = backchar(f, n);
        }

        return (s);
}

/*
 * Kill text. If called without an argument, it kills from dot to the end of
 * the line, unless it is at the end of the line, when it kills the newline.
 * If called with an argument of 0, it kills from the start of the line to dot.
 * If called with a positive argument, it kills from dot forward over that
 * number of newlines. If called with a negative argument it kills backwards
 * that number of newlines. Normally bound to "C-K".
 */
killtext(f, n)
{
        register LINE   *nextp;
        long chunk;

        if (curbp->b_mode&MDVIEW)       /* don't allow this command if*/
                return(rdonly());       /* we are in read only mode*/
        if ((lastflag&CFKILL) == 0)     /* Clear kill buffer if */
                kdelete();              /* last wasn't a kill.  */
        thisflag |= CFKILL;
        if (f == FALSE) {
                chunk = llength(curwp->w_dotp)-curwp->w_doto;
                if (chunk == 0)
                        chunk = 1;
        } else if (n == 0) {
                chunk = curwp->w_doto;
                curwp->w_doto = 0;
        } else if (n > 0) {
                chunk = llength(curwp->w_dotp)-curwp->w_doto+1;
                nextp = lforw(curwp->w_dotp);
                while (--n) {
                        if (nextp == curbp->b_linep)
                                return (FALSE);
                        chunk += llength(nextp)+1;
                        nextp = lforw(nextp);
                }
        } else {
                mlwrite("negative kill");
                return (FALSE);
        }
        return(ldelete(chunk, TRUE));
}

setmode(f, n)   /* prompt and set an editor mode */

int f, n;   /* default and argument */

{
   adjustmode(TRUE, FALSE);
}

delmode(f, n)   /* prompt and delete an editor mode */

int f, n;   /* default and argument */

{
   adjustmode(FALSE, FALSE);
}

setgmode(f, n)   /* prompt and set a global editor mode */

int f, n;   /* default and argument */

{
   adjustmode(TRUE, TRUE);
}

delgmode(f, n)   /* prompt and delete a global editor mode */

int f, n;   /* default and argument */

{
   adjustmode(FALSE, TRUE);
}

adjustmode(kind, global)   /* change the editor mode status */

int kind;   /* true = set,      false = delete */
int global;   /* true = global flag,   false = current buffer flag */
{
   register char *scan;      /* scanning pointer to convert prompt */
   register int i;         /* loop index */
#if   COLOR
   register int uflag;      /* was modename uppercase?   */
#endif
   char prompt[50];   /* string to prompt user with */
   char cbuf[NPAT];      /* buffer to recieve mode name into */

   /* build the proper prompt string */
   if (global)
      strcpy(prompt,"Global mode to ");
   else
      strcpy(prompt,"Mode to ");

   if (kind == TRUE)
      strcat(prompt, "add: ");
   else
      strcat(prompt, "delete: ");

   /* prompt the user and get an answer */

   if (mlreply(prompt, cbuf, NPAT - 1)!=TRUE)
      return (ABORT);

   /* make it uppercase */

   scan = cbuf;
#if   COLOR
   uflag = (*scan >= 'A' && *scan <= 'Z');
#endif
   while (*scan != 0) {
      if (*scan >= 'a' && *scan <= 'z')
         *scan = *scan - 32;
      scan++;
   }

   /* test it first against the colors we know */
   for (i=0; i<NCOLORS; i++) {
      if (strcmp(cbuf, cname[i]) == 0) {
         /* finding the match, we set the color */
#if   COLOR
         if (uflag)
            if (global)
               gfcolor = i;
            else
               curwp->w_fcolor = i;
         else
            if (global)
               gbcolor = i;
            else
               curwp->w_bcolor = i;

         curwp->w_flag |= WFCOLR;
#endif
         mlerase();
         return(TRUE);
      }
   }

   /* test it against the modes we know */

   for (i=0; i < NUMMODES; i++) {
      if (strcmp(cbuf, modename[i]) == 0) {
         /* finding a match, we process it */
         if (kind == TRUE)
            if (global)
               gmode |= (1 << i);
            else
               curwp->w_bufp->b_mode |= (1 << i);
         else
            if (global)
               gmode &= ~(1 << i);
            else
               curwp->w_bufp->b_mode &= ~(1 << i);
         /* display new mode line */
         if (global == 0)
            upmode();
         mlerase();   /* erase the junk */
         return(TRUE);
      }
   }

   mlwrite("No such mode!");
   return(FALSE);
}

/* OVERINS - This function toggles the OVERLAY/INSERT mode switch */
/*           Intended for use with a single key to toggle it      */
overins(f, n)
{
   curwp->w_bufp->b_mode ^= MDOVER;
   upmode();
   return(TRUE);
}

/*   This function simply clears the message line,
      mainly for macro usage         */

clrmes(f, n)

int f, n;   /* arguments ignored */

{
   mlwrite("");
   return(TRUE);
}

/*   This function writes a string on the message line
      mainly for macro usage         */

writemsg(f, n)

int f, n;   /* arguments ignored */

{
   register char *sp;   /* pointer into buf to expand %s */
   register char *np;   /* ptr into nbuf */
   register int status;
   char buf[NPAT];      /* buffer to recieve mode name into */
   char nbuf[NPAT*2];   /* buffer to expand string into */

   if ((status = mlreply("Message to write: ", buf, NPAT - 1)) != TRUE)
      return(status);

   /* expand all '%' to "%%" so mlwrite won't expect arguments */
   sp = buf;
   np = nbuf;
   while (*sp) {
      *np++ = *sp;
      if (*sp++ == '%')
         *np++ = '%';
   }
   *np = '\0';
   mlwrite(nbuf);
   return(TRUE);
}

/*
**   Close fences are matched against their partners, and if
**   on screen the cursor briefly lights there, otherwise
**   match will show on the message line
*/

fencematch(f,n)   
int f, n;   /* arguments ignored */
{
   return (fcmatch(currchar(), 0, 0));
}

fencejump(f,n)   
int f, n;   /* arguments ignored */
{
   return (fcmatch(currchar(), 0, 1));
}


static char intbuf[NLINE];

fcmatch(ch, moveit, jump)

   char ch;                      /* current character in scan */
   int moveit;                   /* Move before scan */
   int jump;                     /* jump to match ? */

{
   register LINE *oldlp;         /* original line pointer */
   register int oldoff;          /* and offset */
   register LINE *toplp, *botlp; /* top & bottom line in current window */
   register int count;           /* current fence level count */
   register char opench;         /* open fence */
   register char c;              /* fence type to match against */
   register int i;
   int step;                     /* used to determine forw/back-char */
   int outofwin;                 /* 0 = display in window, 1 = use mlwrite */
   int index, len;
   LINE   *lp;                   /* line counter */
   int   numlines;               /* get where match is */

   /* first get the display updated out there */
   update(FALSE);

   /* save the original cursor position */
   oldlp = curwp->w_dotp;
   oldoff = curwp->w_doto;

   /* setup proper open fence for passed close fence */
   switch (ch) {
      case ')': opench = '('; step = 1; break;
      case ']': opench = '['; step = 1; break;
      case '}': opench = '{'; step = 1; break;

      case '(': opench = ')'; step = -1; break;
      case '[': opench = ']'; step = -1; break;
      case '{': opench = '}'; step = -1; break;

      default:
         mlwrite("[Not a valid character for matching]");
         return (jump?FALSE:TRUE); /* don't break macros */
   }


   /* find the top and bottom line + set up for scan */
   toplp = curwp->w_linep->l_bp;

   botlp = toplp;
   index = 0;
   while (index <= curwp->w_ntrows) {
      botlp = lforw(botlp);
      index++;
   }
      


   count = 1;
   outofwin = 0;

   /* if true this function is called from CFENCE section in main */
   if (moveit) 
      backchar(FALSE, step); /* backchar doesn't use it's first argument */

   /* scan back until we find it, or reach past the top of the window */
   while (count > 0 && backchar(FALSE,step) == TRUE) {
      if (curwp->w_dotp == toplp || curwp->w_dotp == botlp)
         outofwin = 1;
      if (curwp->w_doto >= llength(curwp->w_dotp))
         continue;
      c = lgetc(curwp->w_dotp, curwp->w_doto);
      if (c == ch)
         ++count;
      if (c == opench)
         --count;
   }

   /* if count is zero, we have a match, display the sucker */
   /* there is a real machine dependent timing problem here we have
      yet to solve......... */
   if (count == 0) {
      if (jump) {
         /* skip restore position */
         return(TRUE);
      }
      if (outofwin) {

         /* starting at the beginning of the buffer */
         lp = lforw(curbp->b_linep);

         /* start counting lines */
         numlines = 1;
         while (lp != curbp->b_linep && lp != curwp->w_dotp) {
            ++numlines;
            lp = lforw(lp);
         } /* yeah, yeah, I could do this piece of code as "for" */

         strcpy(intbuf, "c match line %d:");
         intbuf[0] = ch;

         for (len=index=strlen(intbuf);
            index-len < llength(curwp->w_dotp) && index < term.t_ncol-1;
            index++)
            intbuf[index] = lgetc(curwp->w_dotp, index-len);              

         intbuf[index] = '\0';
         mlwrite(intbuf, numlines);
      }
      else {
         emaxdelay(TRUE);
      }
   } else
      mlwrite("[No match found]");

   /* restore the current position */
   curwp->w_dotp = oldlp;
   curwp->w_doto = oldoff;
   /* discard any flags set by backchar/forwchar */
   curwp->w_flag &= ~(unsigned char)WFMOVE;
   return((jump && count)? FALSE : TRUE);
}

istring(f, n)   /* ask for and insert a string into the current
         buffer at the current point */

int f, n;   /* ignored arguments */

{
   register char *tp;   /* pointer into string to add */
   register int status;   /* status return code */
   char tstring[NPAT+1];   /* string to add */

   /* ask for string to insert */
   status = mlreply("String to insert: ", tstring, NPAT);
   if (status != TRUE)
      return(status);

   /* insert it */
   tp = &tstring[0];
   while (*tp) {
      if (*tp == 0x0a)
         status = lnewline();
      else
         status = linsert(1, *tp);
      ++tp;
      if (status != TRUE)
         return(status);
   }
   return(TRUE);
}

/* Delete spaces around cursor */
decblank (f, n)
{
   int   s;

#define ISWHITE(x) (x==' ' || x=='\t' || x=='\n')

   if (! ISWHITE(lgetc(curwp->w_dotp, curwp->w_doto )))
      return (TRUE);

   /* Delete backward */

   s = backchar(FALSE, 1);
   while (s && ISWHITE( lgetc(curwp->w_dotp, curwp->w_doto ) ) ) {
      if ((s=ldelete(1L, FALSE))==TRUE)
         s = backchar(FALSE, 1);
   }

   /* Delete forward */
   s = forwchar(FALSE, 1);
   while ( s && ISWHITE( lgetc(curwp->w_dotp, curwp->w_doto) ) ) {
      if ((s=ldelete(1L, FALSE))==TRUE)
         s = forwchar(FALSE, 1);
   }
   return (TRUE);
}
#undef ISWHITE

/* special variable */

int scr_change=0;  /* should be placed in header file */

/*
 * Set screen width. If called with an argument, use that, else prompt for
 * it. An empty answer doesn't change anything.
 */
setwidth(f, n)
{
   char prompt[81];
   char reply[81];
   int width;
   extern TERM term;
      
   if (!f)
   {
      width = term.t_ncol;
      sprintf(prompt,"Screen width [%d]:",width);
      if (mlreply(prompt,reply,80) == ABORT)
         return(ABORT);
      trimsides(reply);
      if (reply[0])
         width = atoi(reply);
   }
   else
      width = n;
   if (width<1)
   {
      mlwrite("[Window width too small]");
      return(FALSE);
   }
   if (width>=NLINE)
   {
      mlwrite("[Window width too large]");
      return(FALSE);
   }

   term.t_ncol = width;
   scr_change = 1;

   force_update();

   mlwrite("[Window width = %d]",width);
}

#define SUPPORT 0      /* 0 = doesn't work, 1 may work but probably not */
/*
 * Set screen length. If called with an argument, use that, else prompt for
 * it. An empty answer doesn't change anything.
 */
setlength(f, n)
{
#if SUPPORT
   char prompt[81];
   char reply[81];
   int length;
   extern TERM term;
      
   if (!f)
   {
      length = term.t_nrow;
      sprintf(prompt,"Screen length [%d]:",length);
      if (mlreply(prompt,reply,80) == ABORT)
         return(ABORT);
      trimsides(reply);
      if (reply[0])
         length = atoi(reply);
   }
   else
      length = n;
   if (length<1) /* should be 2 * nwindows */
   {
      mlwrite("[Window length too small]");
      return(FALSE);
   }
   if (length>=200) /* ******************** */
   {
      mlwrite("[Window length too large]");
      return(FALSE);
   }

   vtclose();
   term.t_nrow = length;
   scr_change = 1;
   vtinit();

   force_update();

   mlwrite("[Window length = %d]",length);
#else
   mlwrite("[Sorry, not supported yet]");
#endif
}

force_update()
{
        WINDOW          *wp;
        extern TERM     term;
        int             rows;

        wp = wheadp;
        rows = 0;
        while (wp != NULL) {
                wp->w_flag |= WFHARD|WFMODE|WFFORCE;
                rows += wp->w_ntrows /* + 1 */;
                /* on to the next window */
                wp = wp->w_wndp;
        }

#if SUPPORT
/* This doesn't work !!!!! */

        if (rows < term.t_nrow) {
                for (wp = wheadp; wp != NULL && rows < term.t_nrow;) {
                        if (paste_wind(wp,1)==TRUE)
                                rows++;
                        else
                                wp = wp->w_wndp;
                }
        }
        else if (rows > term.t_nrow) {
                for (wp = wheadp; wp != NULL && rows > term.t_nrow;) {
                        if (cut_wind(wp,1)==TRUE)
                                rows--;
                        else
                                wp = wp->w_wndp;
                }
        }

#endif
        sgarbf = TRUE;
}
#if SUPPORT
/* Do not use from other routines until tested.... (paste_, cut_ - wind)*/
paste_wind(wp,n)
WINDOW  *wp;
int     n;
{
        extern TERM term;

        if (wp == NULL)
                return (FALSE);

        if (n < 0)
                return (FALSE);

        if (wp->w_ntrows+n > term.t_nrow) /* Should be better check */
                return (FALSE);

        wp->w_ntrows += n;
        wp->w_flag |= WFMODE|WFHARD|WFFORCE;

        return (TRUE);
}

cut_wind(wp,n)
WINDOW  *wp;
int     n;
{
        if (wp == NULL)
                return (FALSE);

        if (n < 0)
                return (FALSE);

        if (wp->w_ntrows - n < 2)
                return (FALSE);

        wp->w_ntrows -= n;
        wp->w_flag |= WFMODE|WFHARD|WFFORCE;

        return (TRUE);
}
#endif

emaxdelay(doupd)
{
#if !PR1MOS
   register int i;

   if (doupd)
      for (i = 0; i < term.t_pause; i++)
         update(FALSE);
   else
      for (i = 0; i < (term.t_pause*2); i++) /* compensate call .... */
         ;
#else
   if (doupd)
      update(FALSE);
   sleep$((int)term.t_pause);
#endif
}
