/*   EFUNC.H:   MicroEMACS function declarations and names

      This file list all the C code functions used by MicroEMACS
   and the names to use to bind keys to them. To add functions,
   declare it here in both the extern function list and the name
   binding table.

*/

/*   External function declarations      */

#if MREPFIX
extern   int    setrep1arg();
#endif

#if CTLCFIX
extern   int    repeat_last();          /* repeat last command            */
#endif

#if CTLUFIX
extern   int    setrep4arg();
#endif

#if EMAIL && PRIMOS
extern   int    emailsend();
#endif

#if PRIMOS
extern   int    setclmode();            /* Set command-line mode          */
extern   int    dispclmode();           /* Display command-line modes     */
#endif

#if FUNCFIX
extern   int   funcprefix();  /* prefix characters for function keys */
#endif

#if COUNTUP
extern   int    setcountup();           /* Set count-up interval          */
#endif

extern   int    safefileappend();
extern   int    safefilewrite();
extern   int    safefilesave();

extern   int    find_next_space();
extern   int    find_next_digit();
extern   int    find_next_alpha();

extern   int    find_prev_space();
extern   int    find_prev_digit();
extern   int    find_prev_alpha();

extern   int    find_next_nonspace();
extern   int    find_next_nondigit();
extern   int    find_next_nonalpha();

extern   int    find_prev_nonspace();
extern   int    find_prev_nondigit();
extern   int    find_prev_nonalpha();

extern   int    markwhole();            /* mark whole buffer              */
extern   int    killstartline();        /* kill to start of line          */
extern   int    indregion();            /* indent region                  */
extern   int    unindregion();          /* unindent region                */

extern   int    ifcmd();                /* "if" command                   */
extern   int    elsecmd();              /* "else" command                 */
extern   int    endcmd();               /* "endif" command                */

extern   int    subtractvar();          /* subtract a numeric variable    */
extern   int    addvar();               /* add to a numeric variable      */

extern   int    assignvar();            /* Set a local variable           */
extern   int    backtab();              /* Reversed tab                   */
extern   int    displayvar();           /* Display a local variable       */
extern   int    sayline();              /* Display a line of text         */
extern   int    linefeed();             /* ^J key handler                 */
extern   int    ctrlpkey();             /* ^P key handler                 */
extern   int    ctrlzkey();             /* ^Z key handler                 */
extern   int    killregadd();           /* Kill region & append to killb. */
extern   int    killbufadd();           /* Kill buffer & append to killb. */
extern   int    copyregadd();           /* Append region to kill buffer   */
extern   int    copybufadd();           /* Append buffer to kill buffer   */
extern   int    appendregion();         /* Append region to file          */
extern   int    writeregion();          /* Write region to file           */
extern   int    killthisbuf();          /* Erase buffer                   */
extern   int    killtoend();            /* Erase to end of buffer         */
extern   int    killtostart();          /* Erase to start of buffer       */
extern   int    fileappend();           /* Append buffer to file          */
extern   int    yankappend();           /* Append kill-buffer to file     */
extern   int    yankwrite();            /* Write kill-buffer to file      */
extern   int    setwidth();             /* Set terminal width             */
extern   int    ctrlg();                /* Abort out of things            */
extern   int    quit();                 /* Quit                           */
extern   int    overins();              /* Overlay/Insert mode toggle     */
extern   int    ctlxlp();               /* Begin macro                    */
extern   int    ctlxrp();               /* End macro                      */
extern   int    ctlxe();                /* Execute macro                  */
extern   int    fileread();             /* Get a file, read only          */
extern   int    filefind();             /* Get a file, read write         */
extern   int    filewrite();            /* Write a file                   */
extern   int    filesave();             /* Save current file              */
extern   int    filename();             /* Adjust file name               */
extern   int    getccol();              /* Get current column             */
extern   int    gotobol();              /* Move to start of line          */
extern   int    forwchar();             /* Move forward by characters     */
extern   int    gotoeol();              /* Move to end of line            */
extern   int    backchar();             /* Move backward by characters    */
extern   int    forwline();             /* Move forward by lines          */
extern   int    backline();             /* Move backward by lines         */
extern   int    forwpage();             /* Move forward by pages          */
extern   int    backpage();             /* Move backward by pages         */
extern   int    gotobob();              /* Move to start of buffer        */
extern   int    gotoeob();              /* Move to end of buffer          */
extern   int    setfillcol();           /* Set fill column.               */
extern   int    setmark();              /* Set mark                       */
extern   int    marktop();              /* Mark top of buffer             */
extern   int    markbottom();           /* Mark bottom of buffer          */
extern   int    swapmark();             /* Swap "." and mark              */
extern   int    forwsearch();           /* Search forward                 */
extern   int    backsearch();           /* Search backwards               */
extern   int    sreplace();             /* search and replace             */
extern   int    qreplace();             /* search and replace w/query     */
extern   int    showcpos();             /* Show the cursor position       */
extern   int    nextwind();             /* Move to the next window        */
extern   int    prevwind();             /* Move to the previous window    */
extern   int    onlywind();             /* Make current window only one   */
extern   int    splitwind();            /* Split current window           */
extern   int    mvdnwind();             /* Move window down               */
extern   int    mvupwind();             /* Move window up                 */
extern   int    enlargewind();          /* Enlarge display window.        */
extern   int    shrinkwind();           /* Shrink window.                 */
extern   int    listbuffers();          /* Display list of buffers        */
extern   int    usebuffer();            /* Switch a window to a buffer    */
extern   int    killbuffer();           /* Make a buffer go away.         */
extern   int    reposition();           /* Reposition window              */
extern   int    reposlimit();           /* (A version of 'Reposition')    */
extern   int    refresh();              /* Refresh the screen             */
extern   int    twiddle();              /* Twiddle characters             */
extern   int    tab();                  /* Insert tab                     */
extern   int    newline();              /* Insert CR-LF                   */
extern   int    indent();               /* Insert CR-LF, then indent      */
extern   int    openline();             /* Open up a blank line           */
extern   int    deblank();              /* Delete blank lines             */
extern   int    decblank();             /* Delete spaces around cursor    */
extern   int    quote();                /* Insert literal                 */
extern   int    backword();             /* Backup by words                */
extern   int    forwword();             /* Advance by words               */
extern   int    forwdel();              /* Forward delete                 */
extern   int    backdel();              /* Backward delete                */
extern   int    killtext();             /* Kill forward                   */
extern   int    yank();                 /* Yank back from killbuffer.     */
extern   int    upperword();            /* Upper case word.               */
extern   int    lowerword();            /* Lower case word.               */
extern   int    upperregion();          /* Upper case region.             */
extern   int    lowerregion();          /* Lower case region.             */
extern   int    capword();              /* Initial capitalize word.       */
extern   int    delfword();             /* Delete forward word.           */
extern   int    delbword();             /* Delete backward word.          */
extern   int    killregion();           /* Kill region.                   */
extern   int    copyregion();           /* Copy region to kill buffer.    */
extern   int    spawncli();             /* Run CLI in a subjob.           */
extern   int    spawn();                /* Run a command in a subjob.     */
#if   BSD
extern   int   bktoshell();             /* suspend emacs to parent shell  */
extern   int   rtfrmshell();            /* return from a suspended state  */
#endif
extern   int   quickexit();             /* low keystroke style exit.      */
extern   int   setmode();               /* set an editor mode             */
extern   int   delmode();               /* delete a mode                  */
extern   int   gotoline();              /* go to a numbered line          */
extern   int   namebuffer();            /* rename the current buffer      */
#if   WORDPRO
extern   int   gotobop();               /* go to begining/paragraph       */
extern   int   gotoeop();               /* go to end/paragraph            */
extern   int   fillpara();              /* fill current paragraph         */
#endif
extern   int   help();                  /* get the help file here         */
extern   int   deskey();                /* describe a key's binding       */
extern   int   viewfile();              /* find a file in view mode       */
extern   int   insfile();               /* insert a file                  */
extern   int   scrnextup();             /* scroll next window back        */
extern   int   scrnextdw();             /* scroll next window down        */
extern   int   bindtokey();             /* bind a function to a key       */
extern   int   unbindkey();             /* unbind a key's function        */
extern   int   namedcmd();              /* execute named command          */
extern   int   desbind();               /* describe bindings              */
extern   int   execcmd();               /* execute a command line         */
extern   int   execbuf();               /* exec commands from a buffer    */
extern   int   execfile();              /* exec commands from a file      */
extern   int   nextbuffer();            /* switch to the next buffer      */
#if   WORDPRO
extern   int   killpara();              /* kill the current paragraph     */
#endif
extern   int   setgmode();              /* set a global mode              */
extern   int   delgmode();              /* delete a global mode           */
extern   int   insspace();              /* insert a space forword         */
extern   int   forwhunt();              /* hunt forward for next match    */
extern   int   backhunt();              /* hunt backwards for next match  */
extern   int   pipe();                  /* pipe command into buffer       */
extern   int   filter();                /* filter buffer through dos      */
extern   int   delwind();               /* delete the current window      */
extern   int   cbuf1();                 /* execute numbered comd buffer   */
extern   int   cbuf2();
extern   int   cbuf3();
extern   int   cbuf4();
extern   int   cbuf5();
extern   int   cbuf6();
extern   int   cbuf7();
extern   int   cbuf8();
extern   int   cbuf9();
extern   int   cbuf10();
extern   int   cbuf11();
extern   int   cbuf12();
extern   int   cbuf13();
extern   int   cbuf14();
extern   int   cbuf15();
extern   int   cbuf16();
extern   int   cbuf17();
extern   int   cbuf18();
extern   int   cbuf19();
extern   int   cbuf20();
extern   int   cbuf21();
extern   int   cbuf22();
extern   int   cbuf23();
extern   int   cbuf24();
extern   int   cbuf25();
extern   int   cbuf26();
extern   int   cbuf27();
extern   int   cbuf28();
extern   int   cbuf29();
extern   int   cbuf30();
extern   int   cbuf31();
extern   int   cbuf32();
extern   int   cbuf33();
extern   int   cbuf34();
extern   int   cbuf35();
extern   int   cbuf36();
extern   int   cbuf37();
extern   int   cbuf38();
extern   int   cbuf39();
extern   int   cbuf40();
extern   int   storemac();                 /* store text for macro           */
extern   int   resize();                   /* resize current window          */
extern   int   clrmes();                   /* clear the message line         */
extern   int   meta();                     /* meta prefix dummy function     */
extern   int   cex();                      /* ^X prefix dummy function       */
extern   int   istring();                  /* insert string in text          */
extern   int   unmark();                   /* unmark current buffer          */
#if   ISRCH
extern   int   fisearch();                 /* forward incremental search     */
extern   int   risearch();                 /* reverse incremental search     */
#endif
#if   WORDPRO
extern   int   wordcount();                /* count words in region          */
#endif
extern   int   savewnd();                  /* save current window            */
extern   int   restwnd();                  /* restore current window         */
extern   int   upscreen();                 /* force screen update            */
extern   int   writemsg();                 /* write text on message line     */

/*   Name to function binding table

      This table gives the names of all the bindable functions
   end their C function address. These are used for the bind-to-key
   function.
*/

