/*
 * The routines in this file read and write ASCII files from the disk. All of
 * the knowledge about files are here. A better message writing scheme should
 * be used.
 *
 *
 */

#include        <stdio.h>
#include        <stddef.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"

#if !PRIMOS || C_FIO

#define TABCHR 0211

#if PRIMOS
#include        <keys>
  extern int dam_mode;
#endif

FILE    *ffp;                           /* File pointer, all functions. */

/*
 * Open a file for reading.
 */
ffropen(fn)
char    *fn;
{
        if ((ffp=fopen(fn, "r")) == NULL)
                return (FIOFNF);
        return (FIOSUC);
}

/*
 * Open a file for writing. Return TRUE if all is well, and FALSE on error
 * (cannot create).
 */
ffwopen(fn)
char    *fn;
{
         int    fd;

#if     PRIMOS

         /* Prime's C can only create random-access (DAM) files.
          * They need more disk space than SAM files.
          * With an option, a SAM-file is created by a PRIMOS routine
          * before opened normally.
          *
          * At Rev. 21, SAM-file creating is enabled, but only when using
          * the low-level routine open.
          */

         if (!dam_mode)
         {
            if ((fd = open (fn, 010401)) == -1 ||
               (ffp = fdopen (fd, "w")) == NULL)
            {
               mlwrite("Unable to create samfile");
               return FIOERR;
            }
            return FIOSUC;
         }
#endif

#if     VMS

        if ((fd=creat(fn, 0666, "rfm=var", "rat=cr")) < 0
        || (ffp=fdopen(fd, "w")) == NULL) {
#else
        if ((ffp=fopen(fn, "w")) == NULL) {
#endif
                mlwrite("Cannot open file for writing");
                return (FIOERR);
        }
        return (FIOSUC);
}

/*
 * Open a file for writing at the end of it. Return TRUE if all is well,
 * and FALSE on error (cannot create).
 */
ffaopen(fn)
char    *fn;
{

        if ((ffp=fopen(fn, "wa")) == NULL)
        {
                mlwrite("[Cannot open file for write-append]");
                return (FIOERR);
        }
        return (FIOSUC);
}


/*
 * Close a file. Should look at the status in all systems.
 */
ffclose()
{
#if   MSDOS
   fputc(26, ffp);      /* add a ^Z at the end of the file */
#endif

#if     V7 | USG | BSD | (MSDOS & (LATTICE | MSC)) | PRIMOS
        if (fclose(ffp) != FALSE) {
                mlwrite("Error closing file");
                return(FIOERR);
        }
        return(FIOSUC);
#else
        fclose(ffp);
        return (FIOSUC);
#endif
}
#if FIXSUFF
fix_file_suffix(fname)
char *fname;
{
        static char *suffix[] = {
#ifdef PRIMOS
                "",
                ".c", ".cc", ".h",
                ".cpl", ".comi", ".como",
                ".ftn", ".f77",
                ".p", ".pas", ".pascal",
                ".cbl", ".cobol",
                ".basic", ".bas",
                ".pma", ".cma", ".asm",
                ".pl1", ".pl1g", ".pl/1", ".plp", ".spl",
                ".macro",
                ".rpg",
                ".doc", ".txt",
                ".pro", ".prolog",
                ".lsp", ".lisp",
                ".ins",
                ".sr"
#else
                "",".c",".h"
#endif
        };
        int i,j;
        char tmp[NFILEN];

#if PRIMOS | MSDOS | VMS
        for (j=strlen(fname),i=0; i<j; i++)
                fname[i]=tolower(fname[i]);
#endif

        for (i=0; i<(sizeof(suffix)/sizeof(char *)); i++) {
                strcpy(tmp,fname);
                strcat(tmp,suffix[i]);
                if (ffropen(tmp)==FIOSUC) {
                        strcat(fname,suffix[i]);
                        fclose(ffp);
                        return;
                }
                fclose(ffp);
        }

}
#endif
/*
 * Write a line to the already opened file. The "buf" points to the buffer,
 * and the "nbuf" is its length, less the free newline. Return the status.
 * Check only at the newline.
 */
ffputline(buf, nbuf)
char    buf[];
{
        extern int compress;
        char       *eptr;
        char       *bufp;

        bufp = buf;
        eptr = bufp+nbuf-1;

        /* If compress mode is on, all trailing blanks shall be removed when
        ** saving
        */

        if (compress)
        {
          /* Length shall be shortened so no trailing blanks are written */
          while (eptr >= bufp && *eptr == ' ')
             eptr--;
        }

        while (bufp <= eptr)
            fputc(*bufp++ & 0xFF, ffp);

        fputc('\n', ffp);

        if (ferror(ffp))
        {
           mlwrite("Write I/O error %d", errno);
           return (FIOERR);
        }

        return (FIOSUC);
}

/*
 * Read a line from a file, and store the bytes in the supplied buffer. The
 * "nbuf" is the length of the buffer. Complain about long lines and lines
 * at the end of the file that don't have a newline present. Check for I/O
 * errors too. Return status.
 */
ffgetline(buf, nbuf)
 char   buf[];
{
         int    c;
         int    i;

        i = 0;

        while ((c = fgetc(ffp)) != EOF && c != '\n')
        {
                if (i >= nbuf-2)
                {
                         buf[nbuf - 2] = c;   /* store last char read */
                         buf[nbuf - 1] = 0;   /* and terminate it */
                        return (FIOLNG);
                }
                buf[i++] = c;
        }

        if (c == EOF)
        {
                if (ferror(ffp))
                {
                        mlwrite("File read error");
                        return (FIOERR);
                }

                if (i != 0)
                {
                        buf[i] = 0;
                        return (FIOSUC);
                }
                return (FIOEOF);
        }

        buf[i] = 0;
        return (FIOSUC);
}

#if     PRIMOS
/*
** This routine creates (opens and closes) a SAM-file
*/

createsamfile (fname)
char *fname;
{
   int unit;

   if (fexists (fname))
      if (delete (fname) == -1)
         return FALSE;


   close (unit);
}

/* Filetype returns FALSE if the specified file is of a wrong type */
filetype (fn)
   char *fn;
{
   int type;

   type = ftype (fn);
   if (type != 0 && type != 1 && type != -1)
      return (FALSE);

   return (TRUE);
}
#endif

#endif
