/*
 * This program is in public domain; written by Dave G. Conroy.
 * This file contains the main driving routine, and some keyboard processing
 * code, for the MicroEMACS screen editor.
 *
 */

#include        <stdio.h>
#include        <ctype.h>

/* make global definitions not external */

#include   "estruct.h"   /* global structures and defines         */
#include   "edef.h"      /* global definitions                    */

#include   "efunc.h"     /* function declarations and name table  */
#include   "ebind.h"     /* default key bindings                  */

#if PRIMOS
#  define DOSNAME "EMAX"
#else
#  if
#     define DOSNAME "EMACS"
#  else
#     define DOSNAME "emacs"
#  endif
#endif


#if     VMS
#include        <ssdef.h>
#define GOOD    (SS$_NORMAL)
#endif

#ifndef GOOD
#define GOOD    0
#endif

extern char *ExpandWild();


#if   APROF             /* Declarations needed for AZTEC C profiling */
int _Corg();            /* first address of program                  */
int _Cend();            /* last address of program                   */

short monbuf[NBUCK];    /* buffer for gather info                    */
#endif


extern char EditorName[];


int new_f = 0;
int new_c = 1;

int global_meta_flag = 0;
int global_ctrlx_flag = 0;

int fancy_name_mode = 0;

int LastWasPrefix = 0;

#if CTLCFIX
int (*old_cmdptr)();
int old_c;
int old_f, old_n;
#endif

#if PRIMOS

#  include <keys>

   fortran void quit_handler(),
             logout_handler(),
             dummy_handler(),
             any_handler();

   fortran void mkon$p(), logo$$();

   int next_key       = -1;
   int next_arg       =  1;
   int emulate_ctrl_p =  0;
#endif

#if !SHELLTERM
   char *terminal_type;
#endif
int terminal_width  = 80;
int terminal_height = 24;

#if PRIMOS
   int dam_mode         = FALSE;
   int bmode_flag       = TRUE;
   char logout_name[81] = ".LOGOUT";
   int u2p_flag         = TRUE;
   extern char SearchListName[];

#endif
#if ACMODE
   int cmode_flag       = TRUE;
#endif
#if ALSPMODE
   int lspmode_flag     = TRUE;
#endif
#if REPCOMM
   int rpt_flag         = FALSE;
#endif
#if COUNTUP
   int count_up         = 100;
   int count_up_flag    = TRUE;
#endif
#if COLOR
#if PRIMOS
   int color_on         = FALSE;
#else
   int color_on         = TRUE;
#endif
#endif

char *cmdoptions[] =
   {
      "h",     "help",                 /*  0 */
      "v",     "view",                 /*  1 */
      "e",     "edit",                 /*  2 */
      "s",     "search",               /*  3 */
      "g",     "goto",                 /*  4 */
      "hei",   "height",               /*  5 */
      "w",     "width",                /*  6 */
      "m",     "macro",                /*  7 */
      "t",     "terminal",             /*  8 */
      "c",     "cmode",                /*  9 */
      "nc",    "no_cmode",             /* 10 */
      "b",     "backup",               /* 11 */
      "nb",    "no_backup",            /* 12 */
      "bn",    "backup_name",          /* 13 */
      "ta",    "tabulate",             /* 14 */
      "ln",    "logout_name",          /* 15 */
      "cz",    "ctrl_z_up",            /* 16 */
      "cp",    "ctrl_p_up",            /* 17 */
      "wb",    "word_beginning",       /* 18 */
      "we",    "word_end",             /* 19 */
      "a",     "aproto",               /* 20 */
      "na",    "no_aproto",            /* 21 */
      "co",    "compress",             /* 22 */
      "nco",   "no_compress",          /* 23 */
      "dam",   "dam_file",             /* 24 */
      "sam",   "sam_file",             /* 25 */
      "cu",    "count_up",             /* 26 */
      "ncu",   "no_count_up",          /* 27 */
      "col",    "color",               /* 28 */
      "ncol",   "no_color",            /* 29 */
      "am",    "amlc",                 /* 30 */
      "nam",   "no_amlc",              /* 31 */
      "lsp",   "lspmode",              /* 32 */
      "nlsp",  "no_lspmode",           /* 33 */
      "ux",    "unix",                 /* 34 */
      "nux",   "no_unix",              /* 35 */
      "ecp",   "emulate_ctrl_p",       /* 36 */
      "necp",  "no_emalate_ctrl_p",    /* 37 */
      "sr",    "search_rule",          /* 38 */
      "nsr",   "no_search_rule",       /* 39 */
      "sp",    "split",                /* 40 */
      "nsp",   "no_split",             /* 41 */
      "fnm",   "fancy_name_mode",      /* 42 */
      "nfnm",  "no_fancy_name_mode",   /* 43 */
      NULL,    NULL
   };

char digit_list[11] = "0123456789";
char alpha_list[100];


#define CMDHELP  0
#define CMDVIEW  1
#define CMDEDIT  2
#define CMDSRCH  3
#define CMDGOTO  4
#define CMDHGHT  5
#define CMDWDTH  6
#define CMDMCRO  7
#define CMDTERM  8
#define CMDCMOD  9
#define CMDNCMD 10
#define CMDBMOD 11
#define CMDNBMD 12
#define CMDBNAM 13
#define CMDTABU 14
#define CMDLNAM 15
#define CMDCTLZ 16
#define CMDNCTZ 17
#define CMDWBEG 18
#define CMDWEND 19
#define CMDAPRO 20
#define CMDNAPR 21
#define CMDCOMP 22
#define CMDNCMP 23
#define CMDDAMF 24
#define CMDSAMF 25
#define CMDCOUP 26
#define CMDNCUP 27
#define CMDCOLO 28
#define CMDNCOL 29
#define CMDAMLC 30
#define CMDNAMC 31
#define CMDLSPM 32
#define CMDNLSP 33
#define CMDUNIX 34
#define CMDNUNX 35
#define CMDEMCP 36
#define CMDNECP 37
#define CMDSRFF 38
#define CMDNSRF 39
#define CMDSPLT 40
#define CMDNSPT 41
#define CMDFNMO 42
#define CMDNFNM 43


int ctrlz_mode = FALSE;  /* Use the ^P key to move up. */
int wordbegin  = FALSE;  /* Go to the beginning of word when ESC-F */
int compress   = FALSE;  /* Remove trailing blanks when saving files */

#if PRIMOS
int UseSearchList = TRUE;
int aprot_mode = TRUE;   /* Use the APROTO function to enable usage of CTRL-P */
extern short line_number;
extern int use_amlc;
#endif

#if PRIMOS
/*
** Uuurk! PRIMOS doesn't properly reinitialize global pointers if a program
** is an EPF and executed twice... :-(
*/
init_global_pointers()
{
   bstore  = NULL;
   kbufp   = NULL;
   kbufh   = NULL;
   swindow = NULL;
}
#endif

main(argc, argv)
char    *argv[];
{
   char *fnp;
   int   s, i;
   int   got_files  = 0;
   int   SplitCount = 0;


#if REPCOMM
#if !CTLCFIX
        int old_c;
        int old_f;
        int old_n;
        int first_flag = TRUE;
#endif
#endif
#if PRIMOS
        extern char backup_name[];
#endif
        extern char *strlower();
        extern int tabsize;
         int    loop;      /* temporary loop variable */
         int    c;
         int    f;
         int    n;
         int    mflag;
         BUFFER *bp, *tbp;
         int   ffile;       /* first file flag */
         int   carg;        /* current arg to scan */
         int   startf;      /* startup executed flag */
        int basec;                  /* c stripped of meta character */
        int viewflag;               /* edit file in view flag */
        int gotoflag;               /* do we need to goto a line at start? */
        int gline;                  /* if so, what line? */
        int searchflag;             /* Do we need to search at start? */
        char bname[NBUFN];          /* buffer name of file to read */
        char fname[NFILEN];
        char fname2[NFILEN];
        int bnp, bni;

#define SRCMAX 100
#define MACMAX 100

        int source_file[SRCMAX],    /* buffer for file names      */
            macro_file[MACMAX];     /* buffer for macro_files     */
        int srcloop = 0,            /* source file loop counters  */
            macloop = 0;
        int vwrdflag[SRCMAX];

   printf("[%s]\n", EditorName);

   i = 0;
   for (s = 'A'; s <= 'Z'; s++)
      alpha_list[i++] = s;
   for (s = 'a'; s <= 'z'; s++)
      alpha_list[i++] = s;
   alpha_list[i] = '\0';


#if CTLCFIX
   old_cmdptr = NULL;
   old_c      = -1;
#endif

#if   APROF
   /* if we are doing AZTEC C profiling, start it up */
   /*_intr_sp(18);    set clock interupt for 60/second */
   monitor(_Corg, _Cend, monbuf, NBUCK, 0);
#endif

#if   PRIMOS
   /*
   ** Guess what? PRIMOS doesn't correctly initialize global pointers...
   */
   init_global_pointers();
#endif

   viewflag = FALSE;            /* view mode defaults off in command line */
   gotoflag = FALSE;            /* set to off to begin with */
   searchflag = FALSE;          /* set to off to begin with */

#if !SHELLTERM
   terminal_type = "VT100";     /* default terminal type */
#endif

   /* scan through the command line and get the files to edit */
   for (carg = 1; carg < argc; ++carg)
   {
      /* if it's a switch, process it */
      if (*strlower(argv[carg]) == '-')
      {
         s = strlocate(argv[carg]+1,cmdoptions);
         switch ( s == -1 ? -1 : s >> 1 )
         {
            case CMDFNMO:
               fancy_name_mode = TRUE;
               break;

            case CMDNFNM:
               fancy_name_mode = FALSE;
               break;

#if PRIMOS
            case CMDEMCP:
               emulate_ctrl_p = TRUE;
               break;

            case CMDNECP:
               emulate_ctrl_p = FALSE;
               break;

            case CMDAPRO:
               aprot_mode = TRUE;
               break;
            case CMDNAPR:
               aprot_mode = FALSE;
               break;
            case CMDUNIX:
               u2p_flag = TRUE;
               break;
            case CMDNUNX:
               u2p_flag = FALSE;
               break;
            case CMDSRFF:
               UseSearchList = TRUE;
               if (carg+1 != argc && argv[carg+1][0] != '-')
                  strcpy(SearchListName, argv[++carg]);
               break;
            case CMDNSRF:
               UseSearchList = FALSE;
               break;
#endif
            case CMDSPLT:
               if (carg+1 < argc && isdigit(argv[carg+1][0]))
                  SplitCount = atoi(argv[++carg]);
               else
                  SplitCount = 2;
               break;

            case CMDNSPT:
               SplitCount = 0;
               break;

            case CMDWBEG:
               wordbegin = TRUE;
               break;
            case CMDWEND:
               wordbegin = FALSE;
               break;
            case CMDVIEW:
               viewflag = TRUE;
               break;
            case CMDEDIT:
               viewflag = FALSE;
               break;
            case CMDSRCH:
               if (carg+1 != argc && argv[carg+1][0] != '-')
                  {
                  searchflag = TRUE;
                  strcpy(pat,argv[++carg]);
                  }
               else
                  searchflag = FALSE;
               break;
            case CMDGOTO:
               if (carg+1 != argc && isdigit(argv[carg+1][0]))
                  {
                  gotoflag = TRUE;
                  gline = atoi(argv[++carg]);
                  }
               else
                  gotoflag = FALSE;
               break;
            case CMDMCRO:
               if (carg+1 != argc && macloop != MACMAX
                                  && argv[carg+1][0] != '-')
                  macro_file[macloop++] = ++carg;
               else
                  {
                  puts("Incorrect -MACRO switch! (MicroEMACS)");
                  exit(1);
                  }
               break;
            case CMDCOMP:
              compress= TRUE;
              break;
            case CMDNCMP:
              compress= FALSE;
              break;
#if !SHELLTERM
            case CMDTERM:
               if (carg+1 != argc && argv[carg+1][0] != '-')
                  terminal_type = argv[++carg];
               else
                  {
                  puts("Incorrect -TERMINAL switch! (MicroEMACS)");
                  exit(1);
                  }
               break;
#endif

            case CMDWDTH:
               if (carg+1 != argc && isdigit(argv[carg+1][0]))
                  terminal_width = atoi(argv[++carg]);
               else
                  terminal_width = 80;
               break;

            case CMDHGHT:
               if (carg+1 != argc && isdigit(argv[carg+1][0]))
                  terminal_height = atoi(argv[++carg]);
               else
                  terminal_height = 24;
               break;

#if ACMODE
            case CMDCMOD:
               cmode_flag = TRUE;
               break;

            case CMDNCMD:
               cmode_flag = FALSE;
               break;
#endif
#if ALSPMODE
            case CMDLSPM:
               lspmode_flag = TRUE;
               break;
            case CMDNLSP:
               lspmode_flag = FALSE;
               break;
#endif
            case CMDCTLZ:
               ctrlz_mode = TRUE;
               break;

            case CMDNCTZ:
               ctrlz_mode = FALSE;
               break;
#if PRIMOS

            case CMDBMOD:
               bmode_flag = TRUE;
               break;

            case CMDNBMD:
               bmode_flag = FALSE;
               break;

            case CMDBNAM:
               if (carg+1 != argc && argv[carg+1][0] != '-')
               {
                  strcpy(backup_name,".");
                  strcat(backup_name,argv[++carg]);
               }
               else
                  strcpy(backup_name,".BACKUP");
               break;

            case CMDLNAM:
               if (carg+1 != argc && argv[carg+1][0] != '-')
               {
                  strcpy(logout_name,".");
                  strcat(logout_name,argv[++carg]);
               }
               else
                  strcpy(logout_name,".LOGOUT");
               break;
            case CMDDAMF:
               dam_mode= TRUE;
               break;
            case CMDSAMF:
               dam_mode= FALSE;
               break;
#endif

            case CMDTABU:
               if (carg+1 != argc && isdigit(argv[carg+1][0]))
                  tabsize = atoi(argv[++carg]);
               else
                  tabsize = 3;
               break;
#if COUNTUP
            case CMDCOUP:
              if (carg+1 != argc && isdigit(argv[carg+1][0]))
                 count_up = atoi (argv[++carg]);
              count_up_flag = TRUE;
              break;
            case CMDNCUP:
              count_up_flag = FALSE;
              break;
#endif
#if COLOR
            case CMDCOLO:
               color_on = TRUE;
               break;
            case CMDNCOL:
               color_on = FALSE;
               break;
#endif
#if PRIMOS
            case CMDAMLC:
               if (carg+1 != argc && isdigit(argv[carg+1][0]))
                  line_number = atoi(argv[++carg]);
               else
                  line_number = -1;
               use_amlc = 1;
               break;

            case CMDNAMC:
               line_number = -1;
               use_amlc = 0;
               break;
#endif
            case CMDHELP:
               printf("Syntax: %s <file-1> .. <file-100> <option-1> <option-N>\n", DOSNAME);
               puts("\nWhere <option> may be:\n");
               puts("-Help ................. Display this text.");
               puts("-View ................. All files specified are edited in VIEW mode only.");
               puts("-Edit ................. All files specified are edited in EDIT mode only.");
               puts("-TAbulate <step> ...... Set tabulate step size to <step>.");
               puts("-Search <text>......... Search for <text> and start editing there.");
               puts("-Goto <line> .......... Start editing at line number <line>.");
               puts("-Height <lines> ....... Select the terminals height.");
               puts("-Width <cols> ......... Select the terminals width.");
               puts("-Macro <file> ......... Execute a macro file before editing the files.");
               puts("-Word_Beginning ....... Move to the beginning of words.");
               puts("-Word_End ............. Move to the end of words.");
               puts("-COmpress.............. Trim end of lines when saving.");
               puts("-No_COmpress........... Don't trim eoln when saving.");
               puts("-Ctrl_Z_up ............ Use the ^Z key to move up one line.");
               puts("-Ctrl_P_up ............ Use the ^P key to move up one line.");
#if PRIMOS
               puts("-Emulate_Ctrl_P ....... Enable the Ctrl P emulator.");
               puts("-No_Emulate_Ctrl_P..... Disable the Ctrl P emulator.");
               puts("-Aproto ............... Use the APROTO() function to enable usage of Ctrl_P_up.");
               puts("-No_Aproto ............ Don't use the APROTO() function.");
               puts("-AMlc <line> .......... Communicate through the specified line.");
               puts("-No_AMlc .............. Communicate through the normal terminal.");
               puts("-UniX ................. Allow UNIX pathnames.");
               puts("-No_UniX .............. Do not allow UNIX pathnames.");
#endif
#if !SHELLTERM
               puts("-Terminal <type> ...... Select what terminal type MicroEMACS is run on.");
#endif
#if ACMODE
               puts("-Cmode ................ Enable C mode editing.");
               puts("-No_Cmode ............. Disable C mode editing.");
#endif
#if PRIMOS
               puts("-Backup ............... Enable automatic file backup.");
               puts("-No_Backup ............ Disable automatic file backup.");
               puts("-Backup_Name <name> ... Set name to add to backup-files.");
               puts("-Logout_Name <name> ... Set name to add to the logout-files.");
               puts("-DAM_file ............. Save files as DAM.");
               puts("-SAM_file ............. Save files as SAM.");
#endif
#if COUNTUP
               puts("-Count_Up <interval> .. Set count_up interval.");
               puts("-No_Count_Up .......... Disable count_up.");
#endif
#if ALSPMODE
               puts("-LSPmode .............. Enable Lisp mode editing.");
               puts("-No_LSPmode ........... Disable Lisp mode editing.");
#endif
#if COLOR
               puts("-COLor ................ Enable color screen.");
               puts("-No_COLor ............. Disable coor screen.");
#endif
#if PRIMOS
               puts("-Search_Rule .......... Use search rules.");
               puts("-No_Search_Rule ....... Do not use search rules.");
#endif

               puts("-SPlit [n] ............ Split window when startup.");
               puts("-No_SPlit ............. Only one window when startup.");

               puts("\nAll options can be abbreviated by using the uppercase characters only.");
               exit(0);

            default:
               printf("Incorrect command line switch. Enter %s -Help for help. (MicroEMACS).\n", DOSNAME);
               exit(1);
            }
         }
      else
         if (srcloop != SRCMAX)
            {
            source_file[srcloop] = carg;
            vwrdflag[srcloop++] = viewflag;
            }
      }

   /* initialize the editor */
   strcpy(bname, "main");       /* default buffer name  */
   vtinit();                    /* Displays.            */
   edinit(bname);               /* Buffers, windows.    */
   ffile = TRUE;                /* no file to edit yet */
   startf = FALSE;              /* startup file not executed yet */

   /* setup to process commands */
   lastflag = 0;                          /* Fake last flags.     */
   curbp->b_mode |= gmode;                /* and set default modes*/
   curwp->w_flag |= WFMODE;               /* and force an update   */
#if   COLOR
   curwp->w_fcolor = gfcolor;             /* and set colors   */
   curwp->w_bcolor = gbcolor;
#endif

#if PRIMOS
        if (!aprot_mode)
           mkon$p("QUIT$",             5, quit_handler);

        mkon$p("ARITH$",            6, dummy_handler);
        mkon$p("CLEANUP$",          8, dummy_handler);
        mkon$p("COMI_EOF$",         9, dummy_handler);
        mkon$p("ENDPAGE",           7, dummy_handler);
        mkon$p("EXIT$",             5, dummy_handler);
        mkon$p("FINISH",            6, dummy_handler);
        mkon$p("LISTENER_ORDER$",  15, dummy_handler);
        mkon$p("NAME",              4, dummy_handler);
        mkon$p("NO_AVAIL_SEGS$",   14, dummy_handler);
        mkon$p("NONLOCAL_GOTO$",   14, dummy_handler);
        mkon$p("PAGE_FAULT_ERR$",  15, dummy_handler);
        mkon$p("PAUSE$",            6, dummy_handler);
        mkon$p("PH_LOGO$",          8, dummy_handler);
        mkon$p("REENTER$",          8, dummy_handler);
        mkon$p("ANY$",              4, any_handler);
        mkon$p("LOGOUT$",           7, logout_handler);
#endif
        update(FALSE);                          /* Fix up the screen    */

   /* Execute all macro files */
   startf = FALSE;
   for (loop=0; loop<macloop; loop++)
      if (startup(argv[macro_file[loop]]) == TRUE)
         startf = TRUE;

   if (startf == FALSE)  /* If no other macro files executed, then */
     startup("");        /* run EMAX.INIT                          */

   /* Read in all source files */
   for (loop=0; loop<srcloop; loop++)
   {
      viewflag = vwrdflag[loop];

      /* set up a buffer for this file */
      strcpy(fname,argv[source_file[loop]]);
#if PRIMOS
      Path2Primos(fname, fname2);
      strcpy(fname, fname2);
#endif
#if FIXSUFF
      fix_file_suffix(fname);
#endif

      fnp = ExpandWild(fname);
      s   = 0;

      while (fnp)
      {
         makename(bname, fnp);

         /* if this is the first file, read it in */
         if (ffile || SplitCount > 1)
         {
            if (loop > 0 && SplitCount > 1)
            {
               SplitCount--;
               splitwind(0,1);
               nextwind(0,1);
            }

            if (ffile)
            {
               bp = curbp;

               strcpy(bp->b_bname, bname);
               strcpy(bp->b_fname, fnp);

               if (readin(fnp, TRUE) == TRUE)
                  got_files++;

               bp->b_doto = 0;
               bp->b_dotp = bp->b_linep;
               ffile = FALSE;
            }
            else
            {
               bni = 1;
               bnp = strlen(bname);
               while ((bp = bfind(bname, FALSE, 0)) && bni < 20)
                  sprintf(bname+bnp, "/%d", bni++);
               if (bni == 20)
                  goto GET_NEXT_FILE;

               bp = bfind(bname, TRUE, 0);

               strcpy(bp->b_bname, bname);
               strcpy(bp->b_fname, fnp);

               tbp = curbp;
               curbp = bp;

               if (readin(fnp, TRUE) == TRUE)
                  got_files++;

               curbp = tbp;

               bp->b_doto = 0;
               bp->b_dotp = bp->b_linep;
            }
         }
         else
         {
            /* set this to inactive */
            got_files++;
            bni = 1;
            bnp = strlen(bname);
            while ((bp = bfind(bname, FALSE, 0)) && bni < 20)
               sprintf(bname+bnp, "/%d", bni++);
            if (bni == 20)
               goto GET_NEXT_FILE;

            bp = bfind(bname, TRUE, 0);

            strcpy(bp->b_fname, fnp);
            bp->b_active = FALSE;
         }

         /* set the view mode appropriatly */
         if (viewflag)
            bp->b_mode |= MDVIEW;

GET_NEXT_FILE:
         fnp = ExpandWild(NULL);
      }

   }  /* end for */

      /*
      ** No files found or read in?
      */
      if (got_files == 0)
         bp = bfind("main", TRUE, 0);

      /* Deal with startup gotos and searches */

      if (gotoflag && searchflag)
         {
         update(FALSE);
         mlwrite("[Can not search and goto at the same time!]");
         }
      else
         if (gotoflag)
            {
            if (gotoline(TRUE,gline) == FALSE)
               {
               update(FALSE);
               mlwrite("[Bogus goto argument]");
               }
            }
         else
            if (searchflag)
               {
               if (forscan(&pat[0], 2) == FALSE)
                  {
                  update(FALSE);
                  mlwrite("Not found.");
                  }
               }


MAINLOOP:
   update(FALSE);                          /* Fix up the screen    */

   if (next_key == -1)
   {
      c = getcmd();
      if (mpresf == TRUE)
      {
         clrmes(0,0);
         update(FALSE);
#if   CLRMSG
         if (c == ' ')                   /* ITS EMACS does this  */
            goto MAINLOOP;
#endif
      }

      f = FALSE;
      n = 1;

      /* do META-# processing if needed */

      basec = c & ~META;      /* strip meta char off if there */
      if ((c & META) && ((basec >= '0' && basec <= '9') || basec == '-'))
      {
         LastWasPrefix = 0;
         f = TRUE;      /* there is a # arg */
         n = 0;         /* start with a zero default */
         mflag = 1;      /* current minus flag */
         c = basec;      /* strip the META */
         while ((c >= '0' && c <= '9') || (c == '-'))
         {
            if (c == '-')
            {
               /* already hit a minus or digit? */
               if ((mflag == -1) || (n != 0))
                  break;
               mflag = -1;
            }
            else
            {
               n = n * 10 + (c - '0');
            }

            if ((n == 0) && (mflag == -1))   /* lonely - */
               mlwrite("Arg:");
            else
               mlwrite("Arg: %d",n * mflag);

            c = getcmd();   /* get the next key */
         }

         n = n * mflag;   /* figure in the sign */
      }

      /* do ^U repeat argument processing */
#if !CTLUFIX
      if (c == (CTRL|'U'))                    /* ^U, start argument */
      {
         f = TRUE;
         n = 4;                          /* with argument of 4 */
         mflag = 0;                      /* that can be discarded. */
         mlwrite("Arg: 4");
         while ((c=getcmd()) >='0' && c<='9' || c==(CTRL|'U') || c=='-')
         {
            if (c == (CTRL|'U'))
               n = n*4;

            /*
            * If dash, and start of argument string, set arg.
            * to -1.  Otherwise, insert it.
            */
            else if (c == '-')
            {
               if (mflag)
                  break;

               n = 0;
               mflag = -1;
            }

            /*
            * If first digit entered, replace previous argument
            * with digit and set sign.  Otherwise, append to arg.
            */
            else
            {
               if (!mflag)
               {
                  n = 0;
                  mflag = 1;
               }

               n = 10*n + c - '0';
            }

            mlwrite("Arg: %d", (mflag >=0) ? n : (n ? -n : -1));
         }

         /*
         * Make arguments preceded by a minus sign negative and change
         * the special argument "^U -" to an effective "^U -1".
         */
         if (mflag == -1)
         {
            if (n == 0)
               n++;

            n = -n;
         }
      }
#else
      f = new_f;
      n = new_n;
      new_f = 0;
      new_n = 1;
#endif

   }
#ifdef PRIMOS
   else
   {
      c = next_key;
      n = next_arg;
      f = 1;
   }
#endif

#if !CTLCFIX
   if (c == (CTRL|'C') && !first_flag)
   {
      rpt_flag = TRUE;
      execute( old_c, old_f, old_n);
      rpt_flag = FALSE;
   }
   else
   {
      first_flag = FALSE;
      old_c = c;
      old_f = f;
      old_n = n;
      execute(c, f, n);
   }
#else
   execute(c, f, n);
#endif
   goto MAINLOOP;
}

/*
 * Initialize all of the buffers and windows. The buffer name is passed down
 * as an argument, because the main routine may have been told to read in a
 * file by default, and we want the buffer name to be right.
 */
edinit(bname)
char    bname[];
{
         BUFFER *bp;
         WINDOW *wp;
         char *malloc();

        bp = bfind(bname, TRUE, 0);             /* First buffer         */
        blistp = bfind("[List]", TRUE, BFINVS); /* Buffer list buffer   */
        wp = (WINDOW *) malloc(sizeof(WINDOW)); /* First window         */
        if (bp==NULL || wp==NULL || blistp==NULL)
                {
                puts("Unable to allocate enough memory! (edinit/MicroEMACS)");
                exit(1);
                }
        curbp  = bp;                            /* Make this current    */
        wheadp = wp;
        curwp  = wp;
        wp->w_wndp  = NULL;                     /* Initialize window    */
        wp->w_bufp  = bp;
        bp->b_nwnd  = 1;                        /* Displayed.           */
        wp->w_linep = bp->b_linep;
        wp->w_dotp  = bp->b_linep;
        wp->w_doto  = 0;
        wp->w_markp = NULL;
        wp->w_marko = 0;
        wp->w_toprow = 0;
#if   COLOR
   /* initalize colors to global defaults */
   wp->w_fcolor = gfcolor;
   wp->w_bcolor = gbcolor;
#endif
        wp->w_ntrows = term.t_nrow-1;           /* "-1" for mode line.  */
        wp->w_force = 0;
        wp->w_flag  = WFMODE|WFHARD;            /* Full.                */
}


#if CTLCFIX

int repeat_last(f, n)
int f, n;
{
   int code;


   if (!f)
      n = 1;


   if (!old_cmdptr)
   {
      if (old_c == -1)
      {
         mlwrite("[No command available to repeat]");
         return FALSE;
      }
      else
      {
         rpt_flag = 1;
         while (n-- > 0 && (code = execute(old_c, old_f, old_n)) == TRUE)
            ;

         rpt_flag = 0;
         return code;
      }
   }
   else
   {
      rpt_flag = 1;
      while (n-- > 0 && (code = call_cmd(old_cmdptr, old_f, old_n)) == TRUE)
         ;

      rpt_flag = 0;
      return code;
   }
}

#endif


/*
** This function calls the command - it also checks for the
** "else" and "endif" commands...
*/
int call_cmd(cmdptr, f, n)
int (*cmdptr)();
int f, n;
{
   static int saved_f = 0;
   static int saved_n = 1;


   if (cmdptr != ifcmd && cmdptr != elsecmd && cmdptr != endcmd && skiplevel)
      return TRUE;

#if CTLCFIX
   if ( cmdptr != repeat_last && cmdptr != cex &&
        cmdptr != meta        && cmdptr != funcprefix )
   {
      if (LastWasPrefix)
      {
         f      = saved_f;
         n      = saved_n;

         LastWasPrefix = 0;
      }

      old_cmdptr = cmdptr;
      old_c      = -1;
      old_f      = f;
      old_n      = n;
   }
   else
      if (cmdptr != repeat_last)
      {
         saved_f = f;
         saved_n = n;
         LastWasPrefix = 1;
      }

#endif

   if (!f)
      n = 1;

   return (*cmdptr)(f, n);
}


/*
 * This is the general command execution routine. It handles the fake binding
 * of all the keys to "self-insert". It also clears out the "thisflag" word,
 * and arranges to move it to the "lastflag", so that the next command can
 * look at it. Return the status of command.
 */
execute(c, f, n)
{
   KEYTAB *ktp;
   int    status;

   ktp = &keytab[0];                       /* Look in key table.   */
   while (ktp->k_fp != NULL && ktp->k_code != c)
      ++ktp;

   if (ktp->k_fp != cex && ktp->k_fp != funcprefix && ktp->k_fp != meta)
   {
      if (kbdmip != NULL)                    /* Save macro strokes.  */
      {
         if (kbdmip > &kbdm[NKBDM-6])
         {
            mlwrite("[Out of keyboard macro space]");
            ctrlg(FALSE, 0);
            kbdmip = NULL;
            return FALSE;
         }

         if (f != FALSE)
         {
            *kbdmip++ = (CTRL|'U');
            *kbdmip++ = n;
         }

         *kbdmip++ = c;
      }
   }

   if (ktp->k_fp != NULL)
   {
      if (ktp->k_fp != cex && ktp->k_fp != funcprefix && ktp->k_fp != meta)
         thisflag = 0;

      status   = call_cmd(ktp->k_fp, f, n);
      lastflag = thisflag;
      return (status);
   }

   old_cmdptr = ktp->k_fp;
   old_c = c;
   old_f = f;
   old_n = n;

   /*
    * If a space was typed, fill column is defined, the argument is non-
    * negative, wrap mode is enabled, and we are now past fill column,
    * and we are not read-only, perform word wrap.
    */
   if (c == ' ' && (curwp->w_bufp->b_mode & MDWRAP) && fillcol > 0 &&
                   n >= 0 && getccol(FALSE) > fillcol &&
                   (curwp->w_bufp->b_mode & MDVIEW) == FALSE)
      wrapword();

   if ((c>=ZZSETP(0x20) && c<=ZZSETP(0x7E)) ||   /* Self inserting.      */
       (c>=ZZSETP(0xA0) && c<=ZZSETP(0xFE)))
   {
      if (n <= 0)                    /* Fenceposts.          */
      {
         lastflag = 0;
         return (n<0 ? FALSE : TRUE);
      }
      thisflag = 0;                   /* For the future.      */

      /* if we are in overwrite mode, not at eol,
         and next char is not a tab or we are at a tab stop,
         delete a char forword         */
      if (curwp->w_bufp->b_mode & MDOVER &&
          curwp->w_doto < curwp->w_dotp->l_used &&
          (lgetc(curwp->w_dotp, curwp->w_doto) != '\t' ||
           (curwp->w_doto) % 8 == 7))
         ldelete(1L, FALSE);

      /* do the appropriate insertion */
      if (c == '}' && (curbp->b_mode & MDCMOD) != 0)
         status = insbrace(n, "}");
      else
      if (c == ')' && (curbp->b_mode & MDLSPM))
         status = insbrace (n, ")");
      else
         if (c == '#' && (curbp->b_mode & MDCMOD) != 0)
              status = inspound();
      else
      {
         status = linsert(n, c);
      }

#if   CFENCE
      /* check for CMODE fence matching */
      if ((c == '}' || c == ')' || c == ']') &&
             (curbp->b_mode & MDCMOD) != 0)
         fmatch(c);
#endif
#if   LSPFENCE
      /* Check for LISPMODE fence matching */
      if (c == ')' && (curbp->b_mode & MDLSPM) != 0
                   && (curbp->b_mode & MDCMOD) == 0)
         fmatch (c);
#endif

      lastflag = thisflag;
      return (status);
   }

   (*term.t_beep)();
   mlwrite("[Key not bound]");             /* complain             */
   lastflag = 0;                           /* Fake last flags.     */

   return (FALSE);
}


/*
 * Fancy quit command, as implemented by Norm. If the any buffer has
 * changed do a write on that buffer and exit emacs, otherwise simply exit.
 */
quickexit(f, n)
{
    BUFFER *bp;   /* scanning pointer to buffers */

   bp = bheadp;
   while (bp != NULL) {
           if ((bp->b_flag&BFCHG) != 0   /* Changed.             */
           && (bp->b_flag&BFINVS) == 0) {   /* Real.                */
         curbp = bp;      /* make that buffer cur   */
         mlwrite("[Saving %s]",bp->b_fname);
                   filesave(f, n);
      }
   bp = bp->b_bufp;         /* on to the next buffer */
   }
        quit(f, n);                             /* conditionally quit   */
}

/*
 * Quit command. If an argument, always quit. Otherwise confirm if a buffer
 * has been changed and not written out. Normally bound to "C-X C-C".
 */
quit(f, n)
{
    int    s;

   if (f != FALSE                              /* Argument forces it.  */
       || anycb() == FALSE                     /* All buffers clean.   */
       || (s=mlyesno("Modified buffers exist. Leave anyway")) == TRUE)
   {
#if   FILOCK
      if (lockrel() != TRUE)
      {
         (*term.t_putchar)('\n');
         (*term.t_putchar)('\r');
         (*term.t_close)();
         exit(1);
      }
#endif

      vttidy();

#if   APROF
      /* if doing AZTEC C profiling, close up and write it out */
      monitor(0,0,0,0,0);
#endif

      exit(GOOD);
   }

   clrmes(0,0);
   return (s);
}

/*
 * Begin a keyboard macro.
 * Error if not at the top level in keyboard processing. Set up variables and
 * return.
 */
ctlxlp(f, n)
{
        if (kbdmip!=NULL || kbdmop!=NULL) {
                mlwrite("Not now");
                return (FALSE);
        }
        mlwrite("[Start macro]");
        kbdmip = &kbdm[0];
        return (TRUE);
}

/*
 * End keyboard macro. Check for the same limit conditions as the above
 * routine. Set up the variables and return to the caller.
 */
ctlxrp(f, n)
{
   if (kbdmop)
   {
      kbdmop = NULL;
      return TRUE;
   }

   if (kbdmip == NULL)
   {
      mlwrite("Not now");
      return (FALSE);
   }

   mlwrite("[End macro]");
   kbdmip = NULL;
   return (TRUE);
}

/*
 * Execute a macro.
 * The command argument is the number of times to loop. Quit as soon as a
 * command gets an error. Return TRUE if all ok, else FALSE.
 */
ctlxe(f, n)
{
   int    c;
   int    af;
   int    an;
   int    s;

   int (*tmp_cmdptr)();
   int tmp_c;
   int tmp_f;
   int tmp_n;


   tmp_cmdptr = ctlxe;
   tmp_f      = f;
   tmp_n      = n;


   if (kbdmip!=NULL || kbdmop!=NULL)
   {
      mlwrite("[Can't execute keyboard macro now]");
      return (FALSE);
   }

   if (n <= 0)
      return (TRUE);
   do
   {
      kbdmop = &kbdm[0];

      do
      {
         af = FALSE;
         an = 1;

         if ((c = *kbdmop++) == (CTRL|'U'))
         {
            af = TRUE;
            an = *kbdmop++;
            c  = *kbdmop++;
         }

         s = TRUE;
      } while ( (s=execute(c, af, an) == TRUE) && kbdmop );

      kbdmop = NULL;
   } while (s==TRUE && --n);

   old_cmdptr = tmp_cmdptr;
   old_f      = tmp_f;
   old_c      = tmp_c;
   old_n      = tmp_n;

   global_ctrlx_flag = 0;

   return (s);
}

/*
 * Abort.
 * Beep the beeper. Kill off any keyboard macro, etc., that is in progress.
 * Sometimes called as a routine, to do general aborting of stuff.
 */
ctrlg(f, n)
{
        (*term.t_beep)();
        if (kbdmip != NULL) {
                kbdm[0] = (CTLX|')');
                kbdmip  = NULL;
        }
   mlwrite("[Aborted]");
        return (ABORT);
}

/* tell the user that this command is illegal while we are in
   VIEW (read-only) mode            */

rdonly()

{
   (*term.t_beep)();
   mlwrite("[Key illegal in VIEW mode]");
   return(FALSE);
}


meta()   /* dummy function for binding to meta prefix */
{
   global_meta_flag = 1;
}

cex()   /* dummy function for binding to control-x prefix */
{
   global_ctrlx_flag = 1;
}


#if FUNCFIX
/* Special function prefix characters fix ...... */
int GlobalFuncFlag=0;
funcprefix(f,n)
{
   GlobalFuncFlag=1;
}
#endif


#if PRIMOS

fortran void quit_handler(cp)
int *cp;
{
   if (emulate_ctrl_p)
   {
      if (next_key == CTRL | 'P')
         next_arg++;
      else
      {
         next_key = CTRL|'P';
         next_arg = 1;
      }
   }
   else
   {
      (*term.t_beep)();
      spawncli(1,1);
      updgar();            /* Update screen */
   }
}

fortran void logout_handler(cp)
int *cp;
{
   BUFFER *bp;   /* scanning pointer to buffers */
   int f,n;

   mlwrite("[Logout has been signalled]");
   (*term.t_beep)();
   bp = bheadp;
   while (bp != NULL) {
           if ((bp->b_flag&BFCHG) != 0   /* Changed.             */
           && (bp->b_flag&BFINVS) == 0) {   /* Real.                */
         curbp = bp;      /* make that buffer cur   */
         strcat(bp->b_fname,logout_name);
         mlwrite("[Logout in progress, saving %s]",bp->b_fname);
                   filesave(f, n);
      }
   bp = bp->b_bufp;         /* on to the next buffer */
   }
   (*term.t_flush)();
   (*term.t_close)();
   logo$$(0,0,0,0,0,0);
}


fortran void any_handler(cfh)
int *cfh;
{
   (*term.t_flush)();
   (*term.t_close)();
}

fortran void dummy_handler(cfh)
int *cfh;
{
   fortran void cnsig$();
   short no_onunit;

   cnsig$(no_onunit);
   if (no_onunit)
      any_handler(cfh);
}

#endif


#if MREPFIX
int setrep1arg(f, n)
int f, n;
{
}
#endif


#if CTLUFIX
int setrep4arg(f, n)
int f, n;
{
   f = TRUE;
   n = 4;                          /* with argument of 4 */
   mflag = 0;                      /* that can be discarded. */
   mlwrite("Arg: 4");
                while ((c=getcmd()) >='0' && c<='9' ||
                        c==(CTRL|'U') || c=='-'){
                        if (c == (CTRL|'U'))
                                n = n*4;
                        /*
                         * If dash, and start of argument string, set arg.
                         * to -1.  Otherwise, insert it.
                         */
                        else if (c == '-') {
                                if (mflag)
                                        break;
                                n = 0;
                                mflag = -1;
                        }
                        /*
                         * If first digit entered, replace previous argument
                         * with digit and set sign.  Otherwise, append to arg.
                         */
                        else {
                                if (!mflag) {
                                        n = 0;
                                        mflag = 1;
                                }
                                n = 10*n + c - '0';
                        }
                        mlwrite("Arg: %d", (mflag >=0) ? n : (n ? -n : -1));
                }
                /*
                 * Make arguments preceded by a minus sign negative and change
                 * the special argument "^U -" to an effective "^U -1".
                 */
                if (mflag == -1) {
                        if (n == 0)
                                n++;
                        n = -n;
                }
}
#endif
