/*
** PRIMOS.C                                  Prime/PRIMOS specific routines.
**
** (c) 1989 Findata Finansiell Information AB.
**
** Date     Programmer     History
** -------- -------------- -----------------------------------------------------
** 890511   P. Eriksson    Initial coding.
**                            Inserted the Path2Primos() function.
**                               (Supports Unix, Qnx, MsDos and Primos)
**                            Inserted AsyncInfo function.
**                               (For the Rev.22 async routines)
**                            Inserted the AbbrevsEnabled() function.
**                               (Converted from SPL version)
**                            Inserted the EvaluateComLine() function.
**                               (Supports abbrevs and gvars)
** 890514   P. Eriksson    Created the ScanSearchList() function and
**                         incorporated it into Path2Primos().
** 890521   P. Eriksson    Some bugfixing in ScanSearchList().
**                         Added '~' (IAP) functionality in Path2Primos().
**                         Added SearchListName variable.
*/

#include "estruct.h"

#if PRIMOS

#include <stdio.h>
#include <string.h>
#include <keys.h>
#include <errd.h>
#include <charvar.h>
#include <primos/std.h>
#include <primos/stdos.h>
#include <primos/ndir.h>
#include "edef.h"


#define GVARAREA (char *) (06005 * 128 * 1024)
#define PL1NULL  (char *) (07777 * 128 * 1024)

extern CLDATA cldata;


char SearchListName[33] = "EMACS$";


static isdelim(x)
{
   return( x == '\\' || x == '/' );
}


char *chomeid(s)
char *s;
{
   fortran void GPATH$();
   static char homepath[257];
   short length, code;


   if (!s)
      s = homepath;

   GPATH$(k$inia, 0, (char []) s, 256, length, code);
   if (code)
   {
      errno = code;
      perror("ERROR (GPATH$)");
      return NULL;
   }

   s[length] = '\0';

   return s;
}


void Path2Primos(inpath, outpath)
char *inpath, *outpath;
{
   char *tmp, *top, *first, *outfirst;
   char inpath2[1025];
   static char *getvol(), *getpath();
   extern int u2p_flag;


   if (!u2p_flag)
   {
      ScanSearchList(inpath, outpath);
      return;
   }

   outfirst = outpath;
   tmp = inpath;

   if (tmp[0] == '*' && tmp[1] == '<')    /* Prymos tree-pop - yack! */
      while (*tmp)
         ++tmp;

   while (*tmp == '.')
      ++tmp;

   while ( *tmp &&
           *tmp != '<' && *tmp != '>' &&
           !isdelim(*tmp) && *tmp != ':' && *tmp != '~')
      tmp++;


   if ( (!*tmp && inpath[0] != '*' && inpath[1] != '<' && inpath[0] != '.') ||
        *tmp == '<' || *tmp == '>' ) /* Primos pathname - don't do nothing */
   {
      ScanSearchList(inpath, outpath);
      return;
   }

   top = first = outpath;
   tmp = inpath;

   if (*inpath == '~')     /* relative home directory */
   {
      inpath++;
      while (isdelim(*inpath))
         inpath++;

      strcpy(outpath, chomeid(NULL));
      while (*outpath)
         outpath++;
   }
   else
   {
      while (*tmp && *tmp != ':')      /* device/disk specified? */
         tmp++;

      if (*tmp)
      {
         /*
         ** Yes, append disk to pathname
         */

         *outpath++ = '<';
         while (*inpath != ':')
            *outpath++ = *inpath++;
         *outpath++ = '>';
         if (isdelim(*(++inpath)))
            inpath++;
      }
      else
         if (isdelim(*inpath))      /* top level directory */
            inpath++;
         else                       /* relative pathname */
         {
            strcpy(outpath, getpath());
            while (*outpath)
               outpath++;
         }
   }


   if (*top == '<')
      while (*top++ != '>');

   while (*inpath)
   {
      if (*inpath != '.' && *inpath != '^' &&
          !(inpath[0] == '*' && inpath[1] == '<'))    /* tree-pop ? */
      {
         /*
         ** Nop.
         */
         if (top != outpath)
            *outpath++ = '>';
         while (*inpath && !isdelim(*inpath))
            *outpath++ = *inpath++;
      }
      else
      {
         /*
         ** Yes, scan backwards...
         */
         if (*inpath == '.')
         {
            if (*++inpath != '.')
               goto CUR_UFD;
         }
         else if (*inpath == '*')
            ++inpath;

         do
         {
            while (outpath > top && *--outpath != '>');
         }
         while (*++inpath == '.' || *inpath == '^' || *inpath == '<');

CUR_UFD:
         if (outpath == first)
         {
            strcpy(outpath, getvol());
            while (*outpath)
               outpath++;
            top = outpath;
         }
      }

      if (isdelim(*inpath))
         inpath++;
   }

   *outpath++ = '\0';

   ScanSearchList(outfirst, inpath2);
   strcpy(outfirst, inpath2);
}


static char *getvol()
{
   fortran void GPATH$();

   static char path[1025];
   short length, code;
   char *ptr;

   GPATH$(k$cura, 0, path, 1024, length, code);

   if (code)
      return(NULL);
   else
   {
      ptr = path;
      while (*ptr++ != '>');
      *ptr = '\0';
      return(path);
   }
}


static char *getpath()
{
   fortran void GPATH$();

   static char path[1025];
   short length, code;
   char *ptr;

   GPATH$(k$cura, 0, path, 1024, length, code);

   if (code)
      return(NULL);
   else
   {
      ptr = path;
      while (*ptr++ != '>');
      path[length] = '\0';
      return(ptr);
   }
}


int AbbrevsEnabled()
{
   return cldata.flags.abbrev_on != 0;
}




get_output_size(line_no)
int line_no;
{
   return -2;
}


get_line_no(user_no)
int user_no;
{
   fortran AS$LIN();
   short lineno;
   short code;

   code = 0;
   AS$LIN(lineno, code);
   if (code)
      return -1;
   else
      return lineno;
}




EvaluateComLine(max, text)
char *text;
{
   short error = 0;
   short com_error = 0;
   charvar(1024) cmdline;
   fortran void EVAL_A();


   if (strlen(text) > 1024 || max > 1024)
      com_error = e$bfts;
   else
   {
      setcvar(cmdline, text);
      EVAL_A(cmdline, 0, PL1NULL, GVARAREA, (short) max, error, com_error);
      getcvar(text, cmdline);
   }

   return (com_error);
}

set_protocol(protocol)
char *protocol;
{
   char protostr[200];

   strcpy(protostr,"SET_ASYNC -PROTOCOL ");
   strcat(protostr,protocol);

   system("COMO -NTTY");
   system(protostr);
   system("COMO -TTY");
}

#define e$ok   0


#  if !C_FIO


/*
** MUST link with MSCLIB!
*/
extern void *_alloca();
#define alloca _alloca


extern int dam_mode;
extern int compress;

fortran int SRSFX$();   /* Open or Close a file                */
fortran void WTLIN$();  /* Write a line of text to a file      */
fortran void RDLIN$();  /* Read a line of text from a file     */
fortran void PRWF$$();  /* Position and truncate files         */
fortran void ERTXT$();  /* Get PRIMOS error text               */

typedef struct
{
   short length;
   char  text[1025];
} char1024var;

typedef struct
{
   short length;
   char  text[128];
} char128var;

typedef struct
{
   short length;
   char  text[32];
} char32var;

short fileunit;         /* File unit used by the I/O routines  */
short filekeys;         /* Keys used to open the file          */


char32var suffix_list[] =
{
   {  2, ".Y"              },
   {  2, ".C"              },
   {  3, ".CC"             },
   {  2, ".H"              },
   {  7, ".INS.CC"         },
   {  9, ".H.INS.CC"       },
   {  4, ".MOD"            },
   {  7, ".MODULA"         },
   {  7, ".PASCAL"         },
   {  4, ".PAS"            },
   {  2, ".P"              },
   {  4, ".CPL"            },
   {  5, ".COMI"           },
   {  5, ".COMO"           },
   {  4, ".FTN"            },
   {  4, ".F77"            },
   {  4, ".CBL"            },
   {  6, ".COBOL"          },
   {  6, ".BASIC"          },
   {  4, ".BAS"            },
   {  4, ".PMA"            },
   {  4, ".CMA"            },
   {  4, ".ASM"            },
   {  4, ".PL1"            },
   {  5, ".PL1G"           },
   {  5, ".PL/1"           },
   {  4, ".PLP"            },
   {  4, ".SPL"            },
   {  6, ".MACRO"          },
   {  4, ".RPG"            },
   {  4, ".DOC"            },
   {  4, ".TXT"            },
   {  5, ".TEXT"           },
   {  4, ".PRO"            },
   {  7, ".PROLOG"         },
   {  4, ".LSP"            },
   {  5, ".LISP"           },
   {  4, ".INS"            },
   {  3, ".SR"             },
   {  5, ".HPGL"           },
   {  6, ".HP/GL"          },
   {  4, ".OVL"            },
   {  5, ".LIST"           },
   {  3, ".EM"             },
   {  5, ".RUNI"           }
};


/*
** Get error message
*/
char *errormsg(code)
int code;
{
   static char1024var msg;

   ERTXT$( (short) code, msg);
   msg.text[msg.length] = '\0';
   return msg.text;
}

/*
** Handle file open/close
*/
static int filectrl(filename, keys)
char *filename;
int keys;
{
   char128var pathname;
   char32var  basename;
   short      filetype;
   short      suffix_used;
   short      code;


   strncpy(pathname.text, filename, pathname.length = strlen(filename));
   filekeys = keys;

   SRSFX$( filekeys,
           pathname,
           fileunit,
           filetype,
           0,
           suffix_list, /* not used in this routine... */
           basename,
           suffix_used,
           code );

   errno = code;
   switch (code)
   {
      case e$ok:
         return FIOSUC;

      case e$fntf:
         return FIOFNF;

      default:
         mlwrite("Error: %s", errormsg(code));
         return FIOERR;
   }
}


/*
** Verify existance of a file system object
*/
int fexists(filename)
char *filename;
{
   char128var pathname;
   char32var  basename;
   short      filetype;
   short      suffix_used;
   short      code;


   strncpy(pathname.text, filename, pathname.length = strlen(filename));

   SRSFX$( (short) k$exst,
           pathname,
           fileunit,
           filetype,
           0,
           suffix_list,    /* Dummy usage - not used here... */
           basename,
           suffix_used,
           code );

   errno = code;
   return !code;
}



int fix_file_suffix(filename)
char *filename;
{
   char128var pathname;
   char32var  basename;
   short      filetype;
   short      suffix_used;
   short      code;

#if USEPSR
   return FIOSUC;
#endif
   if (fexists(filename))
      return FIOSUC;

   strncpy(pathname.text, filename, pathname.length = strlen(filename));

   SRSFX$( (short) k$exst,
           pathname,
           fileunit,
           filetype,
           (short) ( sizeof( suffix_list ) / sizeof( char32var )),
           suffix_list,
           basename,
           suffix_used,
           code );

   if (code == e$ok && suffix_used)
      strncat(filename, suffix_list[suffix_used-1].text,
                        suffix_list[suffix_used-1].length);

   strlower(filename);
   errno = code;
   switch (code)
   {
      case e$ok:
         return FIOSUC;

      case e$fntf:
         return FIOFNF;

      default:
         return FIOERR;
   }
}


/*
** Open a file for reading.
**/
ffropen(filename)
char *filename;
{
   return filectrl(filename, (k$read | k$getu));
}

/*
** Open a file for writing. Return TRUE if all is well, and FALSE on error
** (cannot create).
**/
ffwopen(filename)
char *filename;
{
   int status;

   status = filectrl(filename, (k$writ|k$getu|(dam_mode?k$ndam:k$nsam)));
   if (status == FIOFNF)
   {
      mlwrite("[Directory not found]");
      return FIOERR;
   }
   else
      return status;
}


/*
** Open a file for writing at the end of it. Return TRUE if all is well,
** and FALSE on error.
**/
ffaopen(filename)
char *filename;
{
   short rnhw, code;


   if (code = filectrl(filename, (k$writ|k$getu|(dam_mode?k$ndam:k$nsam))))
      return code;

   do
   {
      PRWF$$( (short) k$posn,
              fileunit,
              (void *) NULL,
              0,
              1000000L,
              rnhw,
              code );
   } while (!code);

   errno = code;
   if (code != e$eof)
      mlwrite("Error: %s", errormsg(code));
   return (code == e$eof ? FIOSUC : FIOERR);
}

/*
** Close a file. Should look at the status in all systems.
**/
ffclose()
{
   short rnhw, code;


   if (filekeys & k$writ)
   {
      PRWF$$( (short) k$trnc,
              fileunit,
              (void *) NULL,
              0,
              0L,
              rnhw,
              code );
      errno = code;
   }

   return filectrl("", k$clos);
}

/*
** Write a line to the already opened file. The "buf" points to the buffer,
** and the "nbuf" is its length, less the free newline. Return the status.
** Check only at the newline.
**/
ffputline(buffer, length)
char *buffer;
int length;
{
   char *ptr;
   short code;


   if ((ptr = (char *) alloca(length + 1)) == NULL)
   {
      mlwrite("Error: %s", errormsg(errno));
      return FIOERR;
   }

   strncpy(ptr, buffer, length);
   if (length & 1)
      ptr[length] = ' ';

   WTLIN$( fileunit,
           (char []) ptr,
           (short) ((length + 1) / 2),
           code );

   errno = code;
   if (code)
      mlwrite("Error: %s", errormsg(errno));

   return (code ? FIOERR : FIOSUC);
}


/*
** Read a line from a file, and store the bytes in the supplied buffer. The
** "nbuf" is the length of the buffer. Complain about long lines and lines
** at the end of the file that don't have a newline present. Check for I/O
** errors too. Return status.
**/
ffgetline(buffer, length)
char *buffer;
int length;
{
   char *ptr;
   short code;
   int index;


   if ((ptr = (char *) alloca(length + 1)) == NULL)
   {
      mlwrite("Error: %s", errormsg(errno));
      return FIOERR;
   }

   RDLIN$( fileunit,
           (char []) ptr,
           (short) ((length + 1) / 2),
           code );

   errno = code;
   if (errno && errno != e$eof)
      mlwrite("Error: %s", errormsg(errno));

   for (index = ((length+1)&~1); index > 0 && ptr[index-1] == ' '; index--);

   strncpy(buffer, ptr, length);
   if (index < length)
      buffer[index] = 0;

   switch (code)
   {
      case e$ok:
         return FIOSUC;

      case e$eof:
         return FIOEOF;

      default:
         return FIOERR;
   }
}

#  endif


int ScanSearchList(inpath, outpath)
char *inpath;
char *outpath;
{
   fortran void OPSRS$();

   charvar(32)       ListName;
   charvar(128)      ObjectName;
   charvar(32)       BaseName;
   charvar(128)      FoundPath;
   charvar(128)      RefDir;

   short             Funit;
   short             SuffixSize;
   short             SuffixUsed;
   short             Code;
   short             Type;

   extern int        UseSearchList;

   if (!UseSearchList)
   {
      strcpy(outpath, inpath);
      return 0;
   }

#if FIXSUFF
   SuffixSize = sizeof( suffix_list ) / sizeof( char32var );
#else
   SuffixSize = 0;
#endif

   Code = 0;
   setcvar(ListName,    SearchListName);
   setcvar(ObjectName,  inpath);
   setcvar(RefDir,      "");


   OPSRS$(
      ListName,               /* Search list name                    */
      RefDir,                 /* Char(*) var, for reference dir      */
      (short) k$file,         /* Type of file to look for.           */
      (short) k$exst,         /* We want to know if it exists        */
      ObjectName,             /* The object to search for.           */
      Funit,                  /* File unit (not used).               */
      Type,                   /* Type of file found.                 */
      SuffixSize,             /* Nr of suffixes.                     */
      (void *) suffix_list,   /* List of suffixes.                   */
      BaseName,               /* The basename returned.              */
      SuffixUsed,             /* The suffix used.                    */
      FoundPath,              /* The complete pathname (if found)    */
      Code );                 /* Standard error code.                */

   if (Code)
      strcpy(outpath, inpath);
   else
      getcvar(outpath, FoundPath);

   strlwr(outpath);

   return !Code;
}


int isfile(t)
int t;
{
   return t == 0 || t == 1;
}


char *ExpandWild(path)
char *path;
{
   static Dir *dh          = NULL;
   static char WildName[33];
   static char PathName[257];
   static char OutName[257];
   char *p;
   int   s;


   if (path)
   {
      /*
      ** Any WildCards?
      */
      if (!iswild (path))
         return path;

      strcpy(PathName, path);

      if (p = strrchr(PathName, '>'))   /* non-current directory ? */
      {
         path = PathName;
         *p++ = '\0';
      }
      else
      {
         p = PathName;
         path = NULL;      /* current ufd */
      }

      strcpy(WildName, p);

      if ((dh = DirOpen (path)) == NULL)
         return NULL;

      if (p == PathName)
         PathName[0] = '\0';
      else
         strcat(PathName, ">");
   }

   if (dh == NULL)      /* really should not occur... */
      return NULL;

   while ((s = DirSearch (WildName, 0, dh)) && !isfile (_type (dh)))
      ;

   if (s)
   {
      strcpy(OutName, PathName);
      strcat(OutName, strlwr(_name (dh)));
      return OutName;
   }

   DirClose (dh);
   dh = NULL;
   return NULL;
}


/*
** File Name Completion Handler.
**
** Returns:
**
**    -1 .... Not supported, or path not found.
**     0 .... Filename completion complete.
**    >0 .... There are more than one matching pathname.
*/
int FileNameCompletion(Path)
char *Path;
{
   char PathBuf[257];
   char TmpBuf[257];
   char LastName[33];
   char MatchName[33];
   char *PathDir;
   char *p, *p2;
   char *FileName;
   Dir * dp;
   int  LastType;
   int  counter;
   int  DirDelimiter;
   int  l;


   strcpy (PathBuf, Path);

   if ( (u2p_flag &&
           ( (p = strrchr(PathBuf, '/')) || (p = strrchr(PathBuf, '\\')) ) ) ||
        (p = strrchr(PathBuf, '>')) )
   {
      DirDelimiter = *p;

      /*
      ** In another directory...
      */
      PathDir = PathBuf;
      *p++ = '\0';
      FileName = Path + (p - PathBuf);
   }
   else
   {
      /*
      ** Current directory...
      */

      DirDelimiter = '>';
      PathDir = NULL;
      p = PathBuf;
      FileName = Path;
   }

   strcat(p, "@@");

   if (PathDir && u2p_flag)
   {
      /* Convert the directory pathname */
      Path2Primos(PathDir, TmpBuf);
      PathDir = TmpBuf;
   }

   if ((dp = DirOpen (PathDir)) == NULL)
      return -1;  /* illegal directory */

   counter = 0;
   while (DirSearch (p, 0, dp))
   {
      counter++;
      strcpy(LastName, _name (dp));
      LastType = _type (dp);
      if (counter == 1)
         strcpy(MatchName, LastName);
      else
      {
         p2 = MatchName + strdiff(LastName, MatchName);
         *p2 = '\0';
      }
   }

   strlwr(MatchName);

   if (counter == 0)
   {
      DirClose (dp);
      return -1;
   }
   else
   {
      strcpy(FileName, MatchName);
      if (counter == 1 && LastType == 4)
      {
         FileName[l = strlen(FileName)] = DirDelimiter;
         FileName[l+1] = '\0';
      }
      DirClose (dp);
      return counter - 1;
   }
}

int strdiff(s1, s2)
char *s1;
char *s2;
{
   int i;


   for (i = 0; *s1 && tolower(*s1) == tolower(*s2); s1++, s2++, i++)
      ;

   return i;
}


int CtrlMessage(state)
int state;
{
   return 0;
}


int ReadMessage(buffer)
char *buffer;
{
   return 0;
}


int sendmsg(f, n)
{
   char name[33];
   char text[81];
   int  s;


   return 0;
}



#endif   /* PRIMOS */

