
/*
 * This file contains the command processing functions for a number of random
 * commands. There is no functional grouping here, for sure.
 */

#include        <stdio.h>
#include        "estruct.h"
#include        "edef.h"

int     tabsize
#ifdef PRIMOS
                = 3
#endif
;                        /* Tab size (0: use real tabs)  */

extern char EditorName[];

/*
 * Display a line of text on the message line
 */
sayline(f, n)
{
   char text[256];
   int retcode;

   /* Prompt for the text */
   if ((retcode = mlreply("Text to display: ", text, 255)) != TRUE)
   {
      clrmes(0,0);
      return (retcode);
   }

   mlwrite("%s", text);
   return TRUE;
}


/*
 * Set fill column to n.
 */
setfillcol(f, n)
{
   char number[21];
   int retcode;

   /* Have we used any argument? */
   if (!f)
   {
      /* Prompt for the line number */
      if ((retcode = mlreply("Fill column: ", number, 20)) != TRUE)
      {
         clrmes(0,0);
         return (retcode);
      }

      n = atoi(number);
   }

   fillcol = n;
   mlwrite("[Fill column is %d]",n);
   return(TRUE);
}

/*
 * Display the current position of the cursor, in origin 1 X-Y coordinates,
 * the character that is under the cursor (in hex), and the fraction of the
 * text that is before the cursor. The displayed column is not the current
 * column, but the column that would be used on an infinite width display.
 * Normally this is bound to "C-X =".
 */
showcpos(f, n)
{
         LINE   *lp;      /* current line */
         long   numchars;   /* # of chars in file */
         int   numlines;   /* # of lines in file */
         long   predchars;   /* # chars preceding point */
         int   predlines;   /* # lines preceding point */
         int    curchar;   /* character under cursor */
        int ratio;
        int col;
   int savepos;         /* temp save for current offset */
   int ecol;         /* column pos/end of current line */

   /* starting at the begining of the buffer */
        lp = lforw(curbp->b_linep);

   /* start counting chars and lines */
        numchars = 0;
        numlines = 0;
        while (lp != curbp->b_linep) {
      /* if we are on the current line, record it */
      if (lp == curwp->w_dotp) {
         predlines = numlines;
         predchars = numchars + curwp->w_doto;
         if ((curwp->w_doto) == llength(lp))
            curchar = '\n';
         else
            curchar = lgetc(lp, curwp->w_doto);
      }
      /* on to the next line */
      ++numlines;
      numchars += llength(lp) + 1;
      lp = lforw(lp);
        }

   /* if at end of file, record it */
   if (curwp->w_dotp == curbp->b_linep) {
      predlines = numlines;
      predchars = numchars;
   }

   /* Get real column and end-of-line column. */
   col = getccol(FALSE);
   savepos = curwp->w_doto;
   curwp->w_doto = llength(curwp->w_dotp);
   ecol = getccol(FALSE);
   curwp->w_doto = savepos;

        ratio = 0;              /* Ratio before dot. */
        if (numchars != 0)
                ratio = (100L*predchars) / numchars;

   /* summarize and report the info */
   mlwrite("Line %d/%d Col %d/%d Char %D/%D (%d%%) char = 0x%x",
      predlines+1, numlines+1, col, ecol,
      predchars, numchars, ratio, curchar);
        return (TRUE);
}

/*
 * Return current column.  Stop at first non-blank given TRUE argument.
 */
getccol(bflg)
int bflg;
{
        int c, i, col;
        col = 0;
        for (i=0; i<curwp->w_doto; ++i) {
                c = lgetc(curwp->w_dotp, i);
                if (c!=' ' && c!='\t' && bflg)
                        break;
                if (c == '\t')
                        col |= 0x07;
                else if (c<ZZSETP(0x20) || c==ZZSETP(0x7F))
                        ++col;
                ++col;
        }
        return(col);
}

/*
 * Twiddle the two characters on either side of dot. If dot is at the end of
 * the line twiddle the two characters before it. Return with an error if dot
 * is at the beginning of line; it seems to be a bit pointless to make this
 * work. This fixes up a very common typo with a single stroke. Normally bound
 * to "C-T". This always works within a line, so "WFEDIT" is good enough.
 */
twiddle(f, n)
{
   LINE   *dotp;
   int    doto;
   int    cl;
   int    cr;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */

   dotp = curwp->w_dotp;
   doto = curwp->w_doto;

   if (doto==llength(dotp) && --doto<0)
      return (FALSE);

   cr = lgetc(dotp, doto);
   if (--doto < 0)
      return (FALSE);

   cl = lgetc(dotp, doto);
   lputc(dotp, doto+0, cr);
   lputc(dotp, doto+1, cl);
   lchange(WFEDIT);

   return (TRUE);
}

/*
 * Quote the next character, and insert it into the buffer. All the characters
 * are taken literally, with the exception of the newline, which always has
 * its line splitting meaning. The character is always read, even if it is
 * inserted 0 times, for regularity. Bound to "C-Q"
 */
quote(f, n)
{
         int    s;
         int    c;

        if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
           return(rdonly());        /* we are in read only mode   */
        mlwrite("[Quote]");
        c = (*term.t_getchar)();
        clrmes(0,0);
        if (!f)
         n = 1;

        if (n < 0)
                return (FALSE);
        if (n == 0)
                return (TRUE);
        if (c == '\n') {
                do {
                        s = lnewline();
                } while (s==TRUE && --n);
                return (s);
        }
        return (linsert(n, c));
}

/*
 * Set tab size if given non-default argument (n <> 1).  Otherwise, insert a
 * tab into file.  If given argument, n, of zero, change to true tabs.
 * If n > 1, simulate tab stop every n-characters using spaces. This has to be
 * done in this slightly funny way because the tab (in ASCII) has been turned
 * into "C-I" (in 10 bit code) already. Bound to "C-I".
 *
 */
tab(f, n)
{
        if (n < 0)
                return (FALSE);
        if (n == 0 || n > 1) {
                tabsize = n;
                return(TRUE);
        }

        if (! tabsize)
                return(linsert(1, '\t'));

        return(linsert(tabsize - (getccol(FALSE) % tabsize), ' '));
}

backtab(f, n)
{
   int curcol;
   int nb, sz;


   if (!f)
      n = 1;

   sz = tabsize ? tabsize : 1;

   if (n < 0)
      while (n++ < 0)
         ldelete(sz, FALSE);
   else
      while (n--)
      {
         if ((curcol = getccol (FALSE)) == 0)
               return 0;

         nb = curcol % tabsize;

         if (nb == 0)
            nb = tabsize;

         if (curcol - nb < 0)
            nb = curcol;

         backchar (f, nb);
         ldelete (nb, FALSE);
      }

   return TRUE;
}

/*
 * Open up some blank space. The basic plan is to insert a bunch of newlines,
 * and then back up over them. Everything is done by the subcommand
 * procerssors. They even handle the looping. Normally this is bound to "C-O".
 */
openline(f, n)
{
         int    i;
         int    s;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        if (n < 0)
                return (FALSE);
        if (n == 0)
                return (TRUE);
        i = n;                                  /* Insert newlines.     */
        do {
                s = lnewline();
        } while (s==TRUE && --i);
        if (s == TRUE)                          /* Then back up overtop */
                s = backchar(f, n);             /* of them all.         */
        return (s);
}

/*
 * Insert a newline. Bound to "C-M". If we are in CMODE or INDENT, do
 * automatic indentation as specified.
 */
newline(f, n)
{
    int    s;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
   if (n < 0)
      return (FALSE);

   if (n == 1 && curwp->w_dotp != curbp->b_linep)
   {
      /* C indendation */
      if (curbp->b_mode & MDCMOD)
         return (lindent ("{", "}", 1, 1));
      else
      /* Lisp indentation */
      if (curbp->b_mode & MDLSPM)
         return (lindent ("(", ")", 1, 1));
      else
      /* INDENT indentation */
      if (curbp->b_mode & MDINDENT)
         return (indent (f,n));
   }

   /* insert some lines */
   while (n--) {
      if ((s=lnewline()) != TRUE)
         return (s);
   }
   return (TRUE);
}


/*
 * The LineFeed key command handler.
 */
linefeed(f, n)
{
   extern int aprot_mode;

#if !PRIMOS
   return indent(f, n);
#else
   if (aprot_mode)
      return indent(f, n);
   else
      return newline(f, n);
#endif
}


strinsert(n, c)
char *c;
{
   char *tmp;
   int status;

   while (n--)
   {
      tmp = c;
      while (*tmp)
         status = linsert(1, *tmp++);
   }

   return (status);
}


insbrace(n, c)   /* insert a brace into the text here...we are in CMODE */

int n;   /* repeat count */
char *c;   /* brace to insert (always { for now) */

{
    int ch;   /* last character before input */
    int i;
    int target;   /* column brace should go after */

   /* if we are at the begining of the line, no go */
   if (curwp->w_doto == 0)
      return(strinsert(n,c));

   /* scan to see if all space before this is white space */
   for (i = curwp->w_doto - 1; i >= 0; --i) {
      ch = lgetc(curwp->w_dotp, i);
      if (ch != ' ' && ch != '\t')
         return(strinsert(n, c));
   }

   /* delete back first */
   target = getccol(FALSE);   /* calc where we will delete to */
   target -= 1;
   target -= target % (tabsize == 0 ? 8 : tabsize);
   while (getccol(FALSE) > target)
      backdel(FALSE, 1);

   /* and insert the required brace(s) */
   return(strinsert(n, c));
}

inspound()   /* insert a # into the text here...we are in CMODE */

{
    int ch;   /* last character before input */
    int i;

   /* if we are at the begining of the line, no go */
   if (curwp->w_doto == 0)
      return(linsert(1,'#'));

   /* scan to see if all space before this is white space */
   for (i = curwp->w_doto - 1; i >= 0; --i) {
      ch = lgetc(curwp->w_dotp, i);
      if (ch != ' ' && ch != '\t')
         return(linsert(1, '#'));
   }

   /* delete back first */
   while (getccol(FALSE) >= 1)
      backdel(FALSE, 1);

   /* and insert the required pound */
   return(linsert(1, '#'));
}

/* Delete spaces and empty lines around cursor */
decblank (f, n)
{
   LINE *flp;
   int  fdoto;
   LINE *elp;
   int  edoto;
   int  ready;
   int  count;
   char ch;

   if (curbp->b_mode & MDVIEW)
      return (rdonly ());

   if (llength (curwp->w_dotp) &&
       curwp->w_doto != llength (curwp->w_dotp) &&
       lgetc (curwp->w_dotp, curwp->w_doto) != ' '
      )
      return 0;

   flp = curwp->w_dotp;
   fdoto = curwp->w_doto;
   count = 0;

   ready = FALSE;
   while (!ready)
   {
      while (fdoto > 0 && (ch = lgetc (flp, fdoto-1)) == ' ' || ch == '\t')
      {
         fdoto--;
         count++;
      }

      if (fdoto || lback (flp) == NULL)
         ready = TRUE;
      else
      {
         flp = lback (flp);
         fdoto = llength (flp);
         count++;
      }
   }

   ready = FALSE;
   elp   = curwp->w_dotp;
   edoto = curwp->w_doto;

   while (!ready)
   {
      while (edoto < llength (elp) && (ch = lgetc (elp, edoto+1)) == ' '
             || ch == '\t')
      {
         count++;
         edoto++;
      }

      if (edoto != llength (elp) || lforw (elp) == NULL)
         ready = TRUE;
      else
      {
         elp = lforw (elp);
         edoto = 0;
         count++;
      }
   }

   curwp->w_dotp = flp;
   curwp->w_doto = fdoto;

   return ldelete (count, TRUE);
}



/*
 * Delete blank lines around dot. What this command does depends if dot is
 * sitting on a blank line. If dot is sitting on a blank line, this command
 * deletes all the blank lines above and below the current line. If it is
 * sitting on a non blank line then it deletes all of the blank lines after
 * the line. Normally this command is bound to "C-X C-O". Any argument is
 * ignored.
 */
deblank(f, n)
{
         LINE   *lp1;
         LINE   *lp2;
        long nld;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        lp1 = curwp->w_dotp;
        while (llength(lp1)==0 && (lp2=lback(lp1))!=curbp->b_linep)
                lp1 = lp2;
        lp2 = lp1;
        nld = 0;
        while ((lp2=lforw(lp2))!=curbp->b_linep && llength(lp2)==0)
                ++nld;
        if (nld == 0)
                return (TRUE);
        curwp->w_dotp = lforw(lp1);
        curwp->w_doto = 0;
        return (ldelete(nld, FALSE));
}

/* This routine was made primarily for use with PRIMOS.
** Some compilers doesn't like tabs, and thus I had to remove usage of it.
** Instead of changing the indent routine, I made a new, used by newline.
** Anders Furuhed, April 1988
*/
indent (f,n)
{
    char   *txt;
    short  i;

   i = 0;
   txt = &curwp->w_dotp->l_text[0];
   while ((i < curwp->w_doto) && (txt[i] == ' ' || txt[i] == '\t')
           && i <= NSTRING
         )
      i++;

   if (lnewline () == FALSE)
      return FALSE;

   if (i)
      linsert (i, ' ');
}

/*
 * Delete forward. This is real easy, because the basic delete routine does
 * all of the work. Watches for negative arguments, and does the right thing.
 * If any argument is present, it kills rather than deletes, to prevent loss
 * of text if typed with a big argument. Normally bound to "C-D".
 */
forwdel(f, n)
{
   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        if (n < 0)
                return (backdel(f, -n));
        if (f != FALSE) {                       /* Really a kill.       */
                if ((lastflag&CFKILL) == 0)
                        kdelete();
                thisflag |= CFKILL;
        }
        return (ldelete((long)n, f));
}

/*
 * Delete backwards. This is quite easy too, because it's all done with other
 * functions. Just move the cursor back, and delete forwards. Like delete
 * forward, this actually does a kill if presented with an argument. Bound to
 * both "RUBOUT" and "C-H".
 */
backdel(f, n)
{
     int    s;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());        /* we are in read only mode   */

   if (n < 0)
       return (forwdel(f, -n));

   if (f != FALSE)
       {                       /* Really a kill.       */
       if ((lastflag&CFKILL) == 0)
           kdelete();
       thisflag |= CFKILL;
       }

   if ((s=backchar(f, n)) == TRUE)
       s = ldelete((long)n, f);
   if (curbp->b_mode&MDOVER && s)
       {
       s = linsert(n,' ');
       if (s)
         s = backchar(f, n);
       }
   return (s);
}


/*
** Kill to start of line
*/
killstartline(f, n)
{
   return killtext(TRUE, 0);
}


/*
 * Kill text. If called without an argument, it kills from dot to the end of
 * the line, unless it is at the end of the line, when it kills the newline.
 * If called with an argument of 0, it kills from the start of the line to dot.
 * If called with a positive argument, it kills from dot forward over that
 * number of newlines. If called with a negative argument it kills backwards
 * that number of newlines. Normally bound to "C-K".
 */
killtext(f, n)
{
         LINE   *nextp;
        long chunk;

        if (curbp->b_mode&MDVIEW)       /* don't allow this command if   */
            return(rdonly());           /* we are in read only mode   */
        if ((lastflag&CFKILL) == 0)     /* Clear kill buffer if */
            kdelete();                  /* last wasn't a kill.  */
        thisflag |= CFKILL;
        if (f == FALSE) {
                chunk = llength(curwp->w_dotp)-curwp->w_doto;
                if (chunk == 0)
                        chunk = 1;
        } else if (n == 0) {
                chunk = curwp->w_doto;
                curwp->w_doto = 0;
        } else if (n > 0) {
                chunk = llength(curwp->w_dotp)-curwp->w_doto+1;
                nextp = lforw(curwp->w_dotp);
                while (--n) {
                        if (nextp == curbp->b_linep)
                                return (FALSE);
                        chunk += llength(nextp)+1;
                        nextp = lforw(nextp);
                }
        } else {
                mlwrite("negative kill");
                return (FALSE);
        }
        return(ldelete(chunk, TRUE));
}

setmode(f, n)   /* prompt and set an editor mode */
int f, n;   /* default and argument */
{
   adjustmode(TRUE, FALSE);
}

delmode(f, n)   /* prompt and delete an editor mode */

int f, n;   /* default and argument */

{
   adjustmode(FALSE, FALSE);
}

setgmode(f, n)   /* prompt and set a global editor mode */

int f, n;   /* default and argument */

{
   adjustmode(TRUE, TRUE);
}

delgmode(f, n)   /* prompt and delete a global editor mode */

int f, n;   /* default and argument */

{
   adjustmode(FALSE, TRUE);
}

adjustmode(kind, global)   /* change the editor mode status */

int kind;     /* true = set,           false = delete */
int global;   /* true = global flag,   false = current buffer flag */
{
    char *scan;      /* scanning pointer to convert prompt */
    int i;           /* loop index */
#if   COLOR
    int uflag;       /* was modename uppercase?   */
#endif
   char prompt[50];          /* string to prompt user with */
   char cbuf[NPAT];          /* buffer to recieve mode name into */

   /* build the proper prompt string */
   if (global)
      strcpy(prompt,"Global mode to ");
   else
      strcpy(prompt,"Mode to ");

   if (kind == TRUE)
      strcat(prompt, "add: ");
   else
      strcat(prompt, "delete: ");

   /* prompt the user and get an answer */

   mlreply(prompt, cbuf, NPAT - 1);

   /* make it uppercase */

   scan = cbuf;
#if   COLOR
   uflag = (*scan >= 'A' && *scan <= 'Z');
#endif
   while (*scan != 0) {
      if (*scan >= 'a' && *scan <= 'z')
         *scan = *scan - 32;
      scan++;
   }

   /* test it first against the colors we know */
   for (i=0; i<NCOLORS; i++) {
      if (strcmp(cbuf, cname[i]) == 0) {
         /* finding the match, we set the color */
#if   COLOR
         if (uflag)
            if (global)
               gfcolor = i;
            else
               curwp->w_fcolor = i;
         else
            if (global)
               gbcolor = i;
            else
               curwp->w_bcolor = i;

         curwp->w_flag |= WFCOLR;
#endif
         clrmes(0,0);
         return(TRUE);
      }
   }

   /* test it against the modes we know */

   for (i=0; i < NUMMODES; i++) {
      if (strcmp(cbuf, modename[i]) == 0) {
         /* finding a match, we process it */
         if (kind == TRUE)
            if (global)
               gmode |= (1 << i);
            else
               curwp->w_bufp->b_mode |= (1 << i);
         else
            if (global)
               gmode &= ~(1 << i);
            else
               curwp->w_bufp->b_mode &= ~(1 << i);
         /* display new mode line */
         if (global == 0)
            upmode();
         clrmes(0,0);   /* erase the junk */
         return(TRUE);
      }
   }

   mlwrite("No such mode!");
   return(FALSE);
}


/* OVERINS - This function toggles the OVERLAY/INSERT mode switch */
/*           Intended for use with a single key to toggle it      */
overins(f, n)
{
   curwp->w_bufp->b_mode ^= MDOVER;
   upmode();
   return(TRUE);
}

#if PRIMOS
/* COMMAND-LINE MODES  -  These modes are set when MicroEMACS are started
 *                        Some of them are normal (global) modes,
 *                        some can't be changed or set at all.
 *                        To know how they are set/if they are on,
 *                        one must use the function dispclmode().
 *
 * The functions use the structure clmodes[]
 *
 * Anders Furuhed 23 and 28 December 1987
 */

/* This function sets/modifies a command-line mode. */
setclmode (f, n)
{
  extern     CLMODE clmodes[];
  char       clmode[100];
  int        loop = 0,
             foundflag = FALSE;

  /* Prompt the user for mode */
  mlreply ("Commandlinemode: ", clmode, 99);
  clrmes(0,0);
  /* Does it exist */
  strupper (clmode);

  while ( clmodes[loop].mflag != NULL && !foundflag)
  {
    if (!strcmp (clmode, clmodes[loop].truename) ||
        !strcmp (clmode, clmodes[loop].trueabbrev))
    {
      *clmodes[loop].mflag = TRUE;
      foundflag = TRUE;
    }
    else
    if (!strcmp (clmode, clmodes[loop].falsename) ||
        !strcmp (clmode, clmodes[loop].falseabbrev))
    {
      *clmodes[loop].mflag = FALSE;
      foundflag = TRUE;
    }
    loop++;
  }

  if (!foundflag)
  {
    mlwrite("[No such Command-Line Mode]");
    return (FALSE);
  }
}


/* This function displays all commmand-line modes */
dispclmode (f, n)
{
  extern CLMODE clmodes[];
  int    loop = 0;

  (*term.t_flush)();
  (*term.t_close)();

  /* The modes that aren't on, are put between parentheses */
  puts("\nON             OFF:\n");

  while (clmodes[loop].mflag != NULL)
  {
    if (*clmodes[loop].mflag == TRUE)
      printf ("%-15s(%s)\n", clmodes[loop].truename, clmodes[loop].falsename);
    else
      printf ("%-15s(%s)\n", clmodes[loop].falsename, clmodes[loop].truename);
    loop++;
  }

  puts ("[Press any key to continue]");
  (*term.t_getchar)();
  sgarbf = TRUE;
  (*term.t_open)();
}
#endif


/*   This function simply clears the message line,
      mainly for macro usage         */
clrmes(f, n)
int f, n;   /* arguments ignored */
{
   extern int fancy_name_mode;


   if (f || !fancy_name_mode)
      mlwrite("");
   else
      mlwrite("[%s]", EditorName);

   mpresf = 2; /* VERY_TRUE */

   return(TRUE);
}

/*   This function writes a string on the message line
      mainly for macro usage         */

writemsg(f, n)

int f, n;   /* arguments ignored */

{
    char *sp;   /* pointer into buf to expand %s */
    char *np;   /* ptr into nbuf */
    int status;
   char buf[NPAT];      /* buffer to recieve mode name into */
   char nbuf[NPAT*2];   /* buffer to expand string into */

   if ((status = mlreply("Message to write: ", buf, NPAT - 1)) != TRUE)
      return(status);

   /* expand all '%' to "%%" so mlwrite won't expect arguments */
   sp = buf;
   np = nbuf;
   while (*sp) {
      *np++ = *sp;
      if (*sp++ == '%')
         *np++ = '%';
   }
   *np = '\0';
   mlwrite(nbuf);
   return(TRUE);
}

/*   Close fences are matched against their partners, and if
   on screen the cursor briefly lights there      */

fmatch(ch)
char ch;   /* fence type to match against */
{
    LINE *oldlp;   /* original line pointer */
    int oldoff;   /* and offset */
    LINE *toplp;   /* top line in current window */
    int count;   /* current fence level count */
    char opench;   /* open fence */
    char c;   /* current character in scan */
    int i;

   /* first get the display update out there */
   update(FALSE);

   /* save the original cursor position */
   oldlp = curwp->w_dotp;
   oldoff = curwp->w_doto;

   /* setup proper open fence for passed close fence */
   switch (ch)
   {
      case ')':
         opench = '(';
         break;

      case '}':
         opench = '{';
         break;

      case ']':
         opench = '[';
         break;

      case '"':
         opench = '"';
         break;

      case '\'':
         opench = '\'';
         break;
   }


   /* find the top line and set up for scan */
   toplp = curwp->w_linep->l_bp;
   count = 1;
   backchar(FALSE, 2);

   /* scan back until we find it, or reach past the top of the window */
   while (count > 0 && curwp->w_dotp != toplp)
   {
      if (curwp->w_doto == llength(curwp->w_dotp)) /* do not check if at eol */
      {
         backchar(FALSE, 1);
         continue;
      }
      c = lgetc(curwp->w_dotp, curwp->w_doto);
      if (c == ch)
         ++count;
      if (c == opench)
         --count;
      backchar(FALSE, 1);
      if (curwp->w_dotp == curwp->w_bufp->b_linep->l_fp && curwp->w_doto == 0)
         break;
   }

   /* if count is zero, we have a match, display the sucker */
   /* there is a real machine dependant timing problem here we have
      yet to solve......... */
   if (count == 0) {
      forwchar(FALSE, 1);
      for (i = 0; i < term.t_pause; i++)
         update(FALSE);
   }

   /* restore the current position */
   curwp->w_dotp = oldlp;
   curwp->w_doto = oldoff;
   return(TRUE);
}

istring(f, n)   /* ask for and insert a string into the current
         buffer at the current point */

int f, n;   /* ignored arguments */

{
    char *tp;   /* pointer into string to add */
    int status;   /* status return code */
   char tstring[NPAT+1];   /* string to add */

   /* ask for string to insert */
   status = mlreplyt("String to insert<ESC>: ", tstring, NPAT, METACH);
   if (status != TRUE)
      return(status);

   if (!f)
      n = 1;

   while (n-- > 0)
   {
      /* insert it */
      tp = &tstring[0];
      while (*tp)
      {
         if (*tp == 0x0a)
            status = lnewline();
         else
            status = linsert(1, *tp);
         ++tp;
         if (status != TRUE)
            return(status);
      }
   }
   return(TRUE);
}

#if COUNTUP

/* This routine lets the user set the count-up interval
 * Command: set-countup
 *
 * Anders Furuhed 28 December 1987
 */
setcountup (f, n)
{
  char   reply[80];
  extern int count_up,
             count_up_flag;

  if (!count_up_flag)
  {
    mlwrite("[Count up disabled]");
    return (FALSE);
  }

  mlreply ("Count up interval: ", reply);
  clrmes(0,0);
  count_up = atoi (reply);
}
#endif


