/*
 * The routines in this file
 * deal with the region, that magic space
 * between "." and mark. Some functions are
 * commands. Some functions are just for
 * internal use.
 */
#include <stdio.h>
#include "estruct.h"
#include "edef.h"

/*
 * Append the region to a named file. This is VERY easy, as all the
 * work is done by other routines...
 */
appendregion(f, n)
{
   int err;

   if ((err = copyregion(f, n)) != TRUE)
      return(err);
   return(yankappend(f, n));
}


/*
 * Write the region to a named file. This too is VERY easy...
 */
writeregion(f, n)
{
   int err;

   if ((err = copyregion(f, n)) != TRUE)
      return(err);
   return(yankwrite(f, n));
}


indregion(f, n)
{
   int s;
   REGION region;
   LINE *topline, *botline;
   int sf;


   if (!f)
      n = 1;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */

   if ((s=getregion(&region)) != TRUE)
      return (s);

   topline = region.r_linep;
   botline = curwp->w_markp == region.r_linep ? curwp->w_dotp : curwp->w_markp;

   if (botline == curwp->w_dotp) /* Move cursor to top of region */
   {
      curwp->w_dotp  = curwp->w_markp;
      curwp->w_doto  = curwp->w_marko;
      curwp->w_flag |= WFMOVE;
   }

   while (topline != botline)
   {
      gotobol(0,1);
      for (s = 0; s < n; s++)
         tab(0,1);
      forwline(0,1);
      topline = lforw(topline);
   }

   mlwrite("[Region indented]");

   return TRUE;
}


unindregion(f, n)
{
   int s;
   REGION region;
   LINE *topline, *botline;
   int sf;


   if (!f)
      n = 1;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */

   if ((s=getregion(&region)) != TRUE)
      return (s);

   topline = region.r_linep;
   botline = curwp->w_markp == region.r_linep ? curwp->w_dotp : curwp->w_markp;

   if (botline == curwp->w_dotp)    /* Move cursor to top of region? */
   {
      curwp->w_dotp  = curwp->w_markp;
      curwp->w_doto  = curwp->w_marko;
      curwp->w_flag |= WFMOVE;
   }

   while (topline != botline)
   {
      gotobol(0,1);
      backtab(TRUE,-n);
      forwline(0,1);

      topline = lforw(topline);
   }

   mlwrite("[Region unindented]");
   return TRUE;
}

/*
 * Kill the region. Ask "getregion"
 * to figure out the bounds of the region.
 * Move "." to the start, and kill the characters.
 * Bound to "C-W".
 */
killregion(f, n)
{
         int    s;
        REGION          region;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */

   if ((s=getregion(&region)) != TRUE)
      return (s);

   if ((lastflag&CFKILL) == 0)             /* This is a kill type  */
     kdelete();                      /* command, so do magic */

   thisflag |= CFKILL;                     /* kill buffer stuff.   */
   curwp->w_dotp = region.r_linep;
   curwp->w_doto = region.r_offset;
   mlwrite("[Erased and copied into kill-buffer]");
   return (ldelete(region.r_size, TRUE));
}

/*
 * Copy all of the characters in the
 * region to the kill buffer. Don't move dot
 * at all. This is a bit like a kill region followed
 * by a yank. Bound to "M-W".
 */
copyregion(f, n)
{
         LINE   *linep;
         int    loffs;
         int    s;
        REGION          region;

        if ((s=getregion(&region)) != TRUE)
                return (s);
        if ((lastflag&CFKILL) == 0)             /* Kill type command.   */
                kdelete();
        thisflag |= CFKILL;
        linep = region.r_linep;                 /* Current line.        */
        loffs = region.r_offset;                /* Current offset.      */
        while (region.r_size--) {
                if (loffs == llength(linep)) {  /* End of line.         */
                        if ((s=kinsert('\n')) != TRUE)
                                return (s);
                        linep = lforw(linep);
                        loffs = 0;
                } else {                        /* Middle of line.      */
                        if ((s=kinsert(lgetc(linep, loffs))) != TRUE)
                                return (s);
                        ++loffs;
                }
        }
        mlwrite("[Copied into kill-buffer]");
        return (TRUE);
}


/*
 * Kill region and append it to the kill-buffer
 */
killregadd(f, n)
{
         int    s;
        REGION          region;

        if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
                return(rdonly());   /* we are in read only mode   */
        if ((s=getregion(&region)) != TRUE)
                return (s);
        curwp->w_dotp = region.r_linep;
        curwp->w_doto = region.r_offset;
        mlwrite("[Erased and appended to kill-buffer]");
        return (ldelete(region.r_size, TRUE));
}


/*
 * Append all of the characters in the
 * region to the kill buffer. Don't move dot
 * at all.
 */
copyregadd(f, n)
{
         LINE   *linep;
         int    loffs;
         int    s;
        REGION          region;

        if ((s=getregion(&region)) != TRUE)
                return (s);
        linep = region.r_linep;                 /* Current line.        */
        loffs = region.r_offset;                /* Current offset.      */
        while (region.r_size--) {
                if (loffs == llength(linep)) {  /* End of line.         */
                        if ((s=kinsert('\n')) != TRUE)
                                return (s);
                        linep = lforw(linep);
                        loffs = 0;
                } else {                        /* Middle of line.      */
                        if ((s=kinsert(lgetc(linep, loffs))) != TRUE)
                                return (s);
                        ++loffs;
                }
        }
        mlwrite("[Appended to kill-buffer]");
        return (TRUE);
}


/*
 * Lower case region. Zap all of the upper
 * case characters in the region to lower case. Use
 * the region code to set the limits. Scan the buffer,
 * doing the changes. Call "lchange" to ensure that
 * redisplay is done in all buffers. Bound to
 * "C-X C-L".
 */
lowerregion(f, n)
{
         LINE   *linep;
         int    loffs;
         int    c;
         int    s;
        REGION          region;

        if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
           return(rdonly());   /* we are in read only mode   */
        if ((s=getregion(&region)) != TRUE)
                return (s);
        lchange(WFHARD);
        linep = region.r_linep;
        loffs = region.r_offset;
        while (region.r_size--) {
                if (loffs == llength(linep)) {
                        linep = lforw(linep);
                        loffs = 0;
                } else {
                        c = lgetc(linep, loffs);
                        if (c>='A' && c<='Z')
                                lputc(linep, loffs, c+'a'-'A');
                        ++loffs;
                }
        }
        return (TRUE);
}

/*
 * Upper case region. Zap all of the lower
 * case characters in the region to upper case. Use
 * the region code to set the limits. Scan the buffer,
 * doing the changes. Call "lchange" to ensure that
 * redisplay is done in all buffers. Bound to
 * "C-X C-L".
 */
upperregion(f, n)
{
         LINE   *linep;
         int    loffs;
         int    c;
         int    s;
        REGION          region;

   if (curbp->b_mode&MDVIEW)   /* don't allow this command if   */
      return(rdonly());   /* we are in read only mode   */
        if ((s=getregion(&region)) != TRUE)
                return (s);
        lchange(WFHARD);
        linep = region.r_linep;
        loffs = region.r_offset;
        while (region.r_size--) {
                if (loffs == llength(linep)) {
                        linep = lforw(linep);
                        loffs = 0;
                } else {
                        c = lgetc(linep, loffs);
                        if (c>='a' && c<='z')
                                lputc(linep, loffs, c-'a'+'A');
                        ++loffs;
                }
        }
        return (TRUE);
}

/*
 * This routine figures out the
 * bounds of the region in the current window, and
 * fills in the fields of the "REGION" structure pointed
 * to by "rp". Because the dot and mark are usually very
 * close together, we scan outward from dot looking for
 * mark. This should save time. Return a standard code.
 * Callers of this routine should be prepared to get
 * an "ABORT" status; we might make this have the
 * conform thing later.
 */
getregion(rp)
 REGION *rp;
{
         LINE   *flp;
         LINE   *blp;
        long fsize;
        long bsize;

        if (curwp->w_markp == NULL) {
                mlwrite("[No mark set in this window]");
                return (FALSE);
        }
        if (curwp->w_dotp == curwp->w_markp) {
                rp->r_linep = curwp->w_dotp;
                if (curwp->w_doto < curwp->w_marko) {
                        rp->r_offset = curwp->w_doto;
                        rp->r_size = (long)(curwp->w_marko-curwp->w_doto);
                } else {
                        rp->r_offset = curwp->w_marko;
                        rp->r_size = (long)(curwp->w_doto-curwp->w_marko);
                }
                return (TRUE);
        }
        blp = curwp->w_dotp;
        bsize = (long)curwp->w_doto;
        flp = curwp->w_dotp;
        fsize = (long)(llength(flp)-curwp->w_doto+1);
        while (flp!=curbp->b_linep || lback(blp)!=curbp->b_linep) {
                if (flp != curbp->b_linep) {
                        flp = lforw(flp);
                        if (flp == curwp->w_markp) {
                                rp->r_linep = curwp->w_dotp;
                                rp->r_offset = curwp->w_doto;
                                rp->r_size = fsize+curwp->w_marko;
                                return (TRUE);
                        }
                        fsize += llength(flp)+1;
                }
                if (lback(blp) != curbp->b_linep) {
                        blp = lback(blp);
                        bsize += llength(blp)+1;
                        if (blp == curwp->w_markp) {
                                rp->r_linep = blp;
                                rp->r_offset = curwp->w_marko;
                                rp->r_size = bsize - curwp->w_marko;
                                return (TRUE);
                        }
                }
        }
        mlwrite("[Bug: lost mark]");
        return (FALSE);
}

