/***********************************************************************/
/*                                                                      */
/*   STRINGS.C - The routines in this file perform operations on        */
/*               C-type strings.                                        */
/*                                                                      */
/*    Author: Peter Eriksson, V{sterh|jdskolan Sk|vde Sweden (870406)   */
/*                                                                      */
/************************************************************************/

#include <ctype.h>

#define MIN(x,y) (x<y?x:y)


/* STRUPPER - A routine to convert an ASCII string to uppercase characters */
/*            Please note that it will change to characters in the passed  */
/*            string and also return a pointer to tha string, so that the  */
/*            routine can be used both as a function and a procedure.      */
/* Author   - Peter Eriksson, TecSoft Unlimited (02-Feb-1987)              */

char *strupper(str)
char *str;
{
int loop;
char chr;
for (loop=0;chr=str[loop];loop++)
  str[loop] = chr>='a' && chr<='z' ? chr - 32 : chr;
return(str);
}


/* STRLOWER - A routine to convert an ASCII string to lowercase characters */
/*            Please note that it will change the characters in the passed */
/*            string and also return a pointer to tha string, so that the  */
/*            routine can be used both as a function and a procedure.      */
/* Author   - Peter Eriksson, TecSoft Unlimited (02-Feb-1987)              */

char *strlower(str)
char *str;
{
int loop;
char chr;
for (loop=0;chr=str[loop];loop++)
  str[loop] = chr>='A' && chr<='Z' ? chr + 32 : chr;
return(str);
}


/* STRLEFT - A routine to get a part of a string, without clipping the */
/*           passed one.                                               */
/* Author  - Peter Eriksson, TecSoft Unlimited (02-Feb-1987)           */


char *strleft(str,len)
char *str;
int len;
{
char *rstr;
char *calloc();
int loop;

if (len<0) len = 0;
rstr=calloc(MIN(strlen(str),len)+1,sizeof (char));
for (loop=0;str[loop]&&loop<len;loop++)
  rstr[loop]=str[loop];
rstr[loop]='\0';
return(rstr);
}


/* STRRIGHT - A routine to get a string from another, without clipping the */
/*            passed character string.                                     */
/* Author   - Peter Eriksson, TecSoft Unlimited (02-Feb-1987)              */

char *strright(str,len)
char *str;
int len;
{
char *rstr,*calloc();
int loop1,loop2;

if (len<0) len=0;
rstr = calloc(MIN(strlen(str),len)+1,sizeof (char));
loop2 = strlen(str)-len;
if (loop2<0) loop2=0;
for (loop1 = 0;str[loop2]&&loop1<len;loop1++,loop2++)
  rstr[loop1]=str[loop2];
rstr[loop1]='\0';
return(rstr);
}


/* TRIMLEFT - A routine to remove all trailing blanks from the end of an   */
/*            ASCII character string. The routine can be used, both as a   */
/*            function and a procedure, due to the returning of a pointer. */
/*            [Peter Eriksson, TecSoft Unlimited (02-Feb-1987)]            */
/* Modf.    - Remove all control characters as well as the space character */
/*            [Peter Eriksson, TecSoft Unlimited (03-Feb-1987)]            */

char *trimleft(str)
char *str;
{
int loop;

for (loop=0;str[loop]<=' ' && str[loop];loop++);
if (loop)
  strcpy(str,str+loop);
return(str);
}

/* TRIMRIGHT - A routine to remove all trailing blanks from the end of an   */
/*             ASCII character string. The routine can be used, both as a   */
/*             function and a procedure, due to the returning of a pointer. */
/*             [Petter ]berg, V{sterh|jdskolan Sk|vde (1987)]               */
/* Modf.     - Remove all control characters as well as the space char.     */
/*             [Peter Eriksson, TecSoft Unlimited (03-Feb-1987)]            */

char *trimright(str)
char *str;
{
int pos;

for(pos=strlen(str); str[pos-1]<=' ' && str[pos-1]; pos--);
str[pos] = '\0';
return(str);
}


/* TRIMSIDES- A routine to remove all trailing blanks from the end and the */
/*            start of an ASCII character string. The routine can be used, */
/*            both as a function and a procedure, due to the returning of  */
/*            a pointer.                                                   */
/*            [Peter Eriksson, TecSoft Unlimited (02-Feb-1987)]            */
/* Modf.    - Remove all control characters as well as the space character */
/*            [Peter Eriksson, TecSoft Unlimited (03-Feb-1987)]            */

char *trimsides(str)
char *str;
{
int loop;

for (loop=0; str[loop]<=' ' && str[loop]; loop++);
if (loop) strcpy(str,str+loop);

for(loop=strlen(str); str[loop-1]<=' ' && str[loop-1]; loop--);
str[loop] = '\0';
return(str);
}



/* STRLOCATE - Locate a string in an array of many strings.        */
/*             The function returns the position of the string in  */
/*             the array. It returns -1 if not found.              */
/*                                                                 */
/* Author: Peter Eriksson, V{sterh|jdskolan Prime (100487)         */

strlocate(string,array)
char *string;
char *array[];
{
   int loop = 0;

   while (array[loop])
   {
      if (!strcmp(string,array[loop]))
        return(loop);
      else
        loop++;
   }
   return(-1);
}
