/*   tcap:   Unix V5, V7 and BS4.2 Termcap video driver
      for MicroEMACS
*/

#define   termdef   1         /* don't define "term" external */

#include <stdio.h>
#include "estruct.h"
#include "edef.h"


#if TERMCAP

#define MARGIN   8
#define SCRSIZ   64
#define NPAUSE   10         /* # times thru update to pause */
#define BEL      0x07
#define ESC      0x1B

extern int      ttopen();
extern int      ttgetc();
extern int      ttputc();
extern int      ttflush();
extern int      ttclose();
extern int      tcapmove();
extern int      tcapeeol();
extern int      tcapeeop();
extern int      tcapbeep();
extern int      tcaprev();
extern int      tcapopen();

extern int      tgetnum();
extern int      tputs();
extern char *   tgoto();
extern char *   tgetstr();

extern char *   getenv();

#if   COLOR
extern   int   tcapfcol();
extern   int   tcapbcol();
#endif

#define TCAPSLEN 1024

char tcapbuf[TCAPSLEN];
char PC, *CM, *CE, *CL, *SO, *SE, *IS, *KS, *KE;

TERM term = {
   NULL,      /* these two values are set dynamically at open time */
   NULL,
   MARGIN,
   SCRSIZ,
   NPAUSE,
        tcapopen,
        ttclose,
        ttgetc,
        ttputc,
        ttflush,
        tcapmove,
        tcapeeol,
        tcapeeop,
        tcapbeep,
        tcaprev
#if   COLOR
      , tcapfcol,
        tcapbcol
#endif
};


tcapopen()
{
   char *t, *p;
   char tcbuf[1024];
   char *tv_stype;
   char err_str[72];
   char termname[81];


   goto TESTIT;

GETNEW:
   printf("Enter terminal type (or none to exit): ");
   gets(termname);
   trimsides(termname);
   if (strlen(termname) == 0)
   {
      puts("MicroEMACS aborted - no terminal type defined.");
      exit(1);
   }

   tv_stype = termname;
#if PRIMOS
   gvset(".TERM", termname);
#endif

TESTIT:
   if ((tv_stype = getenv("TERM")) == NULL)
   {
      puts("The environment variable \"TERM\" is undefined.");
      goto GETNEW;
   }

   if ((tgetent(tcbuf, tv_stype)) != 1)
   {
      printf("Unknown terminal type \"%s\"!\n", tv_stype);
      goto GETNEW;
   }


   if ((term.t_nrow=(short)tgetnum("li")-1) == -1)
   {
      puts("TERMCAP database entry incomplete (lines)");
      goto GETNEW;
   }

   if ((term.t_ncol=(short)tgetnum("co")) == -1)
   {
      puts("TERMCAP database entry incomplete (columns)");
      goto GETNEW;
   }

   p = tcapbuf;
   t = tgetstr("pc", &p);
   if (t)
      PC = *t;

   CL = tgetstr("cl", &p);
   CM = tgetstr("cm", &p);
   CE = tgetstr("ce", &p);
   SE = tgetstr("se", &p);
   SO = tgetstr("so", &p);
   IS = tgetstr("is", &p);
   KS = tgetstr("ks", &p);
   KE = tgetstr("ke", &p);

   if (SO != NULL)
      revexist = TRUE;

   if(CL == NULL || CM == NULL)
   {
      puts("Incomplete termcap entry (ClearScreen or CursorMove)\n");
      goto GETNEW;
   }

   if (CE == NULL)      /* will we be able to use clear to EOL? */
      eolexist = FALSE;

   if (p >= &tcapbuf[TCAPSLEN])
   {
      puts("Terminal description too big - aborting MicroEMACS!\n");
      exit(1);
   }

   ttopen();
   if (IS)
      putpad(IS);
}

tkeypad(onoff)
int onoff;
{
   if (KS && KE)
   {
      if (onoff && KS)
         putpad(KS);
      else
         putpad(KE);
   }
}


tcapmove(row, col)
 int row, col;
{
        putpad(tgoto(CM, col, row));
}

tcapeeol()
{
        putpad(CE);
}

tcapeeop()
{
        putpad(CL);
}

tcaprev(state)      /* change reverse video status */

int state;      /* FALSE = normal video, TRUE = reverse video */

{
   static int revstate = FALSE;
   /* mustn't send SE unless SO already sent, and vice versa */

#if 0
   if (state) {
      if ((SO != NULL) && (revstate == FALSE))
         putpad(SO);
   } else
      if ((SE != NULL) && (revstate == TRUE))
         putpad(SE);

   revstate = state;
#endif
   if (state) {
      if (SO != NULL)
         putpad(SO);
   } else
      if (SE != NULL)
         putpad(SE);
}

#if   COLOR
tcapfcol()   /* no colors here, ignore this */
{
}

tcapbcol()   /* no colors here, ignore this */
{
}
#endif

tcapbeep()
{
   ttputc(BEL);
}

putpad(str)
char    *str;
{
   tputs(str, 1, ttputc);
}

putnpad(str, n)
char    *str;
{
   tputs(str, n, ttputc);
}

#else

hello()
{
}

#endif
