/*
 * The routines in this file provide support for ANSI style terminals
 * over a serial line. The serial I/O services are provided by routines in
 * "termio.c". It compiles into nothing if not an ANSI device.
 */

#define   termdef   1         /* don't define "term" external */

#include        <stdio.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"


#if     ANSI

#if   AMIGA
#define NROW 23                      /* Screen size.                 */
#define NCOL 77                      /* Edit if you want to.         */
#endif

#if MSDOS
#define NROW 25                      /* Screen size.                 */
#define NCOL 80                      /* Edit if you want to.         */
#else
#define NROW 24                      /* Screen size.                 */
#define NCOL 80                      /* Edit if you want to.         */
#endif

#define   NPAUSE   100         /* # times thru update to pause */
#define   MARGIN   8           /* size of minimim margin and   */
#define   SCRSIZ   64          /* scroll size for extended lines */
#define BEL     ZZSETP(0x07)   /* BEL character.     */
#define ESC     ZZSETP(0x1B)   /* ESC character.    */

extern  int     ttopen();      /* Forward references.          */
extern  int     ttgetc();
extern  int     ttputc();
extern  int     ttflush();
extern  int     ttclose();
extern  int     ansimove();
extern  int     ansieeol();
extern  int     ansieeop();
extern  int     ansibeep();
extern  int     ansiopen();
extern   int   ansirev();
extern   int   ansiclose();
extern   int   ansiputc();

#if   COLOR
extern   int   ansifcol();
extern   int   ansibcol();

int   cfcolor = -1;      /* current forground color */
int   cbcolor = -1;      /* current background color */
#endif

/*
 * Standard terminal interface dispatch table. Most of the fields point into
 * "termio" code.
 */
TERM    term    = {
        NROW-1,
        NCOL,
   MARGIN,
   SCRSIZ,
   NPAUSE,
        ansiopen,
        ansiclose,
        ttgetc,
        ttputc,
        ttflush,
        ansimove,
        ansieeol,
        ansieeop,
        ansibeep,
   ansirev
#if   COLOR
   , ansifcol,
   ansibcol
#endif
};

#if   COLOR
ansifcol(color)      /* set the current output color */

int color;   /* color to set */

{
   extern int color_on;

   if (!color_on)
      return (FALSE);
   if (color == cfcolor)
      return;
   ttputc(ESC);
   ttputc('[');
   ansiparm(color+30);
   ttputc('m');
   cfcolor = color;
}

ansibcol(color)      /* set the current background color */

int color;   /* color to set */

{
   extern int color_on;

   if (!color_on)
      return (FALSE);
   if (color == cbcolor)
      return;
   ttputc(ESC);
   ttputc('[');
   ansiparm(color+40);
   ttputc('m');
        cbcolor = color;
}
#endif

ansimove(row, col)
{
        ttputc(ESC);
        ttputc('[');
        ansiparm(row+1);
        ttputc(';');
        ansiparm(col+1);
        ttputc('H');
}

ansieeol()
{
        ttputc(ESC);
        ttputc('[');
        ttputc('K');
}

ansieeop()
{
#if   COLOR
   ansifcol(gfcolor);
   ansibcol(gbcolor);
#endif
        ttputc(ESC);
        ttputc('[');
        ttputc('J');
}


ansirev(state)      /* change reverse video state */
int state;   /* TRUE = reverse, FALSE = normal */
{
#if   COLOR
   int ftmp, btmp;      /* temporaries for colors */
#endif
   static int curstate = NOT_SET;

   if (state != curstate)
   {
      ttputc(ESC);
      ttputc('[');
      ttputc(state ? '7': '0');
      ttputc('m');
#if   COLOR
      if (state == FALSE)
      {
         ftmp = cfcolor;
         btmp = cbcolor;
         cfcolor = -1;
         cbcolor = -1;
         ansifcol(ftmp);
         ansibcol(btmp);
      }
#endif
   curstate = state;
   }
}

ansibeep()
{
        ttputc(BEL);
        ttflush();
}

ansiparm(n)
 int    n;
{
         int q,r;

        q = n/10;
        if (q != 0) {
      r = q/10;
      if (r != 0) {
         ttputc((r%10)+'0');
      }
      ttputc((q%10) + '0');
        }
        ttputc((n%10) + '0');
}

ansiopen()
{
#if     V7 | USG | BSD | PRIMOS
        extern char *terminal_type;
        extern int terminal_height;
        extern int terminal_width;
        char *rcp;
        static char ttp[100];

#if SHELLTERM
#if !PRIMOS
        char *getenv();

        if ((terminal_type = getenv("TERM")) == NULL) {
            puts("The shell variable 'TERM' is not defined!");
            exit(1);
        }
#else
        char *gvget();

        if ((terminal_type = gvget(".TERM")) == NULL) {
            puts("The global variable '.TERM' is not defined!");
            exit(1);
        }
#endif
#endif

termloop:
        /* Convert terminal name to uppercase only */
        /* and remove all '-' and '_' characters */

        for (rcp=terminal_type; *rcp != '\0'; rcp++)
            {
            *rcp = toupper(*rcp);
            if (*rcp == '-' || *rcp == '_')
              strcpy(rcp,rcp+1);
            }	
        if (
            (strcmp(terminal_type, "VT100")  != 0) &&   /* Normal VT100 terminal */
            (strcmp(terminal_type, "PST100") != 0) &&   /* Prime terminal */
            (strcmp(terminal_type, "V550")   != 0) &&   /* Visual 550 (80 by 33 screen) */
            (strcmp(terminal_type, "V102")   != 0) &&   /* Visual 102 (80 by 24 screen) */
            (strcmp(terminal_type, "V102W")  != 0)      /* Visual 102 (132 by 24  -"- ) */
           )
            {
                puts("The terminal type is not one of the types that i know of.");
                puts("I currently have knowledge of the following: \n");
                puts("VT100 ....... Standard VT100 terminal");
                puts("PST100 ...... Prime VT100 compatible terminal");
                puts("VA110 ....... Teco VT100 compatible terminal");
                puts("V102 ........ Visual 102 terminal,  80 characters wide screen");
                puts("V102W ....... Visual 102 terminal, 132 characters wide screen");
                puts("V550 ........ Visual 550 terminal,  33 lines high screen");
                printf("\nPlease enter your terminal type (Enter no type to exit): ");
                gets(ttp);
                if (ttp[0] == '\0')
                   exit(0);
                terminal_type = (char *)ttp;
                goto termloop;
            }
#endif
   revexist = TRUE;
   ttopen();
   ansialtkpd(TRUE);
   tputstr("\033[20l");            /* Turn AUTONL mode off */
   if (strcmp(terminal_type, "V102") == 0)
     {
     tputstr("\033[?3l");          /* Set 80 col screen */
     terminal_width  = 80;
     terminal_height = 24;
     }
   if (strcmp(terminal_type, "V102W") == 0)
     {
     tputstr("\033[?3h");          /* Set 132 col screen */
     terminal_width  = 132;
     terminal_height = 24;
     }
   if (strcmp(terminal_type, "V550") == 0)   /* Set 33 line screen */
     {
     tputstr("\033[?1h");
     terminal_width  = 80;
     terminal_height = 33;
     }
term.t_ncol = terminal_width;
term.t_nrow = terminal_height-1;
}

ansiclose()

{
   extern char *terminal_type;
#if   COLOR
   ansifcol(7);
   ansibcol(0);
#endif

   if (strcmp(terminal_type,"V102W") == NULL)  /* Reset terminal to 80 cols */
     tputstr("\033[?3l");
   ansialtkpd(FALSE);                          /* Reset the num. keyboard */

   ttclose();
}

ansialtkpd(onoff)
{
   if (onoff)           /* If argument is TRUE then set the numerical keypad */
     tputstr("\033=");   /* to send send special ESC-sequences                */
   else
     tputstr("\033>");   /* else to transmit numbers and other normal codes   */
}


static int wp_mode[WP_STTH];

ansiputc(c)
int c;
{
   int i;

   if (c > WP_STTH)
      return ttputc(c);
   else
      if (c == WP_OFF)
      {
         for (i = 0; i < WP_STTH - 1; i++)
            wp_mode[i] = 0;

         tputstr("\033[0m");
      }
      else
         setattr(c, !wp_mode[c - 1]);
}

setattr(mode, state)
int mode, state;
{
   int i;
   static int vi_attr[] =
   {
      1,    /* BOLD */
      4,    /* UNDERLINE */
      5,    /* ITALIC */
      5,    /* SUBSCRIPT */
      5,    /* SUPERSCRIPT */
      7,    /* REVERSE */
      5     /* STRIKETHROUGH */
   };

   wp_mode[mode - 1] = state;

   if (state)
   {
      tputstr("\033[");
      ttputc(vi_attr[mode-1] + '0');
      ttputc('m');
   }
   else
   {
      tputstr("\033[0");
      for (i = 0; i < WP_STTH - 1; i++)
         if (wp_mode[i])
         {
            ttputc(';');
            ttputc(vi_attr[i] + '0');
         }
      ttputc('m');
   }
}


#else
ansihello()
{
}
#endif

