/*
 * The routines in this file provide support for FINTCM style terminals
 * over a serial line. The serial I/O services are provided by routines in
 * "termio.c". It compiles into nothing if not an FINTCM device.
 */

#define   termdef   1         /* don't define "term" external */

#include        <stdio.h>
#include        <ctype.h>
#include        "estruct.h"
#include        "edef.h"

#if   FINTCM

#include <tcm.h>
#define NROW    24                      /* Screen size.                 */
#define NCOL    80                      /* Edit if you want to.         */


#define   NPAUSE   100         /* # times thru update to pause */
#define   MARGIN   8         /* size of minimim margin and   */
#define   SCRSIZ   64         /* scroll size for extended lines */

#define BEL     ZZSETP(0x07)                    /* BEL character.     */

extern  int     ttopen();               /* Forward references.          */
extern  int     ttgetc();
extern  int     ttputc();
extern  int     ttflush();
extern  int     ttclose();
extern  int     tc_setcur();
extern  int     tc_clreol();
extern  int     tcmeeop();
extern  int     tcmbeep();
extern  int     tcmopen();
extern  int     tcmrev();
extern  int     tcmclose();

#if   COLOR
extern   int   tcmfcol();
extern   int   tcmbcol();

int   cfcolor = -1;      /* current forground color */
int   cbcolor = -1;      /* current background color */
#endif

/*
 * Standard terminal interface dispatch table. Most of the fields point into
 * "termio" code.
 */
TERM    term    = {
        NROW-1,
        NCOL,
        MARGIN,
        SCRSIZ,
        NPAUSE,
        tcmopen,
        tcmclose,
        ttgetc,
        ttputc,
        ttflush,
        tc_setcur,
        tc_clreol,
        tcmeeop,
        tcmbeep,
        tcmrev
#if   COLOR
   , tcmfcol,
   tcmbcol
#endif
};

#if   COLOR
tcmfcol(color)      /* set the current output color */
int color;
{

}

tcmbcol(color)      /* set the current background color */
int color;
{
}
#endif

tcmeeop()
{
#if   COLOR
   tcmfcol(gfcolor);
   tcmbcol(gbcolor);
#endif          
        tc_clreos(0);
}

tcmrev(state)      /* change reverse video state */

int state;   /* TRUE = reverse, FALSE = normal */

{
#if   COLOR
   int ftmp, btmp;      /* temporaries for colors */
#endif
   
tc_attrib(state ? TC_ATTR_REVERSE: TC_ATTR_NORMAL);

#if   COLOR
   if (state == FALSE) {
      ftmp = cfcolor;
      btmp = cbcolor;
      cfcolor = -1;
      cbcolor = -1;
      tcmfcol(ftmp);
      tcmbcol(btmp);
   }
#endif
}

tcmbeep()
{
        ttputc(BEL);
        ttflush();
}

tcmopen()
{
        tc_init((char *) 0);
        revexist = (tc_attrib(TC_ATTR_REVERSE)==RC_OK);
        if (revexist)
           tc_attrib(TC_ATTR_NORMAL);
        ttopen();
}

tcmclose()

{
#if   COLOR
   tcmfcol(7);
   tcmbcol(0);
#endif
   ttclose();
}
#else
tcmhello()
{
}
#endif
