/*
** VS driver.
*/

#define   termdef   1         /* don't define "term" external */

#include        <stdio.h>
#include        "estruct.h"
#include   "edef.h"

#if VSDRV

#include <vs.h>

static VS *screen;


#define NROW    24                      /* Screen size.                    */
#define NCOL    80                      /* Edit if you want to.            */
#define MARGIN  8                       /* size of minimim margin and      */
#define SCRSIZ  64                      /* scroll size for extended lines  */
#define NPAUSE  30                      /* # times thru update to pause    */
#define ESC     0x1B                    /* ESC character.                  */
#define BEL     0x07                    /* ascii bell character            */

extern  int     vsdopen();               /* Forward references.          */
extern  int     vsdgetc();
extern  int     ttgetc();
extern  int     vsdflush();
extern  int     ttclose();
extern  int     vsdmove();
extern  int     vsdeeol();
extern  int     vsdeeop();
extern  int     vsdbeep();
extern  int     vsdopen();
extern  int     vsdclose();
extern  int     vsdputc();
extern   int    vsdrev();
#if   COLOR
extern   int    vsdfcol();
extern   int    vsdbcol();
#endif

/*
 * Dispatch table. All the
 * hard fields just point into the
 * terminal I/O code.
 */
TERM    term    = {
         NROW-1,
         NCOL,
         MARGIN,
         SCRSIZ,
         NPAUSE,
         &vsdopen,
         &vsdclose,
         &ttgetc,
         &vsdputc,
         &vsdflush,
         &vsdmove,
         &vsdeeol,
         &vsdeeop,
         &vsdbeep,
         &vsdrev
#if   COLOR
   ,     &vsdfcol,
         &vsdbcol
#endif
};


vsdmove(row, col)
{
   vs_goto(col,row,screen);
}

vsdeeol()
{
   vs_clreol(screen);
}

vsdeeop()
{
   vs_clreop(screen);
}

vsdrev(status)   /* set the reverse video state */
int status;   /* TRUE = reverse video, FALSE = normal video */
{
   if (status)
      vs_setattr(VSA_REVERSE, screen);
   else
      vs_setattr(VSA_NORMAL, screen);
}

#if   COLOR
vsdfcol(col)
int col;
{
}

vsdbcol(col)
int col;
{
}
#endif

vsdbeep()
{
   ttputc(BEL);
   ttflush();
}

vsdopen()
{
    ttopen();

    if (vs_init() == -1)
    {
      fprintf(stderr, "Unable to initialize VS system\n");
      exit(1);
    }

    if ((screen = vs_create(NCOL, NROW, 0)) == NULL)
    {
      fprintf(stderr, "Unable to create VS screen\n");
      exit(1);
   }

   if (vs_open(0,0,screen) == -1)
   {
      fprintf(stderr, "Unable to open VS screen\n");
      exit(1);
   }

}


vsdclose()
{
   vs_exit();
}

vsdputc(c)
int c;
{
   vs_putc(c, screen);
}

vsdflush()
{
   vs_flush(screen);
}

#endif
