/* FILEIO for PR1MOS
**
** As certain routines in the CC_LIBRARY Rev 21.0.2 doesn't work very well,
** I decided to rewrite the file handling using Primos subroutines.
** The overall speed is increased, partly due to the poor coding in FILEIO.C
**
** Anders Furuhed, June 1988
**
*/

#include <stdio.h>
#include <ctype.h>
#include "estruct.h"
/* #include "edef.h" */

#if PR1MOS     /* Don't ever try to run this on another machine! */

#include <keys>

#define SAMFILE 0
#define DAMFILE 1
#define SAMSEG  2
#define DAMSEG  3
#define UFD     4

/* Type definitions */
typedef struct
{
   short len;
   char  txt[132];
} char132var;

typedef struct
{
   short len;
   char  txt[32];
} char32var;

fortran int  srsfx$ ();
fortran void srch$$ ();
fortran void rdlin$ ();
fortran void wtlin$ ();

char        *strrchr ();


/* Global declarations */
extern int    dam_mode;    /* Write sequential or direct access files?   */
short         funit;       /* Primos file unit; general for all routines */

/* Suffixlist */
static char32var suffix_list[] =
{
   2, ".C",
   3, ".CC",
   2, ".H",
   4, ".CPL",
   5, ".COMI",
   5, ".COMO",
   4, ".FTN",
   4, ".F77",
   2, ".P",
   4, ".PAS",
   7, ".PASCAL",
   4, ".CBL",
   6, ".COBOL",
   6, ".BASIC",
   4, ".BAS",
   4, ".PMA",
   4, ".CMA",
   4, ".ASM",
   4, ".PL1",
   5, ".PL1G",
   5, ".PL/1",
   4, ".PLP",
   4, ".SPL",
   6, ".MACRO",
   4, ".RPG",
   4, ".DOC",
   4, ".TXT",
   4, ".PRO",
   6, ".PROLOG",
   4, ".LSP",
   5, ".LISP",
   4, ".INS",
   0, ""              /* Easier for check_fsuffix if list is nulled */
};

#define SUFFIXNO 32

/* Open a file for reading */
ffropen (fn)
char *fn;
{
   short      keys;
   char132var pathname;
   short      type;
   short      n_suffixes;
   char32var  basename;
   short      suffix_used;
   short      code;
   int        chrpos;


   pathname.len = strlen (fn);
   strcpy (pathname.txt, fn);

   keys         = k$read + k$getu;
   type         = 0;
   n_suffixes   = SUFFIXNO;
   basename.len = 0;
   suffix_used  = 0;
   code         = 0;
   funit        = 0;

   /* If the file exists, don't check suffixes */
   if (fexists (fn))
      n_suffixes = 0;


   /* Open file with a good suffix */
   chrpos = srsfx$ (keys, pathname, funit, type, n_suffixes,
                    (char32var []) suffix_list, basename, suffix_used, code);

   if (type != SAMFILE && type != DAMFILE)
   {
      ffclose ();
      mlwrite ("That file type isn't supported");
      puts ("Not supported");
      return FIOERR;
   }

   if (code)
      return FIOFNF;

   return FIOSUC;
}

/* Open a file for writing */
ffwopen (fn)
char *fn;
{
   short      keys;
   char132var pathname;
   short      type;
   short      n_suffixes;
   char32var  basename;
   short      suffix_used;
   short      code;
   int        chrpos;

   keys = k$writ + k$getu;
   if (dam_mode)
      keys += k$ndam;
   else
      keys += k$nsam;
   funit        = 0;
   n_suffixes   = 0;
   basename.len = 0;
   suffix_used  = 0;
   code         = 0;

   pathname.len = strlen (fn);
   strcpy (pathname.txt, fn);

   chrpos = srsfx$ (keys, pathname, funit, type, n_suffixes,
                    (char32var []) suffix_list, basename, suffix_used, code);

   printf ("Unit: %d\n", funit);
   if (type != DAMFILE && type != SAMFILE)
   {
      ffclose ();
      mlwrite ("That file type isn't supported");
      puts ("Not supported");
      return FIOERR;
   }

   if (code)
      return FIOERR;

   return FIOSUC;
}

ffaopen (fn)
char *fn;
{
   puts ("FFAOPEN not implemented");
}

ffclose ()
{
   short keys;
   short type;
   short code;
   short len;
   char  *fn;

   keys = k$clos;
   fn   = 0;
   len  = 0;
   type = 0;
   code = 0;

   srch$$ (keys,fn,len,funit,type,code);

   if (code)
   {
      mlwrite ("Error closing file");
      return FIOERR;
   }

   return FIOSUC;
}

/* Write a line to a opened file.
** All trailing blanks are removed
*/

ffputline (buf,nbuf)
char *buf;
int  nbuf;
{
   short hwords;
   short code;

   /* Byte -> Halfword */
   hwords = nbuf / 2;
   if (nbuf != hwords*2)
      hwords++;

   code = 0;
   wtlin$ (funit, (char []) buf, hwords, code);

   if (code)
   {
      mlwrite ("Write I/O error %d", code);
      return FIOERR;
   }

   return FIOSUC;
}

#if FIXSUFF
/* The routine fix_file_suffix has been obsolete as such details are
** handled directly by srsfx$.
** However, sometimes it's very good to know if a buffer matches a filename
** specified without a suffix.
** Returns NULL if not.
*/

BUFFER *check_fsuffix (fname)
char *fname;
{
   BUFFER    *bp;                   /* Buffer to loop with              */
   int       flen;                  /* Length of fname                  */
   char32var *suffp;                /* Ptr to suffix_list               */
   char      *suffn;                /* Ptr to suffix part of buffername */

   flen  = strlen (fname);
   bp    = bheadp;
   suffn = strrchr (fname, '.');

   while (bp)
   {
      /* Is this buffer a possible match? */
      if (bp->b_flag & BFINVS == 0)
         if (!strncmp (fname, bp->b_bname,flen))
         {
            /* Yes, check suffixes if there is one */
            if ((suffn = strchr (bp->b_bname, '.'))) /* If there was one */
            {
               suffp = suffix_list;
               while (suffp.len)
               {
                  if (!strcmp (suffn,suffp.txt)) /* Compare suffix with list   */
                     return bp;                  /* Match! Return this buffer! */
                  suffp++;
               }
            }
            else
               return bp;  /* Didn't have to test suffix as there wasn't one */
         }

      bp = bp->b_bufp;  /* Proceed with next buffer */
   }

   /* No match */
   return NULL;
}
#endif /* FIXSUFFIX only */

#endif /* PR1MOS only */
